@@ -1,5 +1,5 @@
 #! /bin/sh
-source krita_xgettext.sh
+source kundo2_aware_xgettext.sh
 
 $EXTRACTRC `find . -name \*.ui | grep -v '/tests/'` >> rc.cpp
 RCFILES=`find . -name \*.xmlgui                                               \
@@ -14,12 +14,21 @@ ACTIONFILES=`find . -name \*.action`
 ./action_i18n.pl --context=action $ACTIONFILES >> rc.cpp
 
 # extracti18n.pl extracts additional data from brushes, palettes etc.
-perl extracti18n.pl > i18ndata
+perl extracti18n.pl >> rc.cpp
+
+# Extract the name of configuration pages in the metadata editor plugin.
+$EXTRACTATTR --attr=MetaDataEditor,name --context='metadata editor page' \
+  plugins/extensions/metadataeditor/editors/*.xmlgui >> rc.cpp
 
 # Ignore sdk/templates which contains templates for writing future plugins.
+# Also ignore crashreporter, it has it's own catalog
 # None of the placeholder strings inside will be seen by users.
-krita_xgettext krita.pot i18ndata rc.cpp \
-                  `find . -name \*.cc -o -name \*.h  -o -name \*.cpp | grep -v '/tests/' | grep -v './sdk/templates'`
+kundo2_aware_xgettext krita.pot rc.cpp \
+                  `find . -name \*.cc -o -name \*.h  -o -name \*.cpp | \
+                  grep -v '/tests/' | grep -v './sdk/templates' | grep -v './krita/crashreporter/'`
+
+# Extract the messages in Python plugins.
+$XGETTEXT -L Python `find . -name \*.py` -j -o $podir/krita.pot
 
 # Clean up
-rm -f i18ndata rc.cpp
+rm -f rc.cpp
