@@ -64,7 +64,7 @@ void KisAutoBrushTest::testMaskGeneration()
 
     // Check creating a mask dab with a single color
     fdev = new KisFixedPaintDevice(cs);
-    a->mask(fdev, KoColor(Qt::black, cs), 1.0, 1.0, 0.0, info);
+    a->mask(fdev, KoColor(Qt::black, cs), KisDabShape(), info);
 
     result = QImage(QString(FILES_DATA_DIR) + QDir::separator() + "result_autobrush_3.png");
     image = fdev->convertToQImage(0);
@@ -80,7 +80,7 @@ void KisAutoBrushTest::testMaskGeneration()
     dev->fill(0, 0, 100, 100, red.data());
 
     fdev = new KisFixedPaintDevice(cs);
-    a->mask(fdev, dev, 1.0, 1.0, 0.0, info);
+    a->mask(fdev, dev, KisDabShape(), info);
 
     result = QImage(QString(FILES_DATA_DIR) + QDir::separator() + "result_autobrush_4.png");
     image = fdev->convertToQImage(0);
@@ -91,32 +91,32 @@ void KisAutoBrushTest::testMaskGeneration()
 
 }
 
-void KisAutoBrushTest::testSizeRotation()
+static void dabSizeHelper(KisBrushSP const& brush,
+    QString const& name, KisDabShape const& shape, int expectedWidth, int expectedHeight)
 {
-    {
-        KisCircleMaskGenerator* circle = new KisCircleMaskGenerator(10, 0.5, 1.0, 1.0, 2, false);
-        KisBrushSP a = new KisAutoBrush(circle, 0.0, 0.0);
-        QCOMPARE(a->width(), 10);
-        QCOMPARE(a->height(), 5);
-        QCOMPARE(a->maskWidth(1.0, 0.0, 0.0, 0.0, KisPaintInformation()), 10);
-        QCOMPARE(a->maskHeight(1.0, 0.0, 0.0, 0.0, KisPaintInformation()), 5);
-        QCOMPARE(a->maskWidth(2.0, 0.0, 0.0, 0.0, KisPaintInformation()), 20);
-        QCOMPARE(a->maskHeight(2.0, 0.0, 0.0, 0.0, KisPaintInformation()), 10);
-        QCOMPARE(a->maskWidth(0.5, 0.0, 0.0, 0.0, KisPaintInformation()), 5);
-        QCOMPARE(a->maskHeight(0.5, 0.0, 0.0, 0.0, KisPaintInformation()), 3);
-        QCOMPARE(a->maskWidth(1.0, M_PI, 0.0, 0.0, KisPaintInformation()), 10);
-        QCOMPARE(a->maskHeight(1.0, M_PI, 0.0, 0.0, KisPaintInformation()), 5);
-        QCOMPARE(a->maskWidth(1.0, M_PI_2, 0.0, 0.0, KisPaintInformation()), 6); // ceil-rule
-        QCOMPARE(a->maskHeight(1.0, M_PI_2, 0.0, 0.0, KisPaintInformation()), 10);
-        QCOMPARE(a->maskWidth(1.0, -M_PI_2, 0.0, 0.0, KisPaintInformation()), 6); // ceil rule
-        QCOMPARE(a->maskHeight(1.0, -M_PI_2, 0.0, 0.0, KisPaintInformation()), 11);  // ceil rule
-        QCOMPARE(a->maskWidth(1.0, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 11);
-        QCOMPARE(a->maskHeight(1.0, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 11);
-        QCOMPARE(a->maskWidth(2.0, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 22); // ceil rule
-        QCOMPARE(a->maskHeight(2.0, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 22); // ceil rule
-        QCOMPARE(a->maskWidth(0.5, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 6);  // ceil rule
-        QCOMPARE(a->maskHeight(0.5, 0.25 * M_PI, 0.0, 0.0, KisPaintInformation()), 6);  // ceil rule
-    }
+    qDebug() << name;
+    QCOMPARE(brush->maskWidth(shape, 0.0, 0.0, KisPaintInformation()), expectedWidth);
+    QCOMPARE(brush->maskHeight(shape, 0.0, 0.0, KisPaintInformation()), expectedHeight);
+}
+
+void KisAutoBrushTest::testDabSize()
+{
+    KisCircleMaskGenerator* circle = new KisCircleMaskGenerator(10, 0.5, 1.0, 1.0, 2, false);
+    KisBrushSP a = new KisAutoBrush(circle, 0.0, 0.0);
+    QCOMPARE(a->width(), 10);
+    QCOMPARE(a->height(), 5);
+
+    dabSizeHelper(a, "Identity",  KisDabShape(),                        10,  5);
+    dabSizeHelper(a, "Double",    KisDabShape(2.0, 1.0, 0.0),           20, 10);
+    dabSizeHelper(a, "Halve",     KisDabShape(0.5, 1.0, 0.0),            5,  3);
+    dabSizeHelper(a, "180 deg",   KisDabShape(1.0, 1.0, M_PI),          10,  5);
+    dabSizeHelper(a, "90 deg",    KisDabShape(1.0, 1.0, M_PI_2),         6, 10); // ceil rule
+    dabSizeHelper(a, "-90 deg",   KisDabShape(1.0, 1.0, -M_PI_2),        6, 11); // ceil rule
+    dabSizeHelper(a, "45 deg",    KisDabShape(1.0, 1.0, 0.25 * M_PI),   11, 11);
+    dabSizeHelper(a, "2x, 45d",   KisDabShape(2.0, 1.0, 0.25 * M_PI),   22, 22);
+    dabSizeHelper(a, "0.5x, 45d", KisDabShape(0.5, 1.0, 0.25 * M_PI),    6, 6);
+    dabSizeHelper(a, "0.5x, 45d", KisDabShape(0.5, 1.0, 0.25 * M_PI),    6, 6);
+    dabSizeHelper(a, "0.5y",      KisDabShape(1.0, 0.5, 0.0),           10, 5);
 }
 
 //#define SAVE_OUTPUT_IMAGES
@@ -144,7 +144,7 @@ void KisAutoBrushTest::testCopyMasking()
     KisAutoBrush brush(mask, 0, 0);
 
     KisFixedPaintDeviceSP maskDab = new KisFixedPaintDevice(cs);
-    brush.mask(maskDab, black, 1, 1, 0, KisPaintInformation());
+    brush.mask(maskDab, black, KisDabShape(), KisPaintInformation());
     maskDab->convertTo(KoColorSpaceRegistry::instance()->alpha8());
 
 #ifdef SAVE_OUTPUT_IMAGES
@@ -181,13 +181,13 @@ void KisAutoBrushTest::testClone()
     KisPaintInformation info(QPointF(100.0, 100.0), 0.5);
 
     KisFixedPaintDeviceSP fdev1 = new KisFixedPaintDevice(cs);
-    brush->mask(fdev1, KoColor(Qt::black, cs), 0.8, 0.8, 8.0, info);
+    brush->mask(fdev1, KoColor(Qt::black, cs), KisDabShape(0.8, 1.0, 8.0), info);
     QImage res1 = fdev1->convertToQImage(0);
 
     KisBrushSP clone = brush->clone();
 
     KisFixedPaintDeviceSP fdev2 = new KisFixedPaintDevice(cs);
-    clone->mask(fdev2, KoColor(Qt::black, cs), 0.8, 0.8, 8.0, info);
+    clone->mask(fdev2, KoColor(Qt::black, cs), KisDabShape(0.8, 1.0, 8.0), info);
     QImage res2 = fdev2->convertToQImage(0);
 
     QCOMPARE(res1, res2);
