@@ -27,6 +27,7 @@
 
 #include <QScopedPointer>
 
+#include <KoColor.h>
 #include <KoColorSpace.h>
 #include <KoCompositeOpRegistry.h>
 
@@ -41,6 +42,7 @@
 #include "kis_cached_paint_device.h"
 #include "kis_mask_projection_plane.h"
 
+#include "kis_raster_keyframe_channel.h"
 
 struct Q_DECL_HIDDEN KisMask::Private {
     Private(KisMask *_q)
@@ -64,6 +66,7 @@ struct Q_DECL_HIDDEN KisMask::Private {
     QScopedPointer<QPoint> deferredSelectionOffset;
 
     KisAbstractProjectionPlaneSP projectionPlane;
+    KisCachedSelection cachedSelection;
 
     void initSelectionImpl(KisSelectionSP copyFrom, KisLayerSP parentLayer, KisPaintDeviceSP copyFromDevice);
 };
@@ -85,6 +88,12 @@ KisMask::KisMask(const KisMask& rhs)
     if (rhs.m_d->selection) {
         m_d->selection = new KisSelection(*rhs.m_d->selection.data());
         m_d->selection->setParentNode(this);
+
+        KisPixelSelectionSP pixelSelection = m_d->selection->pixelSelection();
+        if (pixelSelection->framesInterface()) {
+            addKeyframeChannel(pixelSelection->keyframeChannel());
+            enableAnimation();
+        }
     }
 }
 
@@ -97,7 +106,9 @@ void KisMask::setImage(KisImageWSP image)
 {
     KisPaintDeviceSP parentPaintDevice = parent() ? parent()->original() : 0;
     KisDefaultBoundsBaseSP defaultBounds = new KisSelectionDefaultBounds(parentPaintDevice, image);
-    m_d->selection->setDefaultBounds(defaultBounds);
+    if (m_d->selection) {
+        m_d->selection->setDefaultBounds(defaultBounds);
+    }
 }
 
 bool KisMask::allowAsChild(KisNodeSP node) const
@@ -121,12 +132,11 @@ const KoCompositeOp * KisMask::compositeOp() const
      * Please think it over...
      */
 
-    KisNodeSP parentNode = parent();
-    if (!parentNode) return 0;
+    const KoColorSpace *colorSpace = this->colorSpace();
+    if (!colorSpace) return 0;
 
-    if (!parentNode->colorSpace()) return 0;
-    const KoCompositeOp* op = parentNode->colorSpace()->compositeOp(compositeOpId());
-    return op ? op : parentNode->colorSpace()->compositeOp(COMPOSITE_OVER);
+    const KoCompositeOp* op = colorSpace->compositeOp(compositeOpId());
+    return op ? op : colorSpace->compositeOp(COMPOSITE_OVER);
 }
 
 void KisMask::initSelection(KisSelectionSP copyFrom, KisLayerSP parentLayer)
@@ -168,9 +178,7 @@ void KisMask::Private::initSelectionImpl(KisSelectionSP copyFrom, KisLayerSP par
 
     } else {
         selection = new KisSelection(new KisSelectionDefaultBounds(parentPaintDevice, parentLayer->image()));
-
-        quint8 newDefaultPixel = MAX_SELECTED;
-        selection->pixelSelection()->setDefaultPixel(&newDefaultPixel);
+        selection->pixelSelection()->setDefaultPixel(KoColor(Qt::white, selection->pixelSelection()->colorSpace()));
 
         if (deferredSelectionOffset) {
             selection->setX(deferredSelectionOffset->x());
@@ -244,17 +252,51 @@ void KisMask::apply(KisPaintDeviceSP projection, const QRect &applyRect, const Q
 
         m_d->selection->updateProjection(applyRect);
 
-        if(!extent().intersects(applyRect))
-            return;
+        KisSelectionSP effectiveSelection = m_d->selection;
+        QRect effectiveExtent;
+
+        {
+            // Access temporary target under the lock held
+            KisIndirectPaintingSupport::ReadLocker l(this);
+
+            // extent of m_d->selection should also be accessed under a lock,
+            // because it might be being merged in by the temporary target atm
+            effectiveExtent = effectiveSelection->selectedRect();
+
+            if (hasTemporaryTarget()) {
+                effectiveExtent |= temporaryTarget()->extent();
+            }
+
+            if(!effectiveExtent.intersects(applyRect)) {
+                return;
+            }
+
+            if (hasTemporaryTarget()) {
+                effectiveSelection = m_d->cachedSelection.getSelection();
+                effectiveSelection->setDefaultBounds(m_d->selection->pixelSelection()->defaultBounds());
+
+                KisPainter::copyAreaOptimized(applyRect.topLeft(),
+                                              m_d->selection->pixelSelection(),
+                                              effectiveSelection->pixelSelection(), applyRect);
+
+                KisPainter gc(effectiveSelection->pixelSelection());
+                setupTemporaryPainter(&gc);
+                gc.bitBlt(applyRect.topLeft(), temporaryTarget(), applyRect);
+            }
+        }
 
         KisPaintDeviceSP cacheDevice = m_d->paintDeviceCache.getDevice(projection);
 
         QRect updatedRect = decorateRect(projection, cacheDevice, applyRect, maskPos);
 
         // masks don't have any compositioning
-        KisPainter::copyAreaOptimized(updatedRect.topLeft(), cacheDevice, projection, updatedRect, m_d->selection);
+        KisPainter::copyAreaOptimized(updatedRect.topLeft(), cacheDevice, projection, updatedRect, effectiveSelection);
         m_d->paintDeviceCache.putDevice(cacheDevice);
 
+        if (effectiveSelection != m_d->selection) {
+            m_d->cachedSelection.putSelection(effectiveSelection);
+        }
+
     } else {
         KisPaintDeviceSP cacheDevice = m_d->paintDeviceCache.getDevice(projection);
 
@@ -271,32 +313,65 @@ QRect KisMask::needRect(const QRect &rect,  PositionToFilthy pos) const
 {
     Q_UNUSED(pos);
     QRect resultRect = rect;
-    if (m_d->selection)
-        resultRect &= m_d->selection->selectedRect();
+    if (m_d->selection) {
+        QRect selectionExtent = m_d->selection->selectedRect();
+
+        // copy for thread safety!
+        KisPaintDeviceSP temporaryTarget = this->temporaryTarget();
+
+        if (temporaryTarget) {
+            selectionExtent |= temporaryTarget->extent();
+        }
+
+        resultRect &= selectionExtent;
+    }
 
     return resultRect;
 }
 
 QRect KisMask::changeRect(const QRect &rect, PositionToFilthy pos) const
 {
-    Q_UNUSED(pos);
-    QRect resultRect = rect;
-    if (m_d->selection)
-        resultRect &= m_d->selection->selectedRect();
-
-    return resultRect;
+    return KisMask::needRect(rect, pos);
 }
 
 QRect KisMask::extent() const
 {
-    return m_d->selection ? m_d->selection->selectedRect() :
-           parent() ? parent()->extent() : QRect();
+    QRect resultRect;
+
+    if (m_d->selection) {
+        resultRect = m_d->selection->selectedRect();
+
+        // copy for thread safety!
+        KisPaintDeviceSP temporaryTarget = this->temporaryTarget();
+
+        if (temporaryTarget) {
+            resultRect |= temporaryTarget->extent();
+        }
+    } else if (KisNodeSP parent = this->parent()) {
+        resultRect = parent->extent();
+    }
+
+    return resultRect;
 }
 
 QRect KisMask::exactBounds() const
 {
-    return m_d->selection ? m_d->selection->selectedExactRect() :
-           parent() ? parent()->exactBounds() : QRect();
+    QRect resultRect;
+
+    if (m_d->selection) {
+        resultRect = m_d->selection->selectedExactRect();
+
+        // copy for thread safety!
+        KisPaintDeviceSP temporaryTarget = this->temporaryTarget();
+
+        if (temporaryTarget) {
+            resultRect |= temporaryTarget->exactBounds();
+        }
+    } else if (KisNodeSP parent = this->parent()) {
+        resultRect = parent->exactBounds();
+    }
+
+    return resultRect;
 }
 
 qint32 KisMask::x() const
@@ -335,13 +410,18 @@ void KisMask::setY(qint32 y)
     }
 }
 
+QRect KisMask::nonDependentExtent() const
+{
+    return QRect();
+}
+
 QImage KisMask::createThumbnail(qint32 w, qint32 h)
 {
     KisPaintDeviceSP originalDevice =
         selection() ? selection()->projection() : 0;
 
     return originalDevice ?
-           originalDevice->createThumbnail(w, h,
+           originalDevice->createThumbnail(w, h, 1,
                                            KoColorConversionTransformation::internalRenderingIntent(),
                                            KoColorConversionTransformation::internalConversionFlags()) : QImage();
 }
@@ -359,3 +439,26 @@ void KisMask::testingInitSelection(const QRect &rect, KisLayerSP parentLayer)
     m_d->selection->setParentNode(this);
 }
 
+KisKeyframeChannel *KisMask::requestKeyframeChannel(const QString &id)
+{
+    if (id == KisKeyframeChannel::Content.id()) {
+        KisPaintDeviceSP device = paintDevice();
+        if (device) {
+            KisRasterKeyframeChannel *contentChannel = device->createKeyframeChannel(KisKeyframeChannel::Content);
+            contentChannel->setFilenameSuffix(".pixelselection");
+            return contentChannel;
+       }
+    }
+
+    return KisNode::requestKeyframeChannel(id);
+}
+
+void KisMask::baseNodeChangedCallback()
+{
+    KisNodeSP up = parent();
+    KisLayer *layer = dynamic_cast<KisLayer*>(up.data());
+    if (layer) {
+        layer->notifyChildMaskChanged(this);
+    }
+    KisNode::baseNodeChangedCallback();
+}
