@@ -28,21 +28,30 @@
 QList<KisMimeDatabase::KisMimeType> KisMimeDatabase::s_mimeDatabase;
 
 
-QString KisMimeDatabase::mimeTypeForFile(const QString &file)
+QString KisMimeDatabase::mimeTypeForFile(const QString &file, bool checkExistingFiles)
 {
     fillMimeData();
 
     QFileInfo fi(file);
-    QString suffix = fi.suffix();
+    QString suffix = fi.suffix().toLower();
     Q_FOREACH(const KisMimeDatabase::KisMimeType &mimeType, s_mimeDatabase) {
-        if (mimeType.suffixes.contains("*." + suffix)) {
+        if (mimeType.suffixes.contains(suffix)) {
             debugPlugin << "mimeTypeForFile(). KisMimeDatabase returned" << mimeType.mimeType << "for" << file;
             return mimeType.mimeType;
         }
     }
 
     QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForFile(file);
+    QMimeType mime;
+    if (checkExistingFiles && fi.size() > 0) {
+        mime = db.mimeTypeForFile(file, QMimeDatabase::MatchContent);
+        if (mime.name() != "application/octet-stream" && mime.name() != "application/zip") {
+            debugPlugin << "mimeTypeForFile(). QMimeDatabase returned" << mime.name() << "for" << file;
+            return mime.name();
+        }
+    }
+
+    mime = db.mimeTypeForFile(file);
     if (mime.name() != "application/octet-stream") {
         debugPlugin << "mimeTypeForFile(). QMimeDatabase returned" << mime.name() << "for" << file;
         return mime.name();
@@ -55,10 +64,7 @@ QString KisMimeDatabase::mimeTypeForSuffix(const QString &suffix)
     fillMimeData();
     QMimeDatabase db;
 
-    QString s = suffix;
-    if (!s.startsWith("*.")) {
-        s = "*." + s;
-    }
+    QString s = suffix.toLower();
 
     Q_FOREACH(const KisMimeDatabase::KisMimeType &mimeType, s_mimeDatabase) {
         if (mimeType.suffixes.contains(s)) {
@@ -67,12 +73,11 @@ QString KisMimeDatabase::mimeTypeForSuffix(const QString &suffix)
         }
     }
 
-    QMimeType mime = db.mimeTypeForFile(s);
-    if (mime.name() != "application/octet-stream") {
-        debugPlugin << "mimeTypeForSuffix(). QMimeDatabase returned" << mime.name() << "for" << s;
-        return mime.name();
-    }
-    return "";
+
+    // make the file look like a file so Qt would recognize it
+    s = "file." + s;
+
+    return mimeTypeForFile(s);
 }
 
 QString KisMimeDatabase::mimeTypeForData(const QByteArray ba)
@@ -118,6 +123,12 @@ QStringList KisMimeDatabase::suffixesForMimeType(const QString &mimeType)
     QMimeType mime = db.mimeTypeForName(mimeType);
     if (mime.name() != "application/octet-stream" && !mime.suffixes().isEmpty()) {
         QString preferredSuffix = mime.preferredSuffix();
+        if (mimeType == "image/x-tga") {
+            preferredSuffix = "tga";
+        }
+        if (mimeType == "image/jpeg") {
+            preferredSuffix = "jpg";
+        }
         QStringList suffixes = mime.suffixes();
         if (preferredSuffix != suffixes.first()) {
             suffixes.removeAll(preferredSuffix);
@@ -148,110 +159,128 @@ void KisMimeDatabase::fillMimeData()
 
         mimeType.mimeType = "image/x-gimp-brush";
         mimeType.description = i18nc("description of a file type", "Gimp Brush");
-        mimeType.suffixes = QStringList() << "*.gbr" << "*.vbr";
+        mimeType.suffixes = QStringList() << "gbr" << "vbr";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-gimp-brush-animated";
         mimeType.description = i18nc("description of a file type", "Gimp Image Hose Brush");
-        mimeType.suffixes = QStringList() << "*.gih";
+        mimeType.suffixes = QStringList() << "gih";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-adobe-brushlibrary";
         mimeType.description = i18nc("description of a file type", "Adobe Brush Library");
-        mimeType.suffixes = QStringList() << "*.abr";
+        mimeType.suffixes = QStringList() << "abr";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-krita-paintoppreset";
         mimeType.description = i18nc("description of a file type", "Krita Brush Preset");
-        mimeType.suffixes = QStringList() << "*.kpp";
+        mimeType.suffixes = QStringList() << "kpp";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-krita-assistant";
         mimeType.description = i18nc("description of a file type", "Krita Assistant");
-        mimeType.suffixes = QStringList() << "*.paintingassistant";
+        mimeType.suffixes = QStringList() << "paintingassistant";
+        s_mimeDatabase << mimeType;
+
+        mimeType.mimeType = "image/x-r32";
+        mimeType.description = i18nc("description of a file type", "R32 Heightmap");
+        mimeType.suffixes = QStringList() <<  "r32";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-r16";
         mimeType.description = i18nc("description of a file type", "R16 Heightmap");
-        mimeType.suffixes = QStringList() <<  "*.r16";
+        mimeType.suffixes = QStringList() <<  "r16";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-r8";
         mimeType.description = i18nc("description of a file type", "R8 Heightmap");
-        mimeType.suffixes = QStringList() << "*.r8";
+        mimeType.suffixes = QStringList() << "r8";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-spriter";
         mimeType.description = i18nc("description of a file type", "Spriter SCML");
-        mimeType.suffixes = QStringList() << "*.scml";
+        mimeType.suffixes = QStringList() << "scml";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-svm";
         mimeType.description = i18nc("description of a file type", "Starview Metafile");
-        mimeType.suffixes = QStringList() << "*.svm";
+        mimeType.suffixes = QStringList() << "svm";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/openraster";
         mimeType.description = i18nc("description of a file type", "OpenRaster Image");
-        mimeType.suffixes = QStringList() << "*.ora";
+        mimeType.suffixes = QStringList() << "ora";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-photoshop-style-library";
         mimeType.description = i18nc("description of a file type", "Photoshop Layer Style Library");
-        mimeType.suffixes = QStringList() << "*.asl";
+        mimeType.suffixes = QStringList() << "asl";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-gimp-color-palette";
-        mimeType.description = i18nc("description of a file type", "Gimp Color Palette");
-        mimeType.suffixes = QStringList() << "*.gpl" << "*.pal" << "*.act" << "*.aco" << "*.colors";
+        mimeType.description = i18nc("description of a file type", "Color Palette");
+        mimeType.suffixes = QStringList() << "gpl" << "pal" << "act" << "aco" << "colors" << "xml" << "sbz";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-opencolorio-configuration";
         mimeType.description = i18nc("description of a file type", "OpenColorIO Configuration");
-        mimeType.suffixes = QStringList() << "*.ocio";
-        s_mimeDatabase << mimeType;
-
-        mimeType.mimeType = "application/x-krita-recorded-macro";
-        mimeType.description = i18nc("description of a file type", "Krita Recorded Action");
-        mimeType.suffixes = QStringList() << "*.krarec";
+        mimeType.suffixes = QStringList() << "ocio";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-gimp-gradient";
         mimeType.description = i18nc("description of a file type", "GIMP Gradients");
-        mimeType.suffixes = QStringList() << "*.ggr";
+        mimeType.suffixes = QStringList() << "ggr";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-gimp-pattern";
         mimeType.description = i18nc("description of a file type", "GIMP Patterns");
-        mimeType.suffixes = QStringList() << "*.pat";
+        mimeType.suffixes = QStringList() << "pat";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-karbon-gradient";
         mimeType.description = i18nc("description of a file type", "Karbon Gradients");
-        mimeType.suffixes = QStringList() << "*.kgr";
+        mimeType.suffixes = QStringList() << "kgr";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-krita-bundle";
         mimeType.description = i18nc("description of a file type", "Krita Resource Bundle");
-        mimeType.suffixes = QStringList() << "*.bundle";
+        mimeType.suffixes = QStringList() << "bundle";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-krita-workspace";
         mimeType.description = i18nc("description of a file type", "Krita Workspace");
-        mimeType.suffixes = QStringList() << "*.kts";
+        mimeType.suffixes = QStringList() << "kws";
+        s_mimeDatabase << mimeType;
+
+        mimeType.mimeType = "application/x-krita-taskset";
+        mimeType.description = i18nc("description of a file type", "Krita Taskset");
+        mimeType.suffixes = QStringList() << "kts";
+        s_mimeDatabase << mimeType;
+
+        mimeType.mimeType = "application/x-krita-reference-images";
+        mimeType.description = i18nc("description of a file type", "Krita Reference Image Collection");
+        mimeType.suffixes = QStringList() << "krf";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "image/x-krita-raw";
         mimeType.description = i18nc("description of a file type", "Camera Raw Files");
-        mimeType.suffixes = QStringList() << "*.nef" << "*.cr2" << "*.sr2" << "*.crw" << "*.pef" << "*.x3f" << "*.kdc" << "*.mrw" << "*.arw" << "*.k25" << "*.dcr" << "*.orf" << "*.raw" << "*.raw" << "*.raf" << "*.srf" << "*.dng";
+        mimeType.suffixes = QStringList() << "bay" << "bmq" << "cr2" << "crw" << "cs1" << "dc2" << "dcr" << "dng" << "erf" << "fff" << "hdr" << "k25" << "kdc" << "mdc" << "mos" << "mrw" << "nef" << "orf" << "pef" << "pxn" << "raf" << "raw" << "rdc" << "sr2" << "srf" << "x3f" << "arw" << "3fr" << "cine" << "ia" << "kc2" << "mef" << "nrw" << "qtk" << "rw2" << "sti" << "rwl" << "srw";
         s_mimeDatabase << mimeType;
 
         mimeType.mimeType = "application/x-extension-exr";
         mimeType.description = i18nc("description of a file type", "OpenEXR (Extended)");
-        mimeType.suffixes = QStringList() << "*.exr";
+        mimeType.suffixes = QStringList() << "exr";
         s_mimeDatabase << mimeType;
 
+        mimeType.mimeType = "image/x-psb";
+        mimeType.description = i18nc("description of a file type", "Photoshop Image (Large)");
+        mimeType.suffixes = QStringList() << "psb";
+        s_mimeDatabase << mimeType;
 
+        mimeType.mimeType = "image/heic";
+        mimeType.description = i18nc("description of a file type", "HEIC/HEIF Image");
+        mimeType.suffixes = QStringList() << "heic" << "heif";
+        s_mimeDatabase << mimeType;
 
         debugPlugin << "Filled mimedatabase with" << s_mimeDatabase.count() << "special mimetypes";
     }
