@@ -48,9 +48,9 @@
 #include <QMessageBox>
 #include <KoResourcePaths.h>
 #include <kis_debug.h>
-#include <kis_icon.h>
 #include <kconfiggroup.h>
 #include <QUrl>
+#include <KoFileDialog.h>
 
 #include <ksharedconfig.h>
 
@@ -92,10 +92,9 @@ public:
 
 KisTemplateCreateDia::KisTemplateCreateDia(const QString &templatesResourcePath,
                                            const QString &filePath, const QPixmap &thumbnail, QWidget *parent)
-  : KoDialog(parent)
-  , d(new KisTemplateCreateDiaPrivate(templatesResourcePath, filePath, thumbnail))
+    : KoDialog(parent)
+    , d(new KisTemplateCreateDiaPrivate(templatesResourcePath, filePath, thumbnail))
 {
-
     setButtons( KoDialog::Ok|KoDialog::Cancel );
     setDefaultButton( KoDialog::Ok );
     setCaption( i18n( "Create Template" ) );
@@ -163,6 +162,7 @@ KisTemplateCreateDia::KisTemplateCreateDia(const QString &templatesResourcePath,
 
     d->m_defaultTemplate = new QCheckBox( i18n("Use the new template as default"), mainwidget );
     d->m_defaultTemplate->setChecked( true );
+    d->m_defaultTemplate->setVisible( false );
     d->m_defaultTemplate->setToolTip(i18n("Use the new template every time Krita starts"));
     rightbox->addWidget( d->m_defaultTemplate );
 
@@ -173,7 +173,7 @@ KisTemplateCreateDia::KisTemplateCreateDia(const QString &templatesResourcePath,
     connect(d->m_groups, SIGNAL(itemSelectionChanged()), this, SLOT(slotSelectionChanged()));
 
     d->m_remove->setEnabled(d->m_groups->currentItem());
-    connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
+    connect(this, SIGNAL(okClicked()), this, SLOT(slotOk()));
 }
 
 KisTemplateCreateDia::~KisTemplateCreateDia() {
@@ -194,24 +194,26 @@ void KisTemplateCreateDia::slotSelectionChanged()
 }
 
 void KisTemplateCreateDia::createTemplate(const QString &templatesResourcePath,
-                                         const char *suffix,
-                                         KisDocument *document, QWidget *parent)
+                                          const char *suffix,
+                                          KisDocument *document, QWidget *parent)
 {
-    QTemporaryFile *tempFile = new QTemporaryFile(QDir::tempPath() + QLatin1String("/krita_XXXXXX") + QLatin1String(QLatin1String(suffix)));
-    //Check that creation of temp file was successful
-    if (!tempFile->open()) {
-        delete tempFile;
-        qWarning("Creation of temporary file to store template failed.");
+    Q_UNUSED(suffix);
+    QString fileName;
+    {
+        QTemporaryFile tempFile;
+        if (!tempFile.open()) {
+            qWarning("Creation of temporary file to store template failed.");
+            return;
+        }
+        fileName = tempFile.fileName();
+    }
+
+    bool retval = document->exportDocumentSync(QUrl::fromLocalFile(fileName), KisDocument::nativeFormatMimeType());
+    if (!retval) {
+        qWarning("Could not save template");
         return;
     }
-    const QString fileName = tempFile->fileName();
-    tempFile->close(); // need to close on Windows before we can open it again to save
-    delete tempFile; // now the file has disappeared and we can create a new file with the generated name
-
-    document->saveNativeFormat(fileName);
-
     const QPixmap thumbnail = document->generatePreview(QSize(thumbnailExtent, thumbnailExtent));
-
     KisTemplateCreateDia *dia = new KisTemplateCreateDia(templatesResourcePath, fileName, thumbnail, parent);
     dia->exec();
     delete dia;
@@ -220,8 +222,7 @@ void KisTemplateCreateDia::createTemplate(const QString &templatesResourcePath,
     d.remove(fileName);
 }
 
-static void
-saveAsQuadraticPng(const QPixmap &pixmap, const QString &fileName)
+static void saveAsQuadraticPng(const QPixmap &pixmap, const QString &fileName)
 {
     QImage icon = pixmap.toImage();
     icon = icon.convertToFormat(QImage::Format_ARGB32);
@@ -234,30 +235,30 @@ void KisTemplateCreateDia::slotOk() {
 
     // get the current item, if there is one...
     QTreeWidgetItem *item = d->m_groups->currentItem();
-    if(!item)
+    if (!item)
         item = d->m_groups->topLevelItem(0);
-    if(!item) {    // safe :)
+    if (!item) {    // safe :)
         d->m_tree.writeTemplateTree();
         slotButtonClicked( KoDialog::Cancel );
         return;
     }
     // is it a group or a template? anyway - get the group :)
-    if(item->parent() != 0)
+    if (item->parent() != 0)
         item=item->parent();
-    if(!item) {    // *very* safe :P
+    if (!item) {    // *very* safe :P
         d->m_tree.writeTemplateTree();
         slotButtonClicked( KoDialog::Cancel );
         return;
     }
 
     KisTemplateGroup *group=d->m_tree.find(item->text(0));
-    if(!group) {    // even safer
+    if (!group) {    // even safer
         d->m_tree.writeTemplateTree();
         slotButtonClicked( KoDialog::Cancel );
         return;
     }
 
-    if(d->m_name->text().isEmpty()) {
+    if (d->m_name->text().isEmpty()) {
         d->m_tree.writeTemplateTree();
         slotButtonClicked( KoDialog::Cancel );
         return;
@@ -265,38 +266,30 @@ void KisTemplateCreateDia::slotOk() {
 
     // copy the tmp file and the picture the app provides
     QString dir = KoResourcePaths::saveLocation("data", d->m_tree.templatesResourcePath());
-    dir+=group->name();
-    QString templateDir=dir+"/.source/";
-    QString iconDir=dir+"/.icon/";
+    dir += group->name();
+    QString templateDir = dir+"/.source/";
+    QString iconDir = dir+"/.icon/";
 
-    QString file=KisTemplates::trimmed(d->m_name->text());
-    QString tmpIcon=".icon/"+file;
-    tmpIcon+=".png";
+    QString file = KisTemplates::trimmed(d->m_name->text());
+    QString tmpIcon = ".icon/"+file;
+    tmpIcon += ".png";
     QString icon=iconDir+file;
-    icon+=".png";
+    icon += ".png";
 
-    // try to find the extension for the template file :P
-    const int pos = d->m_filePath.lastIndexOf(QLatin1Char('.'));
-    QString ext;
-    if ( pos > -1 )
-        ext = d->m_filePath.mid(pos);
-    else
-        warnUI << "Template extension not found!";
+    QString ext = ".kra";
 
-    QUrl dest;
-    dest.setPath(templateDir+file+ext);
-    if (QFile::exists(dest.toLocalFile())) {
+    QString dest = templateDir + file + ext;
+    if (QFile::exists(dest)) {
         do {
-            file.prepend( '_' );
-            dest.setPath( templateDir + file + ext );
+            file = file.prepend( '_' );
+            dest = templateDir + file + ext;
             tmpIcon=".icon/" + file + ".png";
             icon=iconDir + file + ".png";
         }
-        while (QFile(dest.toLocalFile()).exists());
+        while (QFile(dest).exists());
     }
     bool ignore = false;
-    dbgUI <<"Trying to create template:" << d->m_name->text() <<"URL=" <<".source/"+file+ext <<" ICON=" << tmpIcon;
-    KisTemplate *t=new KisTemplate(d->m_name->text(), QString(), ".source/"+file+ext, tmpIcon, "", "", false, true);
+    KisTemplate *t = new KisTemplate(d->m_name->text(), QString(), ".source/"+ file + ext, tmpIcon, "", "", false, true);
     if (!group->add(t)) {
         KisTemplate *existingTemplate=group->find(d->m_name->text());
         if (existingTemplate && !existingTemplate->isHidden()) {
@@ -323,16 +316,18 @@ void KisTemplateCreateDia::slotOk() {
         return;
     }
 
-    QUrl orig;
-    orig.setPath(d->m_filePath);
+    QString orig;
+    orig = d->m_filePath;
     // don't overwrite the hidden template file with a new non-hidden one
-    if ( !ignore )
-    {
-        QFile::copy(d->m_filePath, dest.toLocalFile());
+    if (!ignore) {
+        if (!QFile::copy(d->m_filePath, dest)) {
+            qWarning() << "Could not copy" << d->m_filePath << "to" << dest;
+        }
+
         // save the picture as icon
-        if(d->m_default->isChecked() && !d->m_thumbnail.isNull()) {
+        if (d->m_default->isChecked() && !d->m_thumbnail.isNull()) {
             saveAsQuadraticPng(d->m_thumbnail, icon);
-        } else if(!d->m_customPixmap.isNull()) {
+        } else if (!d->m_customPixmap.isNull()) {
             saveAsQuadraticPng(d->m_customPixmap, icon);
         } else {
             warnUI << "Could not save the preview picture!";
@@ -343,14 +338,14 @@ void KisTemplateCreateDia::slotOk() {
     bool ready=false;
     QStringList tmp=group->dirs();
     for(QStringList::ConstIterator it=tmp.constBegin(); it!=tmp.constEnd() && !ready; ++it) {
-        if((*it).contains(dir)==0) {
-            orig.setPath( (*it)+".directory" );
+        if ((*it).contains(dir)==0) {
+            orig = (*it) + ".directory";
             // Check if we can read the file
-            if (QFile(orig.toLocalFile()).exists()) {
-                dest.setPath(dir + "/.directory");
+            if (QFile(orig).exists()) {
+                dest = dir + "/.directory";
                 // We copy the file with overwrite
-                if (!QFile(orig.toLocalFile()).copy(dest.toLocalFile())) {
-                    warnKrita << "Failed to copy from" << orig.toLocalFile() << "to" << dest.toLocalFile();
+                if (!QFile(orig).copy(dest)) {
+                    warnKrita << "Failed to copy from" << orig << "to" << dest;
                 }
                 ready = true;
             }
@@ -362,10 +357,10 @@ void KisTemplateCreateDia::slotOk() {
     if ( d->m_defaultTemplate->isChecked() )
     {
 
-      KConfigGroup grp( KSharedConfig::openConfig(), "TemplateChooserDialog");
-      grp.writeEntry( "LastReturnType", "Template" );
-      grp.writePathEntry( "FullTemplateName", dir + '/' + t->file() );
-      grp.writePathEntry( "AlwaysUseTemplate", dir + '/' + t->file() );
+        KConfigGroup grp( KSharedConfig::openConfig(), "TemplateChooserDialog");
+        grp.writeEntry( "LastReturnType", "Template" );
+        grp.writePathEntry( "FullTemplateName", dir + '/' + t->file() );
+        grp.writePathEntry( "AlwaysUseTemplate", dir + '/' + t->file() );
     }
 }
 
@@ -380,7 +375,7 @@ void KisTemplateCreateDia::slotCustom() {
 
     d->m_default->setChecked(false);
     d->m_custom->setChecked(true);
-    if(d->m_customFile.isEmpty())
+    if (d->m_customFile.isEmpty())
         slotSelect();
     else
         updatePixmap();
@@ -391,25 +386,30 @@ void KisTemplateCreateDia::slotSelect() {
     d->m_default->setChecked(false);
     d->m_custom->setChecked(true);
 
-    // QT5TODO
-//    QString name = KIconDialog::getIcon();
-//    if( name.isEmpty() ) {
-//        if(d->m_customFile.isEmpty()) {
-//            d->m_default->setChecked(true);
-//            d->m_custom->setChecked(false);
-//        }
-//        return;
-//    }
-
-//    const QString path = KIconLoader::global()->iconPath(name, -thumbnailExtent);
-    d->m_customFile = QString();// path;
+    KoFileDialog dlg(this, KoFileDialog::OpenFile, "TemplateImages");
+    dlg.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
+    dlg.setImageFilters();
+    dlg.setCaption(i18n("Select an image"));
+    QString fn = dlg.filename();
+    if (fn.isEmpty()) {
+        if (d->m_customFile.isEmpty()) {
+            d->m_default->setChecked(true);
+            d->m_custom->setChecked(false);
+        }
+        return;
+    }
+    QImage image(fn);
+    if (image.isNull()) {
+        QMessageBox::warning(this, i18nc("@title:window", "Krita"), i18n("%1 is not a valid image file!", fn));
+    }
+    d->m_customFile = fn;
     d->m_customPixmap = QPixmap();
     updatePixmap();
 }
 
 void KisTemplateCreateDia::slotNameChanged(const QString &name) {
 
-    if( ( name.trimmed().isEmpty() || !d->m_groups->topLevelItem(0) ) && !d->m_changed )
+    if ( ( name.trimmed().isEmpty() || !d->m_groups->topLevelItem(0) ) && !d->m_changed )
         enableButtonOk(false);
     else
         enableButtonOk(true);
@@ -438,18 +438,18 @@ void KisTemplateCreateDia::slotAddGroup() {
 void KisTemplateCreateDia::slotRemove() {
 
     QTreeWidgetItem *item = d->m_groups->currentItem();
-    if(!item)
+    if (!item)
         return;
 
     QString what;
-        QString removed;
-        if (item->parent() == 0) {
-                what =  i18n("Do you really want to remove that group?");
-                removed = i18nc("@title:window", "Remove Group");
-        } else {
-                what =  i18n("Do you really want to remove that template?");
+    QString removed;
+    if (item->parent() == 0) {
+        what =  i18n("Do you really want to remove that group?");
+        removed = i18nc("@title:window", "Remove Group");
+    } else {
+        what =  i18n("Do you really want to remove that template?");
         removed = i18nc("@title:window", "Remove Template");
-        }
+    }
 
     if (QMessageBox::warning(this,
                              removed,
@@ -459,9 +459,9 @@ void KisTemplateCreateDia::slotRemove() {
         return;
     }
 
-    if(item->parent() == 0) {
+    if (item->parent() == 0) {
         KisTemplateGroup *group=d->m_tree.find(item->text(0));
-        if(group)
+        if (group)
             group->setHidden(true);
     }
     else {
@@ -471,7 +471,7 @@ void KisTemplateCreateDia::slotRemove() {
         for(; it != groups.constEnd() && !done; ++it) {
             KisTemplate *t = (*it)->find(item->text(0));
 
-            if(t) {
+            if (t) {
                 t->setHidden(true);
                 done=true;
             }
@@ -486,36 +486,42 @@ void KisTemplateCreateDia::slotRemove() {
 
 void KisTemplateCreateDia::updatePixmap() {
 
-    if(d->m_default->isChecked() && !d->m_thumbnail.isNull())
+    if (d->m_default->isChecked() && !d->m_thumbnail.isNull()) {
         d->m_preview->setPixmap(d->m_thumbnail);
-    else if(d->m_custom->isChecked() && !d->m_customFile.isEmpty()) {
-        if(d->m_customPixmap.isNull()) {
+    }
+    else if (d->m_custom->isChecked() && !d->m_customFile.isEmpty()) {
+
+        if (d->m_customPixmap.isNull()) {
             dbgUI <<"Trying to load picture" << d->m_customFile;
             // use the code in KisTemplate to load the image... hacky, I know :)
             KisTemplate t("foo", "bar", QString(), d->m_customFile);
-            d->m_customPixmap=t.loadPicture();
+            d->m_customPixmap = t.loadPicture();
         }
-        else
+        else {
             warnUI << "Trying to load picture";
+        }
 
-        if(!d->m_customPixmap.isNull())
+        if (!d->m_customPixmap.isNull()) {
             d->m_preview->setPixmap(d->m_customPixmap);
-        else
+        }
+        else {
             d->m_preview->setText(i18n("Could not load picture."));
+        }
     }
-    else
+    else {
         d->m_preview->setText(i18n("No picture available."));
+    }
 }
 
 void KisTemplateCreateDia::fillGroupTree() {
 
     Q_FOREACH (KisTemplateGroup *group, d->m_tree.groups()) {
-        if(group->isHidden())
+        if (group->isHidden())
             continue;
         QTreeWidgetItem *groupItem=new QTreeWidgetItem(d->m_groups, QStringList() << group->name());
 
         Q_FOREACH (KisTemplate *t, group->templates()) {
-            if(t->isHidden())
+            if (t->isHidden())
                 continue;
             (void)new QTreeWidgetItem(groupItem, QStringList() << t->name());
         }
