@@ -37,7 +37,8 @@
 #include "kis_snap_line_strategy.h"
 #include "kis_change_guides_command.h"
 #include "kis_snap_config.h"
-
+#include  "kis_canvas2.h"
+#include "kis_signal_compressor.h"
 
 struct KisGuidesManager::Private
 {
@@ -47,7 +48,9 @@ struct KisGuidesManager::Private
           invalidGuide(Qt::Horizontal, -1),
           currentGuide(invalidGuide),
           cursorSwitched(false),
-          dragStartGuidePos(0) {}
+          dragStartGuidePos(0),
+          updateDocumentCompressor(40, KisSignalCompressor::FIRST_ACTIVE),
+          shouldSetModified(false) {}
 
     KisGuidesManager *q;
 
@@ -65,7 +68,7 @@ struct KisGuidesManager::Private
     void deleteGuide(const GuideHandle &h);
     const GuideHandle invalidGuide;
 
-    bool updateCursor(const QPointF &docPos);
+    bool updateCursor(const QPointF &docPos, bool forceDisableCursor = false);
 
     void initDragStart(const GuideHandle &guide,
                        const QPointF &dragStart,
@@ -91,12 +94,16 @@ struct KisGuidesManager::Private
     qreal dragStartGuidePos;
 
     KisSignalAutoConnectionsStore viewConnections;
+
+    KisSignalCompressor updateDocumentCompressor;
+    bool shouldSetModified;
 };
 
 KisGuidesManager::KisGuidesManager(QObject *parent)
     : QObject(parent),
       m_d(new Private(this))
 {
+    connect(&m_d->updateDocumentCompressor, SIGNAL(timeout()), SLOT(slotUploadConfigToDocument()));
 }
 
 KisGuidesManager::~KisGuidesManager()
@@ -115,20 +122,15 @@ void KisGuidesManager::slotDocumentRequestedConfig(const KisGuidesConfig &config
     setGuidesConfigImpl(config, false);
 }
 
-void KisGuidesManager::setGuidesConfigImpl(const KisGuidesConfig &value, bool emitModified)
+void KisGuidesManager::slotUploadConfigToDocument()
 {
-    m_d->guidesConfig = value;
-
-    if (m_d->decoration && value != m_d->decoration->guidesConfig()) {
-        m_d->decoration->setVisible(value.showGuides());
-        m_d->decoration->setGuidesConfig(value);
-    }
+    const KisGuidesConfig &value = m_d->guidesConfig;
 
     KisDocument *doc = m_d->view ? m_d->view->document() : 0;
     if (doc) {
         KisSignalsBlocker b(doc);
 
-        if (emitModified) {
+        if (m_d->shouldSetModified) {
             KUndo2Command *cmd = new KisChangeGuidesCommand(doc, value);
             doc->addCommand(cmd);
         } else {
@@ -138,6 +140,21 @@ void KisGuidesManager::setGuidesConfigImpl(const KisGuidesConfig &value, bool em
         value.saveStaticData();
     }
 
+    m_d->shouldSetModified = false;
+}
+
+void KisGuidesManager::setGuidesConfigImpl(const KisGuidesConfig &value, bool emitModified)
+{
+    m_d->guidesConfig = value;
+
+    if (m_d->decoration && value != m_d->decoration->guidesConfig()) {
+        m_d->decoration->setVisible(value.showGuides());
+        m_d->decoration->setGuidesConfig(value);
+    }
+
+    m_d->shouldSetModified |= emitModified;
+    m_d->updateDocumentCompressor.start();
+
     const bool shouldFilterEvent =
         value.showGuides() && !value.lockGuides() && value.hasGuides();
 
@@ -148,6 +165,11 @@ void KisGuidesManager::setGuidesConfigImpl(const KisGuidesConfig &value, bool em
         m_d->updateSnappingStatus(value);
     }
 
+    if (m_d->view) {
+        m_d->view->document()->setUnit(KoUnit(m_d->guidesConfig.unitType()));
+        m_d->view->viewManager()->actionManager()->actionByName("ruler_pixel_multiple2")->setChecked(value.rulersMultiple2());
+    }
+
     emit sigRequestUpdateGuidesConfig(m_d->guidesConfig);
 }
 
@@ -251,6 +273,28 @@ void KisGuidesManager::setSnapToGuides(bool value)
     setGuidesConfigImpl(m_d->guidesConfig);
 }
 
+bool KisGuidesManager::rulersMultiple2() const
+{
+    return m_d->guidesConfig.rulersMultiple2();
+}
+
+void KisGuidesManager::setRulersMultiple2(bool value)
+{
+    m_d->guidesConfig.setRulersMultiple2(value);
+    setGuidesConfigImpl(m_d->guidesConfig);
+}
+
+KoUnit::Type KisGuidesManager::unitType() const
+{
+    return m_d->guidesConfig.unitType();
+}
+
+void KisGuidesManager::setUnitType(const KoUnit::Type type)
+{
+    m_d->guidesConfig.setUnitType(type);
+    setGuidesConfigImpl(m_d->guidesConfig, false);
+}
+
 void KisGuidesManager::setup(KisActionManager *actionManager)
 {
     KisAction *action = 0;
@@ -299,6 +343,11 @@ void KisGuidesManager::setView(QPointer<KisView> view)
         snapGuide->overrideSnapStrategy(KoSnapGuide::GuideLineSnapping, 0);
         snapGuide->enableSnapStrategy(KoSnapGuide::GuideLineSnapping, false);
 
+        if (m_d->updateDocumentCompressor.isActive()) {
+            m_d->updateDocumentCompressor.stop();
+            slotUploadConfigToDocument();
+        }
+
         m_d->decoration = 0;
         m_d->viewConnections.clear();
         attachEventFilterImpl(false);
@@ -307,7 +356,7 @@ void KisGuidesManager::setView(QPointer<KisView> view)
     m_d->view = view;
 
     if (m_d->view) {
-        KisGuidesDecoration* decoration = qobject_cast<KisGuidesDecoration*>(m_d->view->canvasBase()->decoration(GUIDES_DECORATION_ID));
+        KisGuidesDecoration* decoration = qobject_cast<KisGuidesDecoration*>(m_d->view->canvasBase()->decoration(GUIDES_DECORATION_ID).data());
         if (!decoration) {
             decoration = new KisGuidesDecoration(m_d->view);
             m_d->view->canvasBase()->addDecoration(decoration);
@@ -406,12 +455,12 @@ void KisGuidesManager::Private::deleteGuide(const GuideHandle &h)
     }
 }
 
-bool KisGuidesManager::Private::updateCursor(const QPointF &docPos)
+bool KisGuidesManager::Private::updateCursor(const QPointF &docPos, bool forceDisableCursor)
 {
     KisCanvas2 *canvas = view->canvasBase();
 
     const GuideHandle guide = findGuide(docPos);
-    const bool guideValid = isGuideValid(guide);
+    const bool guideValid = isGuideValid(guide) && !forceDisableCursor;
 
     if (guideValid && !cursorSwitched) {
         oldCursor = canvas->canvasWidget()->cursor();
@@ -526,7 +575,10 @@ QPointF KisGuidesManager::Private::getDocPointFromEvent(QEvent *event)
     KisCanvas2 *canvas = view->canvasBase();
     const KisCoordinatesConverter *converter = canvas->coordinatesConverter();
 
-    if (event->type() == QEvent::MouseMove ||
+    if (event->type() == QEvent::Enter) {
+        QEnterEvent *enterEvent = static_cast<QEnterEvent*>(event);
+        result = alignToPixels(converter->widgetToDocument(enterEvent->pos()));
+    } else if (event->type() == QEvent::MouseMove ||
         event->type() == QEvent::MouseButtonPress ||
         event->type() == QEvent::MouseButtonRelease) {
 
@@ -539,6 +591,10 @@ QPointF KisGuidesManager::Private::getDocPointFromEvent(QEvent *event)
 
         QTabletEvent *tabletEvent = static_cast<QTabletEvent*>(event);
         result = alignToPixels(converter->widgetToDocument(tabletEvent->pos()));
+    } else {
+        // we shouldn't silently return QPointF(0,0), higher level code may
+        // snap to some unexpected guide
+        KIS_SAFE_ASSERT_RECOVER_NOOP(0 && "event type is not supported!");
     }
 
     return result;
@@ -573,13 +629,17 @@ bool KisGuidesManager::eventFilter(QObject *obj, QEvent *event)
     bool retval = false;
 
     switch (event->type()) {
-    case QEvent::Enter:
     case QEvent::Leave:
+        m_d->updateCursor(QPointF(), true);
+        break;
+    case QEvent::Enter:
     case QEvent::TabletMove:
     case QEvent::MouseMove: {
         const QPointF docPos = m_d->getDocPointFromEvent(event);
         const Qt::KeyboardModifiers modifiers = qApp->keyboardModifiers();
-        retval = m_d->mouseMoveHandler(docPos, modifiers);
+
+        // we should never eat Enter events, input manager may get crazy about it
+        retval = m_d->mouseMoveHandler(docPos, modifiers) && event->type() != QEvent::Enter;
 
         break;
     }
