@@ -23,10 +23,15 @@
 #define _KIS_DLG_PREFERENCES_H_
 
 #include <QWidget>
+#include <QButtonGroup>
+#include <QMap>
+#include <QString>
 
 #include <kpagedialog.h>
+#include <kis_config.h>
 
 #include "kis_global.h"
+#include <squeezedcombobox.h>
 
 #include "ui_wdggeneralsettings.h"
 #include "ui_wdgdisplaysettings.h"
@@ -67,19 +72,27 @@ public:
     CursorStyle cursorStyle();
     OutlineStyle outlineStyle();
 
+    KisConfig::SessionOnStartup sessionOnStartup() const;
+    bool saveSessionOnQuit() const;
+
     bool showRootLayer();
     int autoSaveInterval();
     void setDefault();
     int undoStackSize();
     bool showOutlineWhilePainting();
     bool hideSplashScreen();
+
     int mdiMode();
     int favoritePresets();
     bool showCanvasMessages();
     bool compressKra();
     bool toolOptionsInDocker();
+    int kineticScrollingGesture();
+    int kineticScrollingSensitivity();
+    bool kineticScrollingScrollbar();
     bool switchSelectionCtrlAlt();
     bool convertToImageColorspaceOnImport();
+
 private Q_SLOTS:
     void getBackgroundImage();
     void clearBackgroundImage();
@@ -104,6 +117,8 @@ public:
     { }
 };
 
+class KisActionsSnapshot;
+
 class ShortcutSettingsTab : public QWidget
 {
     Q_OBJECT
@@ -111,15 +126,16 @@ class ShortcutSettingsTab : public QWidget
 public:
 
     ShortcutSettingsTab(QWidget *parent = 0, const char *name = 0);
+    ~ShortcutSettingsTab() override;
 
 public:
     void setDefault();
     WdgShortcutSettings  *m_page;
-
+    QScopedPointer<KisActionsSnapshot> m_snapshot;
 
 public Q_SLOTS:
     void saveChanges();
-    void revertChanges();
+    void cancelChanges();
 };
 
 
@@ -149,7 +165,6 @@ public:
 private Q_SLOTS:
 
     void refillMonitorProfiles(const KoID & s);
-    void refillPrintProfiles(const KoID & s);
     void installProfile();
     void toggleAllowMonitorProfileSelection(bool useSystemProfile);
 
@@ -179,6 +194,9 @@ class TabletSettingsTab : public QWidget {
 public:
     TabletSettingsTab(QWidget *parent = 0, const char  *name = 0);
 
+private Q_SLOTS:
+    void slotTabletTest();
+
 public:
     void setDefault();
     WdgTabletSettings  *m_page;
@@ -211,7 +229,7 @@ class PerformanceTab : public WdgPerformanceSettings
 public:
     PerformanceTab(QWidget *parent = 0, const char *name = 0);
 
-    ~PerformanceTab();
+    ~PerformanceTab() override;
 
     void load(bool requestDefault);
     void save();
@@ -220,11 +238,16 @@ private Q_SLOTS:
 
     void selectSwapDir();
 
+    void slotThreadsLimitChanged(int value);
+    void slotFrameClonesLimitChanged(int value);
+
 private:
     int realTilesRAM();
 
 private:
     QVector<SliderAndSpinBoxSync*> m_syncs;
+    int m_lastUsedThreadsLimit;
+    int m_lastUsedClonesLimit;
 };
 
 //=======================
@@ -243,7 +266,6 @@ public:
  *  Display settings tab for preferences dialog
  */
 
-class KoColorPopupAction;
 
 class DisplaySettingsTab : public WdgDisplaySettings
 {
@@ -258,7 +280,6 @@ protected Q_SLOTS:
     void slotUseOpenGLToggled(bool isChecked);
 
 public:
-    KoColorPopupAction *m_selectionOverlayColorAction;
 };
 
 //=======================
@@ -305,7 +326,9 @@ public:
 protected:
 
     KisDlgPreferences(QWidget *parent = 0, const char *name = 0);
-    ~KisDlgPreferences();
+    ~KisDlgPreferences() override;
+
+    void showEvent(QShowEvent *event) override;
 
 protected:
 
@@ -319,6 +342,8 @@ protected:
     KisInputConfigurationPage *m_inputConfiguration;
     KoConfigAuthorPage *m_authorPage;
 
+    QList<KPageWidgetItem*> m_pages;
+
 protected Q_SLOTS:
 
     void slotDefault();
