@@ -55,28 +55,37 @@ public:
         CurrentPattern,
         CurrentGradient,
         CurrentDisplayProfile,
-        CurrentImage,
         CurrentKritaNode,
         CurrentPaintOpPreset,
         CurrentGeneratorConfiguration,
         CurrentCompositeOp,
         CurrentEffectiveCompositeOp,
-        LodAvailability,
+        LodAvailability, ///<-user choice
+        LodSizeThreshold, ///<-user choice
+        LodSizeThresholdSupported, ///<-paintop property
+        EffectiveLodAvailablility, ///<- a superposition of user choice, threshold and paintop traits
         EraserMode,
         MirrorHorizontal,
         MirrorVertical,
-        MirrorAxesCenter,
+        MirrorHorizontalLock,
+        MirrorVerticalLock,
+        MirrorVerticalHideDecorations,
+        MirrorHorizontalHideDecorations,
         Opacity,
+        Flow,
+        Size,
         HdrGamma,
         GlobalAlphaLock,
+        DisablePressure,
         PreviousPaintOpPreset,
         EffectiveZoom, ///<-Used only by painting tools for non-displaying purposes
-        PresetAllowsLod
+        SelectionAction,
+        SelectionMode
     };
 
 
     KisCanvasResourceProvider(KisViewManager * view);
-    ~KisCanvasResourceProvider();
+    ~KisCanvasResourceProvider() override;
 
     void setResourceManager(KoCanvasResourceManager *resourceManager);
     KoCanvasResourceManager* resourceManager();
@@ -112,8 +121,6 @@ public:
     KisPaintOpPresetSP previousPreset() const;
     void setPreviousPaintOpPreset(const KisPaintOpPresetSP preset);
 
-    KisFilterConfiguration* currentGeneratorConfiguration() const;
-
     void setCurrentCompositeOp(const QString& compositeOp);
     QString currentCompositeOp() const;
 
@@ -128,12 +135,35 @@ public:
     void setMirrorVertical(bool mirrorVertical);
     bool mirrorVertical() const;
 
+    // options for horizontal and vertical mirror toolbar
+    void setMirrorHorizontalLock(bool isLocked);
+    bool mirrorHorizontalLock();
+    void setMirrorVerticalLock(bool isLocked);
+    bool mirrorVerticalLock();
+
+    void setMirrorVerticalHideDecorations(bool hide);
+    bool mirrorVerticalHideDecorations();
+
+    void setMirrorHorizontalHideDecorations(bool hide);
+    bool mirrorHorizontalHideDecorations();
+
+    void mirrorVerticalMoveCanvasToCenter();
+    void mirrorHorizontalMoveCanvasToCenter();
+
     void setOpacity(qreal opacity);
     qreal opacity() const;
 
+    void setFlow(qreal opacity);
+    qreal flow() const;
+
+    void setSize(qreal size);
+    qreal size() const;
+
     void setGlobalAlphaLock(bool lock);
     bool globalAlphaLock() const;
 
+    void setDisablePressure(bool value);
+    bool disablePressure() const;
 
     ///Notify that the workspace is saved and settings should be saved to it
     void notifySavingWorkspace(KisWorkspaceResource* workspace);
@@ -141,6 +171,12 @@ public:
     ///Notify that the workspace is loaded and settings can be read
     void notifyLoadingWorkspace(KisWorkspaceResource* workspace);
 
+    int selectionAction();
+    void setSelectionAction(int action);
+    int selectionMode();
+    void setSelectionMode(int mode);
+
+
 public Q_SLOTS:
 
     void slotSetFGColor(const KoColor& c);
@@ -148,7 +184,6 @@ public Q_SLOTS:
     void slotPatternActivated(KoResource *pattern);
     void slotGradientActivated(KoResource *gradient);
     void slotNodeActivated(const KisNodeSP node);
-    void slotGeneratorConfigurationActivated(KisFilterConfiguration * generatorConfiguration);
     void slotPainting();
 
     /**
@@ -183,8 +218,13 @@ Q_SIGNALS:
     void sigOpacityChanged(qreal);
     void sigSavingWorkspace(KisWorkspaceResource* workspace);
     void sigLoadingWorkspace(KisWorkspaceResource* workspace);
+    void sigSelectionActionChanged(const int);
+    void sigSelectionModeChanged(const int);
 
     void mirrorModeChanged();
+    void moveMirrorVerticalCenter();
+    void moveMirrorHorizontalCenter();
+
 
 private:
 
