@@ -51,27 +51,27 @@ class KRITAUI_EXPORT KisNodeManager : public QObject
 public:
 
     KisNodeManager(KisViewManager * view);
-    ~KisNodeManager();
-    
+    ~KisNodeManager() override;
+
     void setView(QPointer<KisView>imageView);
 
 Q_SIGNALS:
 
     /// emitted whenever a node is selected.
     void sigNodeActivated(KisNodeSP node);
-    
+
     /// emitted whenever a different layer is selected.
     void sigLayerActivated(KisLayerSP layer);
-    
+
     /// for the layer box: this sets the current node in the layerbox
     /// without telling the node manager that the node is activated,
     /// preventing loops (I think...)
     void sigUiNeedChangeActiveNode(KisNodeSP node);
 
     void sigUiNeedChangeSelectedNodes(const KisNodeList &nodes);
-    
+
 public:
-    
+
     void setup(KActionCollection * collection, KisActionManager* actionManager);
     void updateGUI();
 
@@ -81,7 +81,7 @@ public:
     /// convenience function to get the active layer. If a mask is
     /// active, it's parent layer is the active layer.
     KisLayerSP activeLayer();
-    
+
     /// Get the paint device the user wants to paint on now
     KisPaintDeviceSP activePaintDevice();
 
@@ -106,6 +106,8 @@ public:
     KisNodeSelectionAdapter* nodeSelectionAdapter() const;
     KisNodeInsertionAdapter* nodeInsertionAdapter() const;
 
+    static bool isNodeHidden(KisNodeSP node, bool isGlobalSelectionHidden);
+
 public Q_SLOTS:
 
     /**
@@ -153,14 +155,20 @@ public Q_SLOTS:
      */
     void copyNodesDirect(KisNodeList nodes, KisNodeSP parent, KisNodeSP aboveThis);
 
+    /**
+     * Create new layer from actually visible
+     */
+    void createFromVisible();
+
+    void slotShowHideTimeline(bool value);
 
     void toggleIsolateActiveNode();
     void toggleIsolateMode(bool checked);
     void slotUpdateIsolateModeAction();
-    void slotTryFinishIsolatedMode();
+    void slotTryRestartIsolatedMode();
 
     void moveNodeAt(KisNodeSP node, KisNodeSP parent, int index);
-    void createNode(const QString& nodeType, bool quiet = false, KisPaintDeviceSP copyFrom = 0);
+    KisNodeSP createNode(const QString& nodeType, bool quiet = false, KisPaintDeviceSP copyFrom = 0);
     void convertNode(const QString &nodeType);
     void nodesUpdated();
     void nodeProperties(KisNodeSP node);
@@ -173,6 +181,7 @@ public Q_SLOTS:
     void mirrorNode(KisNodeSP node, const KUndo2MagicString& commandName, Qt::Orientation orientation);
     void activateNextNode();
     void activatePreviousNode();
+    void switchToPreviouslyActiveNode();
 
     /**
      * move the active node up the nodestack.
@@ -190,14 +199,17 @@ public Q_SLOTS:
     void rotateRight90();
 
     void saveNodeAsImage();
-
-    // merges the active layer with the layer below it.
-    void mergeLayer();
+    void saveVectorLayerAsImage();
 
     void slotSplitAlphaIntoMask();
     void slotSplitAlphaWrite();
     void slotSplitAlphaSaveMerged();
 
+    void toggleLock();
+    void toggleVisibility();
+    void toggleAlphaLock();
+    void toggleInheritAlpha();
+
     /**
      * @brief slotSetSelectedNodes set the list of nodes selected in the layerbox. Selected nodes are not necessarily active nodes.
      * @param nodes the selected nodes
@@ -222,7 +234,7 @@ public Q_SLOTS:
 
 public:
 
-    
+
     void shear(double angleX, double angleY);
 
     void scale(double sx, double sy, KisFilterStrategy *filterStrategy);
