@@ -28,28 +28,6 @@
 #include <kritaui_export.h>
 #include <kis_types.h>
 
-
-#define PRESS_CONDITION(_event, _mode, _button, _modifier)              \
-    (this->mode() == (_mode) && (_event)->button() == (_button) &&            \
-     (_event)->modifiers() == (_modifier))
-
-#define PRESS_CONDITION_WB(_event, _mode, _button, _modifier)            \
-    (this->mode() == (_mode) && (_event)->button() & (_button) &&            \
-     (_event)->modifiers() == (_modifier))
-
-#define PRESS_CONDITION_OM(_event, _mode, _button, _modifier)           \
-    (this->mode() == (_mode) && (_event)->button() == (_button) &&      \
-     ((_event)->modifiers() & (_modifier) ||                            \
-      (_event)->modifiers() == Qt::NoModifier))
-
-#define RELEASE_CONDITION(_event, _mode, _button)               \
-    (this->mode() == (_mode) && (_event)->button() == (_button))
-
-#define RELEASE_CONDITION_WB(_event, _mode, _button)               \
-    (this->mode() == (_mode) && (_event)->button() & (_button))
-
-#define MOVE_CONDITION(_event, _mode) (this->mode() == (_mode))
-
 #ifdef __GNUC__
 #define WARN_WRONG_MODE(_mode) warnKrita << "Unexpected tool event has come to" << __func__ << "while being mode" << _mode << "!"
 #else
@@ -65,15 +43,13 @@ class KisFilterConfiguration;
 class QPainter;
 class QPainterPath;
 class QPolygonF;
-class KisRecordedPaintAction;
 
 /// Definitions of the toolgroups of Krita
 static const QString TOOL_TYPE_SHAPE = "0 Krita/Shape";         // Geometric shapes like ellipses and lines
-static const QString TOOL_TYPE_FREEHAND = "1 Krita/Freehand";   // Freehand drawing tools
 static const QString TOOL_TYPE_TRANSFORM = "2 Krita/Transform"; // Tools that transform the layer;
 static const QString TOOL_TYPE_FILL = "3 Krita/Fill";                // Tools that fill parts of the canvas
 static const QString TOOL_TYPE_VIEW = "4 Krita/View";                // Tools that affect the canvas: pan, zoom, etc.
-static const QString TOOL_TYPE_SELECTED = "5 Krita/Select";          // Tools that select pixels
+static const QString TOOL_TYPE_SELECTION = "5 Krita/Select";          // Tools that select pixels
 
 //activation id for Krita tools, Krita tools are always active and handle locked and invisible layers by themself
 static const QString KRITA_TOOL_ACTIVATION_ID = "flake/always";
@@ -86,14 +62,14 @@ class  KRITAUI_EXPORT KisTool
     Q_PROPERTY(bool isActive READ isActive NOTIFY isActiveChanged)
 
 public:
-    enum { FLAG_USES_CUSTOM_PRESET=0x01, FLAG_USES_CUSTOM_COMPOSITEOP=0x02 };
+    enum { FLAG_USES_CUSTOM_PRESET=0x01, FLAG_USES_CUSTOM_COMPOSITEOP=0x02, FLAG_USES_CUSTOM_SIZE=0x04 };
 
     KisTool(KoCanvasBase * canvas, const QCursor & cursor);
-    virtual ~KisTool();
+    ~KisTool() override;
 
     virtual int flags() const { return 0; }
 
-    void deleteSelection();
+    void deleteSelection() override;
 // KoToolBase Implementation.
 
 public:
@@ -170,12 +146,14 @@ public:
         Alternate_NONE = 10000
     };
 
+    // Technically users are allowed to configure this, but nobody ever would do that.
+    // So these can basically be thought of as aliases to ctrl+click, etc.
     enum AlternateAction {
-        ChangeSize = AlternateChangeSize,
-        PickFgNode = AlternatePickFgNode,
-        PickBgNode = AlternatePickBgNode,
-        PickFgImage = AlternatePickFgImage,
-        PickBgImage = AlternatePickBgImage,
+        ChangeSize = AlternateChangeSize, // Default: Shift+Left click
+        PickFgNode = AlternatePickFgNode, // Default: Ctrl+Alt+Left click
+        PickBgNode = AlternatePickBgNode, // Default: Ctrl+Alt+Right click
+        PickFgImage = AlternatePickFgImage, // Default: Ctrl+Left click
+        PickBgImage = AlternatePickBgImage, // Default: Ctrl+Right click
         Secondary = AlternateSecondary,
         Third = AlternateThird,
         Fourth = AlternateFourth,
@@ -194,18 +172,18 @@ public:
     virtual void beginAlternateDoubleClickAction(KoPointerEvent *event, AlternateAction action);
 
 
-    void mousePressEvent(KoPointerEvent *event);
-    void mouseDoubleClickEvent(KoPointerEvent *event);
-    void mouseTripleClickEvent(KoPointerEvent *event);
-    void mouseReleaseEvent(KoPointerEvent *event);
-    void mouseMoveEvent(KoPointerEvent *event);
+    void mousePressEvent(KoPointerEvent *event) override;
+    void mouseDoubleClickEvent(KoPointerEvent *event) override;
+    void mouseTripleClickEvent(KoPointerEvent *event) override;
+    void mouseReleaseEvent(KoPointerEvent *event) override;
+    void mouseMoveEvent(KoPointerEvent *event) override;
 
     bool isActive() const;
 
 public Q_SLOTS:
-    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
-    virtual void deactivate();
-    virtual void canvasResourceChanged(int key, const QVariant & res);
+    void activate(ToolActivation activation, const QSet<KoShape*> &shapes) override;
+    void deactivate() override;
+    void canvasResourceChanged(int key, const QVariant & res) override;
     // Implement this slot in case there are any widgets or properties which need
     // to be updated after certain operations, to reflect the inner state correctly.
     // At the moment this is used for smoothing options in the freehand brush, but
@@ -232,12 +210,13 @@ protected:
     QPointF convertDocumentToWidget(const QPointF& pt);
 
     /// Convert from native (postscript points) to integer image pixel
-    /// coordinates. This truncates the floating point components and
+    /// coordinates. This rounds down (not truncate) the pixel coordinates and
     /// should be used in preference to QPointF::toPoint(), which rounds,
     /// to ensure the cursor acts on the pixel it is visually over.
-    QPoint convertToIntPixelCoord(KoPointerEvent *e);
+    QPoint convertToImagePixelCoordFloored(KoPointerEvent *e);
 
     QRectF convertToPt(const QRectF &rect);
+    qreal convertToPt(qreal value);
 
     QPointF viewToPixel(const QPointF &viewCoord) const;
     /// Convert an integer pixel coordinate into a view coordinate.
@@ -262,7 +241,7 @@ protected:
     /// Update the canvas for the given rectangle in view coordinates.
     void updateCanvasViewRect(const QRectF &viewRect);
 
-    virtual QWidget* createOptionWidget();
+    QWidget* createOptionWidget() override;
 
     /**
      * To determine whether this tool will change its behavior when
@@ -285,22 +264,17 @@ protected:
     KisImageWSP currentImage();
     KoPattern* currentPattern();
     KoAbstractGradient *currentGradient();
-    KisNodeSP currentNode();
+    KisNodeSP currentNode() const;
     KisNodeList selectedNodes() const;
     KoColor currentFgColor();
     KoColor currentBgColor();
     KisPaintOpPresetSP currentPaintOpPreset();
-    KisFilterConfiguration *currentGenerator();
-
-    virtual void setupPaintAction(KisRecordedPaintAction* action);
+    KisFilterConfigurationSP currentGenerator();
 
     /// paint the path which is in view coordinates, default paint mode is XOR_MODE, BW_MODE is also possible
     /// never apply transformations to the painter, they would be useless, if drawing in OpenGL mode. The coordinates in the path should be in view coordinates.
     void paintToolOutline(QPainter * painter, const QPainterPath &path);
 
-    /// Sets the systemLocked for the current node, this will not deactivate the tool buttons
-    void setCurrentNodeLocked(bool locked);
-
     /// Checks checks if the current node is editable
     bool nodeEditable();
 
@@ -310,6 +284,9 @@ protected:
     /// Override the cursor appropriately if current node is not editable
     bool overrideCursorIfNotEditable();
 
+    bool blockUntilOperationsFinished();
+    void blockUntilOperationsFinishedForced();
+
 protected:
     enum ToolMode {
         HOVER_MODE,
@@ -323,6 +300,7 @@ protected:
 
     virtual void setMode(ToolMode mode);
     virtual ToolMode mode() const;
+    void setCursor(const QCursor &cursor);
 
 protected Q_SLOTS:
     /**
@@ -330,31 +308,6 @@ protected Q_SLOTS:
      */
     virtual void resetCursorStyle();
 
-    /**
-     * Called when the user requested undo while the stroke is
-     * active. If you tool supports undo of the part of its actions,
-     * override this method and do the needed work there.
-     *
-     * NOTE: Default implementation forwards this request to
-     *       requestStrokeCancellation() method, so that the stroke
-     *       would be cancelled.
-     */
-    virtual void requestUndoDuringStroke();
-
-    /**
-     * Called when the user requested the cancellation of the current
-     * stroke. If you tool supports cancelling, override this method
-     * and do the needed work there
-     */
-    virtual void requestStrokeCancellation();
-
-    /**
-     * Called when the image decided that the stroke should better be
-     * ended. If you tool supports long strokes, override this method
-     * and do the needed work there
-     */
-    virtual void requestStrokeEnd();
-
 private Q_SLOTS:
     void slotToggleFgBg();
     void slotResetFgBg();
