@@ -26,17 +26,15 @@
 #include "kis_types.h"
 #include "kis_tool_paint.h"
 #include "kis_smoothing_options.h"
+#include "kis_signal_compressor_with_param.h"
 
 #include "kritaui_export.h"
 
 class KoPointerEvent;
 class KoCanvasBase;
 
-
-
 class KisPaintingInformationBuilder;
 class KisToolFreehandHelper;
-class KisRecordingAdapter;
 
 
 class KRITAUI_EXPORT KisToolFreehand : public KisToolPaint
@@ -45,53 +43,52 @@ class KRITAUI_EXPORT KisToolFreehand : public KisToolPaint
     Q_OBJECT
 
 public:
-    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
-
     KisToolFreehand(KoCanvasBase * canvas, const QCursor & cursor, const KUndo2MagicString &transactionText);
-    virtual ~KisToolFreehand();
-    virtual int flags() const;
+    ~KisToolFreehand() override;
+    int flags() const override;
+    void mouseMoveEvent(KoPointerEvent *event) override;
 
 public Q_SLOTS:
-    virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
-    void deactivate();
+    void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes) override;
+    void deactivate() override;
 
 protected:
     bool tryPickByPaintOp(KoPointerEvent *event, AlternateAction action);
 
-    bool primaryActionSupportsHiResEvents() const;
-    void beginPrimaryAction(KoPointerEvent *event);
-    void continuePrimaryAction(KoPointerEvent *event);
-    void endPrimaryAction(KoPointerEvent *event);
+    bool primaryActionSupportsHiResEvents() const override;
+    void beginPrimaryAction(KoPointerEvent *event) override;
+    void continuePrimaryAction(KoPointerEvent *event) override;
+    void endPrimaryAction(KoPointerEvent *event) override;
 
-    void activateAlternateAction(AlternateAction action);
-    void deactivateAlternateAction(AlternateAction action);
+    void activateAlternateAction(AlternateAction action) override;
+    void deactivateAlternateAction(AlternateAction action) override;
 
-    void beginAlternateAction(KoPointerEvent *event, AlternateAction action);
-    void continueAlternateAction(KoPointerEvent *event, AlternateAction action);
-    void endAlternateAction(KoPointerEvent *event, AlternateAction action);
+    void beginAlternateAction(KoPointerEvent *event, AlternateAction action) override;
+    void continueAlternateAction(KoPointerEvent *event, AlternateAction action) override;
+    void endAlternateAction(KoPointerEvent *event, AlternateAction action) override;
 
-    virtual bool wantsAutoScroll() const;
+    bool wantsAutoScroll() const override;
 
 
     virtual void initStroke(KoPointerEvent *event);
     virtual void doStroke(KoPointerEvent *event);
     virtual void endStroke();
 
-    virtual QPainterPath getOutlinePath(const QPointF &documentPos,
+    QPainterPath getOutlinePath(const QPointF &documentPos,
                                         const KoPointerEvent *event,
-                                        KisPaintOpSettings::OutlineMode outlineMode);
+                                        KisPaintOpSettings::OutlineMode outlineMode) override;
 
 
     KisPaintingInformationBuilder* paintingInformationBuilder() const;
-    KisRecordingAdapter* recordingAdapter() const;
     void resetHelper(KisToolFreehandHelper *helper);
 
 protected Q_SLOTS:
 
     void explicitUpdateOutline();
-    virtual void resetCursorStyle();
+    void resetCursorStyle() override;
     void setAssistant(bool assistant);
     void setOnlyOneAssistantSnap(bool assistant);
+    void slotDoResizeBrush(qreal newSize);
 
 private:
     friend class KisToolFreehandPaintingInformationBuilder;
@@ -111,7 +108,6 @@ private:
 protected:
     friend class KisViewManager;
     friend class KisView;
-    friend class KisSketchView;
     KisSmoothingOptionsSP smoothingOptions() const;
     bool m_assistant;
     double m_magnetism;
@@ -120,14 +116,16 @@ protected:
 private:
     KisPaintingInformationBuilder *m_infoBuilder;
     KisToolFreehandHelper *m_helper;
-    KisRecordingAdapter *m_recordingAdapter;
 
     QPointF m_initialGestureDocPoint;
     QPointF m_lastDocumentPoint;
+    qreal m_lastPaintOpSize;
     QPoint m_initialGestureGlobalPoint;
+
+    bool m_paintopBasedPickingInAction;
+    KisSignalCompressorWithParam<qreal> m_brushResizeCompressor;
 };
 
 
 
 #endif // KIS_TOOL_FREEHAND_H_
-
