@@ -38,10 +38,8 @@
 #include <kis_debug.h>
 #include <QPoint>
 
-#include <kis_debug.h>
 #include <klocalizedstring.h>
 #include <kactioncollection.h>
-#include <QAction>
 
 #include <kis_icon.h>
 #include <KoShape.h>
@@ -61,6 +59,8 @@
 #include <kis_canvas2.h>
 #include <kis_cubic_curve.h>
 #include "kis_display_color_converter.h"
+#include <KisDocument.h>
+#include <KisReferenceImagesLayer.h>
 
 #include "kis_config.h"
 #include "kis_config_notifier.h"
@@ -68,7 +68,6 @@
 #include "widgets/kis_cmb_composite.h"
 #include "widgets/kis_slider_spin_box.h"
 #include "kis_canvas_resource_provider.h"
-#include <recorder/kis_recorded_paint_action.h>
 #include "kis_tool_utils.h"
 #include <brushengine/kis_paintop.h>
 #include <brushengine/kis_paintop_preset.h>
@@ -77,7 +76,7 @@
 #include "strokes/kis_color_picker_stroke_strategy.h"
 
 
-KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
+KisToolPaint::KisToolPaint(KoCanvasBase *canvas, const QCursor &cursor)
     : KisTool(canvas, cursor),
       m_showColorPreview(false),
       m_colorPreviewShowComparePlate(false),
@@ -89,12 +88,10 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
 
     m_opacity = OPACITY_OPAQUE_U8;
 
-    updateTabletPressureSamples();
-
     m_supportOutline = false;
 
     {
-        const int maxSize = 1000;
+        int maxSize = KisConfig().readEntry("maximumBrushSize", 1000);
 
         int brushSize = 1;
         do {
@@ -106,7 +103,7 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
         m_standardBrushSizes.push_back(maxSize);
     }
 
-    KisCanvas2 * kiscanvas = dynamic_cast<KisCanvas2*>(canvas);
+    KisCanvas2 *kiscanvas = dynamic_cast<KisCanvas2*>(canvas);
     KisActionManager *actionManager = kiscanvas->viewManager()->actionManager();
 
     // XXX: Perhaps a better place for these?
@@ -125,9 +122,7 @@ KisToolPaint::KisToolPaint(KoCanvasBase * canvas, const QCursor & cursor)
     addAction("increase_brush_size", dynamic_cast<QAction *>(actionManager->actionByName("increase_brush_size")));
     addAction("decrease_brush_size", dynamic_cast<QAction *>(actionManager->actionByName("decrease_brush_size")));
 
-    if (kiscanvas && kiscanvas->viewManager()) {
-        connect(this, SIGNAL(sigPaintingFinished()), kiscanvas->viewManager()->resourceProvider(), SLOT(slotPainting()));
-    }
+    connect(this, SIGNAL(sigPaintingFinished()), kiscanvas->viewManager()->resourceProvider(), SLOT(slotPainting()));
 
     m_colorPickerDelayTimer.setSingleShot(true);
     connect(&m_colorPickerDelayTimer, SIGNAL(timeout()), this, SLOT(activatePickColorDelayed()));
@@ -162,23 +157,39 @@ void KisToolPaint::canvasResourceChanged(int key, const QVariant& v)
     }
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(resetCursorStyle()), Qt::UniqueConnection);
-    connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(updateTabletPressureSamples()), Qt::UniqueConnection);
 
 }
 
 
 void KisToolPaint::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
-    if (currentPaintOpPreset()) emit statusTextChanged(currentPaintOpPreset()->name());
+    if (currentPaintOpPreset()) {
+        QString formattedBrushName = currentPaintOpPreset()->name().replace("_", " ");
+        emit statusTextChanged(formattedBrushName);
+    }
+
     KisTool::activate(toolActivation, shapes);
-    connect(actions().value("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()), Qt::UniqueConnection);
-    connect(actions().value("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
+    if (flags() & KisTool::FLAG_USES_CUSTOM_SIZE) {
+        connect(action("increase_brush_size"), SIGNAL(triggered()), SLOT(increaseBrushSize()), Qt::UniqueConnection);
+        connect(action("decrease_brush_size"), SIGNAL(triggered()), SLOT(decreaseBrushSize()), Qt::UniqueConnection);
+    }
+
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    m_oldOpacity = provider->opacity();
+    provider->setOpacity(m_localOpacity);
 }
 
 void KisToolPaint::deactivate()
 {
-    disconnect(actions().value("increase_brush_size"), 0, this, 0);
-    disconnect(actions().value("decrease_brush_size"), 0, this, 0);
+    if (flags() & KisTool::FLAG_USES_CUSTOM_SIZE) {
+        disconnect(action("increase_brush_size"), 0, this, 0);
+        disconnect(action("decrease_brush_size"), 0, this, 0);
+    }
+
+    KisCanvasResourceProvider *provider = qobject_cast<KisCanvas2*>(canvas())->viewManager()->resourceProvider();
+    m_localOpacity = provider->opacity();
+    provider->setOpacity(m_oldOpacity);
+
     KisTool::deactivate();
 }
 
@@ -289,11 +300,15 @@ void KisToolPaint::activateAlternateAction(AlternateAction action)
 {
     switch (action) {
     case PickFgNode:
+        /* Falls through */
     case PickBgNode:
+        /* Falls through */
     case PickFgImage:
+        /* Falls through */
     case PickBgImage:
         delayedAction = action;
         m_colorPickerDelayTimer.start(100);
+        /* Falls through */
     default:
         pickColorWasOverridden();
         KisTool::activateAlternateAction(action);
@@ -324,7 +339,6 @@ void KisToolPaint::activatePickColorDelayed()
     };
 
     repaintDecorations();
-
 }
 
 bool KisToolPaint::isPickingAction(AlternateAction action) {
@@ -358,15 +372,34 @@ void KisToolPaint::addPickerJob(const PickingJob &pickingJob)
 
     KIS_ASSERT_RECOVER_RETURN(isPickingAction(pickingJob.action));
 
-    const QPoint imagePoint = image()->documentToIntPixel(pickingJob.documentPixel);
+    const QPoint imagePoint = image()->documentToImagePixelFloored(pickingJob.documentPixel);
     const bool fromCurrentNode = pickingJob.action == PickFgNode || pickingJob.action == PickBgNode;
     m_pickingResource = colorPreviewResourceId(pickingJob.action);
 
+    if (!fromCurrentNode) {
+        auto *kisCanvas = dynamic_cast<KisCanvas2*>(canvas());
+        KIS_SAFE_ASSERT_RECOVER_RETURN(kisCanvas);
+        KisSharedPtr<KisReferenceImagesLayer> referencesLayer = kisCanvas->imageView()->document()->referenceImagesLayer();
+        if (referencesLayer && kisCanvas->referenceImagesDecoration()->visible()) {
+            QColor color = referencesLayer->getPixel(imagePoint);
+            if (color.isValid() && color.alpha() != 0) {
+                slotColorPickingFinished(KoColor(color, image()->colorSpace()));
+                return;
+            }
+        }
+    }
+
     KisPaintDeviceSP device = fromCurrentNode ?
-        currentNode()->projection() : image()->projection();
+        currentNode()->colorPickSourceDevice() : image()->projection();
+
+    // Used for color picker blending.
+    KoColor currentColor = canvas()->resourceManager()->foregroundColor();
+    if( pickingJob.action == PickBgNode || pickingJob.action == PickBgImage ){
+        currentColor = canvas()->resourceManager()->backgroundColor();
+    }
 
     image()->addJob(m_pickerStrokeId,
-                    new KisColorPickerStrokeStrategy::Data(device, imagePoint));
+                    new KisColorPickerStrokeStrategy::Data(device, imagePoint, currentColor));
 }
 
 void KisToolPaint::beginAlternateAction(KoPointerEvent *event, AlternateAction action)
@@ -462,13 +495,13 @@ void KisToolPaint::mouseReleaseEvent(KoPointerEvent *event)
 
 QWidget * KisToolPaint::createOptionWidget()
 {
-    QWidget * optionWidget = new QWidget();
+    QWidget *optionWidget = new QWidget();
     optionWidget->setObjectName(toolId());
 
-    QVBoxLayout* verticalLayout = new QVBoxLayout(optionWidget);
+    QVBoxLayout *verticalLayout = new QVBoxLayout(optionWidget);
     verticalLayout->setObjectName("KisToolPaint::OptionWidget::VerticalLayout");
     verticalLayout->setContentsMargins(0,0,0,0);
-    verticalLayout->setSpacing(1);
+    verticalLayout->setSpacing(5);
 
     // See https://bugs.kde.org/show_bug.cgi?id=316896
     QWidget *specialSpacer = new QWidget(optionWidget);
@@ -479,16 +512,14 @@ QWidget * KisToolPaint::createOptionWidget()
 
     m_optionsWidgetLayout = new QGridLayout();
     m_optionsWidgetLayout->setColumnStretch(1, 1);
-
     verticalLayout->addLayout(m_optionsWidgetLayout);
     m_optionsWidgetLayout->setContentsMargins(0,0,0,0);
-    m_optionsWidgetLayout->setSpacing(1);
+    m_optionsWidgetLayout->setSpacing(5);
 
     if (!quickHelp().isEmpty()) {
-        QPushButton* push = new QPushButton(KisIconUtils::loadIcon("help-contents"), QString(), optionWidget);
+        QPushButton *push = new QPushButton(KisIconUtils::loadIcon("help-contents"), QString(), optionWidget);
         connect(push, SIGNAL(clicked()), this, SLOT(slotPopupQuickHelp()));
-
-        QHBoxLayout* hLayout = new QHBoxLayout(optionWidget);
+        QHBoxLayout *hLayout = new QHBoxLayout();
         hLayout->addWidget(push);
         hLayout->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Fixed));
         verticalLayout->addLayout(hLayout);
@@ -583,33 +614,18 @@ void KisToolPaint::slotPopupQuickHelp()
     QWhatsThis::showText(QCursor::pos(), quickHelp());
 }
 
-void KisToolPaint::updateTabletPressureSamples()
-{
-    KisConfig cfg;
-    KisCubicCurve curve;
-    curve.fromString(cfg.pressureTabletCurve());
-    m_pressureSamples = curve.floatTransfer(LEVEL_OF_PRESSURE_RESOLUTION + 1);
-}
-
-void KisToolPaint::setupPaintAction(KisRecordedPaintAction* action)
-{
-    KisTool::setupPaintAction(action);
-    action->setOpacity(m_opacity / qreal(255.0));
-    const KoCompositeOp* op = compositeOp();
-    if (op) {
-        action->setCompositeOp(op->id());
-    }
-}
-
 KisToolPaint::NodePaintAbility KisToolPaint::nodePaintAbility()
 {
     KisNodeSP node = currentNode();
-    if (!node || node->systemLocked()) {
+    if (!node) {
         return NONE;
     }
     if (node->inherits("KisShapeLayer")) {
         return VECTOR;
     }
+    if (node->inherits("KisCloneLayer")) {
+        return CLONE;
+    }
     if (node->paintDevice()) {
         return PAINT;
     }
@@ -642,24 +658,22 @@ void KisToolPaint::setOutlineEnabled(bool value)
 
 void KisToolPaint::increaseBrushSize()
 {
-    qreal paintopSize = currentPaintOpPreset()->settings()->paintOpSize().width();
+    qreal paintopSize = currentPaintOpPreset()->settings()->paintOpSize();
 
     std::vector<int>::iterator result =
         std::upper_bound(m_standardBrushSizes.begin(),
                          m_standardBrushSizes.end(),
-                         (int)paintopSize);
+                         qRound(paintopSize));
 
     int newValue = result != m_standardBrushSizes.end() ? *result : m_standardBrushSizes.back();
 
-    qreal increment = newValue - paintopSize;
-
-    currentPaintOpPreset()->settings()->changePaintOpSize(increment, 0);
+    currentPaintOpPreset()->settings()->setPaintOpSize(newValue);
     requestUpdateOutline(m_outlineDocPoint, 0);
 }
 
 void KisToolPaint::decreaseBrushSize()
 {
-    qreal paintopSize = currentPaintOpPreset()->settings()->paintOpSize().width();
+    qreal paintopSize = currentPaintOpPreset()->settings()->paintOpSize();
 
     std::vector<int>::reverse_iterator result =
         std::upper_bound(m_standardBrushSizes.rbegin(),
@@ -668,9 +682,8 @@ void KisToolPaint::decreaseBrushSize()
                          std::greater<int>());
 
     int newValue = result != m_standardBrushSizes.rend() ? *result : m_standardBrushSizes.front();
-    qreal decrement = newValue - paintopSize;
 
-    currentPaintOpPreset()->settings()->changePaintOpSize(decrement, 0);
+    currentPaintOpPreset()->settings()->setPaintOpSize(newValue);
     requestUpdateOutline(m_outlineDocPoint, 0);
 }
 
@@ -692,28 +705,29 @@ void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const Ko
 
     KisConfig cfg;
     KisPaintOpSettings::OutlineMode outlineMode;
-    outlineMode = KisPaintOpSettings::CursorNoOutline;
 
     if (isOutlineEnabled() &&
         (mode() == KisTool::GESTURE_MODE ||
          ((cfg.newOutlineStyle() == OUTLINE_FULL ||
            cfg.newOutlineStyle() == OUTLINE_CIRCLE ||
-           cfg.newOutlineStyle() == OUTLINE_TILT ||
-           cfg.newOutlineStyle() == OUTLINE_COLOR ) &&
+           cfg.newOutlineStyle() == OUTLINE_TILT) &&
           ((mode() == HOVER_MODE) ||
            (mode() == PAINT_MODE && cfg.showOutlineWhilePainting()))))) { // lisp forever!
 
-        if(cfg.newOutlineStyle() == OUTLINE_CIRCLE) {
-            outlineMode = KisPaintOpSettings::CursorIsCircleOutline;
+        outlineMode.isVisible = true;
+
+        if (cfg.newOutlineStyle() == OUTLINE_CIRCLE) {
+            outlineMode.forceCircle = true;
         } else if(cfg.newOutlineStyle() == OUTLINE_TILT) {
-            outlineMode = KisPaintOpSettings::CursorTiltOutline;
-        } else if(cfg.newOutlineStyle() == OUTLINE_COLOR) {
-            outlineMode = KisPaintOpSettings::CursorColorOutline;
+            outlineMode.forceCircle = true;
+            outlineMode.showTiltDecoration = true;
         } else {
-            outlineMode = KisPaintOpSettings::CursorIsOutline;
+            // noop
         }
     }
 
+    outlineMode.forceFullSize = cfg.forceAlwaysFullSizedOutline();
+
     m_outlineDocPoint = outlineDocPoint;
     m_currentOutline = getOutlinePath(m_outlineDocPoint, event, outlineMode);
 
@@ -721,7 +735,7 @@ void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const Ko
     QRectF outlineDocRect = currentImage()->pixelToDocument(outlinePixelRect);
 
     // This adjusted call is needed as we paint with a 3 pixel wide brush and the pen is outside the bounds of the path
-    // Pen uses view coordinates so we have to zoom the document value to match 2 pixel in view coordiates
+    // Pen uses view coordinates so we have to zoom the document value to match 2 pixel in view coordinates
     // See BUG 275829
     qreal zoomX;
     qreal zoomY;
@@ -739,20 +753,30 @@ void KisToolPaint::requestUpdateOutline(const QPointF &outlineDocPoint, const Ko
         colorPreviewDocUpdateRect.adjust(-xoffset,-yoffset,xoffset,yoffset);
     }
 
-    if (!m_oldColorPreviewUpdateRect.isEmpty()) {
-        canvas()->updateCanvas(m_oldColorPreviewUpdateRect);
-    }
+    // DIRTY HACK ALERT: we should fetch the assistant's dirty rect when requesting
+    //                   the update, instead of just dumbly update the entire canvas!
 
-    if (!m_oldOutlineRect.isEmpty()) {
-        canvas()->updateCanvas(m_oldOutlineRect);
-    }
+    KisCanvas2 * kiscanvas = dynamic_cast<KisCanvas2*>(canvas());
+    KisPaintingAssistantsDecorationSP decoration = kiscanvas->paintingAssistantsDecoration();
+    if (decoration && decoration->visible()) {
+        kiscanvas->updateCanvas();
+    } else {
+        // TODO: only this branch should be present!
+        if (!m_oldColorPreviewUpdateRect.isEmpty()) {
+            canvas()->updateCanvas(m_oldColorPreviewUpdateRect);
+        }
 
-    if (!outlineDocRect.isEmpty()) {
-        canvas()->updateCanvas(outlineDocRect);
-    }
+        if (!m_oldOutlineRect.isEmpty()) {
+            canvas()->updateCanvas(m_oldOutlineRect);
+        }
 
-    if (!colorPreviewDocUpdateRect.isEmpty()) {
-        canvas()->updateCanvas(colorPreviewDocUpdateRect);
+        if (!outlineDocRect.isEmpty()) {
+            canvas()->updateCanvas(outlineDocRect);
+        }
+
+        if (!colorPreviewDocUpdateRect.isEmpty()) {
+            canvas()->updateCanvas(colorPreviewDocUpdateRect);
+        }
     }
 
     m_oldOutlineRect = outlineDocRect;
