@@ -28,6 +28,7 @@
 #include <QApplication>
 #include <QClipboard>
 #include <QDesktopWidget>
+#include <QDialogButtonBox>
 #include <QFile>
 #include <QSpacerItem>
 
@@ -59,7 +60,7 @@
 #include "kis_clipboard.h"
 #include "KisDocument.h"
 #include "widgets/kis_cmb_idlist.h"
-#include "widgets/squeezedcombobox.h"
+#include <squeezedcombobox.h>
 
 
 KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qint32 defHeight, double resolution, const QString& defColorModel, const QString& defColorDepth, const QString& defColorProfile, const QString& imageName)
@@ -105,8 +106,18 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
             this, SLOT(heightUnitChanged(int)));
     connect(doubleHeight, SIGNAL(valueChanged(double)),
             this, SLOT(heightChanged(double)));
-    connect(createButton, SIGNAL(clicked()), this, SLOT(createImage()));
-    createButton->setDefault(true);
+
+
+    // Create image
+    newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setText(i18n("Create"));
+    connect(newDialogConfirmationButtonBox, SIGNAL(accepted()), this, SLOT(createImage()));
+
+
+    // Cancel Create image button
+    connect(newDialogConfirmationButtonBox, SIGNAL(rejected()), this->parentWidget(), SLOT(close()));
+    connect(newDialogConfirmationButtonBox, SIGNAL(rejected()), this->parentWidget(), SLOT(deleteLater()));
+
+
 
     bnPortrait->setIcon(KisIconUtils::loadIcon("portrait"));
     connect(bnPortrait, SIGNAL(clicked()), SLOT(setPortrait()));
@@ -120,16 +131,21 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     colorSpaceSelector->setCurrentColorModel(KoID(defColorModel));
     colorSpaceSelector->setCurrentColorDepth(KoID(defColorDepth));
     colorSpaceSelector->setCurrentProfile(defColorProfile);
+    connect(colorSpaceSelector, SIGNAL(colorSpaceChanged(const KoColorSpace*)), this, SLOT(changeDocumentInfoLabel()));
 
     //connect(chkFromClipboard,SIGNAL(stateChanged(int)),this,SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(dataChanged()), this, SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(selectionChanged()), this, SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(changed(QClipboard::Mode)), this, SLOT(clipboardDataChanged()));
-    connect(colorSpaceSelector, SIGNAL(selectionChanged(bool)), createButton, SLOT(setEnabled(bool)));
+
+    connect(colorSpaceSelector, SIGNAL(selectionChanged(bool)), newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok), SLOT(setEnabled(bool)));
+
 
     KisConfig cfg;
     intNumLayers->setValue(cfg.numDefaultLayers());
-    cmbColor->setColor(cfg.defaultBackgroundColor());
+    KoColor bcol(KoColorSpaceRegistry::instance()->rgb8());
+    bcol.fromQColor(cfg.defaultBackgroundColor());
+    cmbColor->setColor(bcol);
     setBackgroundOpacity(cfg.defaultBackgroundOpacity());
 
     KisConfig::BackgroundStyle bgStyle = cfg.defaultBackgroundStyle();
@@ -144,25 +160,25 @@ KisCustomImageWidget::KisCustomImageWidget(QWidget* parent, qint32 defWidth, qin
     switchPortraitLandscape();
 
     // this makes the portrait and landscape buttons more
-    // obvious what is selected by changing the higlight color
+    // obvious what is selected by changing the highlight color
     QPalette p = QApplication::palette();
     QPalette palette_highlight(p );
     QColor c = p.color(QPalette::Highlight);
     palette_highlight.setColor(QPalette::Button, c);
     bnLandscape->setPalette(palette_highlight);
     bnPortrait->setPalette(palette_highlight);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::showEvent(QShowEvent *)
 {
     fillPredefined();
-    this->createButton->setFocus();
-    this->createButton->setEnabled(true);
+    newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setFocus();
+    newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
 }
 
 KisCustomImageWidget::~KisCustomImageWidget()
 {
-    qDeleteAll(m_predefined);
     m_predefined.clear();
 }
 
@@ -177,6 +193,7 @@ void KisCustomImageWidget::resolutionChanged(double res)
         m_heightUnit.setFactor(res / 72.0);
         m_height = m_heightUnit.fromUserValue(doubleHeight->value());
     }
+    changeDocumentInfoLabel();
 }
 
 
@@ -195,11 +212,13 @@ void KisCustomImageWidget::widthUnitChanged(int index)
     doubleWidth->setValue(KoUnit::ptToUnit(m_width, m_widthUnit));
 
     doubleWidth->blockSignals(false);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::widthChanged(double value)
 {
     m_width = m_widthUnit.fromUserValue(value);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::heightUnitChanged(int index)
@@ -217,16 +236,18 @@ void KisCustomImageWidget::heightUnitChanged(int index)
     doubleHeight->setValue(KoUnit::ptToUnit(m_height, m_heightUnit));
 
     doubleHeight->blockSignals(false);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::heightChanged(double value)
 {
     m_height = m_heightUnit.fromUserValue(value);
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::createImage()
 {
-    createButton->setEnabled(false);
+    newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
     KisDocument *doc = createNewImage();
     if (doc) {
         doc->setModified(false);
@@ -255,7 +276,7 @@ KisDocument* KisCustomImageWidget::createNewImage()
                                      i18n("Linear gamma RGB color spaces are not supposed to be used "
                                           "in 8-bit integer modes. It is suggested to use 16-bit integer "
                                           "or any floating point colorspace for linear profiles.\n\n"
-                                          "Press \"Continue\" to create a 8-bit integer linear RGB color space "
+                                          "Press \"Ok\" to create a 8-bit integer linear RGB color space "
                                           "or \"Cancel\" to return to the settings dialog."),
                                      QMessageBox::Ok | QMessageBox::Cancel, QMessageBox::Cancel);
 
@@ -277,7 +298,7 @@ KisDocument* KisCustomImageWidget::createNewImage()
     width = static_cast<qint32>(0.5  + KoUnit::ptToUnit(m_width, KoUnit(KoUnit::Pixel, resolution)));
     height = static_cast<qint32>(0.5 + KoUnit::ptToUnit(m_height, KoUnit(KoUnit::Pixel, resolution)));
 
-    QColor qc = cmbColor->color();
+    QColor qc = cmbColor->color().toQColor();
     qc.setAlpha(backgroundOpacity());
     KoColor bgColor(qc, cs);
 
@@ -288,7 +309,7 @@ KisDocument* KisCustomImageWidget::createNewImage()
     KisConfig cfg;
     cfg.setNumDefaultLayers(intNumLayers->value());
     cfg.setDefaultBackgroundOpacity(backgroundOpacity());
-    cfg.setDefaultBackgroundColor(cmbColor->color());
+    cfg.setDefaultBackgroundColor(cmbColor->color().toQColor());
     cfg.setDefaultBackgroundStyle(backgroundAsLayer ? KisConfig::LAYER : KisConfig::PROJECTION);
 
     return doc;
@@ -317,12 +338,9 @@ void KisCustomImageWidget::clipboardDataChanged()
 {
 }
 
-
-
 void KisCustomImageWidget::fillPredefined()
 {
     cmbPredefined->clear();
-    qDeleteAll(m_predefined);
     m_predefined.clear();
 
     cmbPredefined->addItem("");
@@ -337,7 +355,7 @@ void KisCustomImageWidget::fillPredefined()
             f.open(QIODevice::ReadOnly);
             if (f.exists()) {
                 QString xml = QString::fromUtf8(f.readAll());
-                KisPropertiesConfiguration *predefined = new KisPropertiesConfiguration;
+                KisPropertiesConfigurationSP predefined = new KisPropertiesConfiguration;
                 predefined->fromXML(xml);
                 if (predefined->hasProperty("name")
                         && predefined->hasProperty("width")
@@ -361,7 +379,7 @@ void KisCustomImageWidget::predefinedClicked(int index)
 {
     if (index < 1 || index > m_predefined.size()) return;
 
-    KisPropertiesConfiguration *predefined = m_predefined[index - 1];
+    KisPropertiesConfigurationSP predefined = m_predefined[index - 1];
     txtPredefinedName->setText(predefined->getString("name"));
     doubleResolution->setValue(predefined->getDouble("resolution"));
     cmbWidthUnit->setCurrentIndex(predefined->getInt("x-unit"));
@@ -372,6 +390,7 @@ void KisCustomImageWidget::predefinedClicked(int index)
 
     doubleWidth->setValue(predefined->getDouble("width"));
     doubleHeight->setValue(predefined->getDouble("height"));
+    changeDocumentInfoLabel();
 
 }
 
@@ -382,10 +401,10 @@ void KisCustomImageWidget::saveAsPredefined()
         return;
     }
     QString saveLocation = KoResourcePaths::saveLocation("data", "predefined_image_sizes/", true);
-    QFile f(saveLocation + '/' + fileName.replace(' ', '_').replace('(', '_').replace(')', '_') + ".predefinedimage");
+    QFile f(saveLocation + '/' + fileName.replace(' ', '_').replace('(', '_').replace(')', '_').replace(':', '_') + ".predefinedimage");
 
     f.open(QIODevice::WriteOnly | QIODevice::Truncate);
-    KisPropertiesConfiguration *predefined = new KisPropertiesConfiguration();
+    KisPropertiesConfigurationSP predefined = new KisPropertiesConfiguration();
     predefined->setProperty("name", txtPredefinedName->text());
     predefined->setProperty("width", doubleWidth->value());
     predefined->setProperty("height", doubleHeight->value());
@@ -401,7 +420,7 @@ void KisCustomImageWidget::saveAsPredefined()
 
     int i = 0;
     bool found = false;
-    Q_FOREACH (KisPropertiesConfiguration *pr, m_predefined) {
+    Q_FOREACH (KisPropertiesConfigurationSP pr, m_predefined) {
         if (pr->getString("name") == txtPredefinedName->text()) {
             found = true;
             break;
@@ -455,6 +474,7 @@ void KisCustomImageWidget::switchWidthHeight()
 
     widthChanged(doubleWidth->value());
     heightChanged(doubleHeight->value());
+    changeDocumentInfoLabel();
 }
 
 void KisCustomImageWidget::switchPortraitLandscape()
@@ -465,3 +485,41 @@ void KisCustomImageWidget::switchPortraitLandscape()
         bnPortrait->setChecked(true);
 }
 
+void KisCustomImageWidget::changeDocumentInfoLabel()
+{
+
+    qint64 width, height;
+    double resolution;
+    resolution = doubleResolution->value() / 72.0;  // internal resolution is in pixels per pt
+
+    width = static_cast<qint64>(0.5  + KoUnit::ptToUnit(m_width, KoUnit(KoUnit::Pixel, resolution)));
+    height = static_cast<qint64>(0.5 + KoUnit::ptToUnit(m_height, KoUnit(KoUnit::Pixel, resolution)));
+
+    qint64 layerSize = width * height;
+    const KoColorSpace *cs = colorSpaceSelector->currentColorSpace();
+    int bitSize = 8 * cs->pixelSize(); //pixelsize is in bytes.
+    layerSize = layerSize * cs->pixelSize();
+    QString byte = i18n("bytes");
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for kilobyte", "KB");
+    }
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for megabyte", "MB");
+    }
+    if (layerSize>1024) {
+        layerSize/=1024;
+        byte = i18nc("Abbreviation for gigabyte", "GB");
+    }
+    QString text = i18nc("arg1: width. arg2: height. arg3: colorspace name. arg4: size of a channel in bits. arg5: size in unites of arg6. Arg6: KB, MB or GB",
+                         "This document will be %1 pixels by %2 pixels in %3, which means the pixel size is %4 bit. A single paint layer will thus take up %5 %6 of RAM.",
+                         width,
+                         height,
+                         cs->name(),
+                         bitSize,
+                         layerSize,
+                         byte);
+    lblDocumentInfo->setText(text);
+}
+
