@@ -30,8 +30,6 @@
 #include <QClipboard>
 #include <QDesktopWidget>
 #include <QFile>
-#include <QGraphicsPixmapItem>
-#include <QGraphicsScene>
 
 #include <kis_debug.h>
 
@@ -50,10 +48,11 @@
 #include <kis_paint_device.h>
 #include <kis_painter.h>
 
+#include "kis_import_catcher.h"
 #include "kis_clipboard.h"
 #include "KisDocument.h"
 #include "widgets/kis_cmb_idlist.h"
-#include "widgets/squeezedcombobox.h"
+#include <squeezedcombobox.h>
 
 
 KisImageFromClipboard::KisImageFromClipboard(QWidget* parent, qint32 defWidth, qint32 defHeight, double resolution, const QString& defColorModel, const QString& defColorDepth, const QString& defColorProfile, const QString& imageName)
@@ -70,9 +69,10 @@ KisImageFromClipboard::KisImageFromClipboard(QWidget* parent, qint32 defWidth, q
     connect(QApplication::clipboard(), SIGNAL(dataChanged()), this, SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(selectionChanged()), this, SLOT(clipboardDataChanged()));
     connect(QApplication::clipboard(), SIGNAL(changed(QClipboard::Mode)), this, SLOT(clipboardDataChanged()));
-    
-    disconnect(createButton, SIGNAL(clicked()), 0, 0); //disable normal signal
-    connect(createButton, SIGNAL(clicked()), this, SLOT(createImage()));
+
+
+    disconnect(newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), 0, 0); //disable normal signal
+    connect(newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), this, SLOT(createImage()));
     setNumberOfLayers(1);
 }
 
@@ -84,19 +84,19 @@ void KisImageFromClipboard::createImage()
 {
     KisDocument *doc = createNewImage();
     
-    KisImageWSP image = doc->image();
+    KisImageSP image = doc->image();
     if (image && image->root() && image->root()->firstChild()) {
-        KisLayer * layer = dynamic_cast<KisLayer*>(image->root()->firstChild().data());
+        KisLayer * layer = qobject_cast<KisLayer*>(image->root()->firstChild().data());
 
         KisPaintDeviceSP clip = KisClipboard::instance()->clip(QRect(), true);
         if (clip) {
-            QRect r = clip->exactBounds();
+            KisImportCatcher::adaptClipToImageColorSpace(clip, image);
 
+            QRect r = clip->exactBounds();
             KisPainter::copyAreaOptimized(QPoint(), clip, layer->paintDevice(), r);
 
             layer->setDirty();
         }
-
     }
     doc->setModified(true);
     emit m_openPane->documentSelected(doc);
@@ -121,14 +121,14 @@ void KisImageFromClipboard::createClipboardPreview()
         if ((cbData && cbData->hasFormat(mimeType)) || !qimage.isNull()) {
             lblPreview->setPixmap(QPixmap::fromImage(qimage.scaled(75, 75, Qt::KeepAspectRatio)));
             lblPreview->show();
-            createButton->setEnabled(true);
+            newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
 
             doubleWidth->setValue(qimage.width());
             doubleHeight->setValue(qimage.height());
         }
     }
     else {
-        createButton->setEnabled(false);
+        newDialogConfirmationButtonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
         lblPreview->hide();
     }
     
