@@ -40,7 +40,7 @@ class KRITAUI_EXPORT KisAbstractSliderSpinBox : public QWidget
 protected:
     explicit KisAbstractSliderSpinBox(QWidget* parent, KisAbstractSliderSpinBoxPrivate*);
 public:
-    virtual ~KisAbstractSliderSpinBox();
+    ~KisAbstractSliderSpinBox() override;
 
     void showEdit();
     void hideEdit();
@@ -57,19 +57,22 @@ public:
      */
     void setBlockUpdateSignalOnDrag(bool block);
 
-    virtual QSize sizeHint() const;
-    virtual QSize minimumSizeHint() const;
+    QSize sizeHint() const override;
+    QSize minimumSizeHint() const override;
     virtual QSize minimumSize() const;
 
-protected:
-    virtual void paintEvent(QPaintEvent* e);
-    virtual void mousePressEvent(QMouseEvent* e);
-    virtual void mouseReleaseEvent(QMouseEvent* e);
-    virtual void mouseMoveEvent(QMouseEvent* e);
-    virtual void keyPressEvent(QKeyEvent* e);
-    virtual void wheelEvent(QWheelEvent *);
+    bool isDragging() const;
 
-    virtual bool eventFilter(QObject* recv, QEvent* e);
+protected:
+    void paintEvent(QPaintEvent* e) override;
+    void mousePressEvent(QMouseEvent* e) override;
+    void mouseReleaseEvent(QMouseEvent* e) override;
+    void mouseMoveEvent(QMouseEvent* e) override;
+    void keyPressEvent(QKeyEvent* e) override;
+    void wheelEvent(QWheelEvent *) override;
+    bool event(QEvent *event) override;
+
+    bool eventFilter(QObject* recv, QEvent* e) override;
 
     QStyleOptionSpinBox spinBoxOptions() const;
     QStyleOptionProgressBar progressBarOptions() const;
@@ -89,15 +92,20 @@ protected:
      */
     virtual void setInternalValue(int value, bool blockUpdateSignal) = 0;
 
+    /**
+     * Allows inheriting classes to directly set the value
+     */
+    void setPrivateValue(int value);
+
 protected Q_SLOTS:
-    void contextMenuEvent(QContextMenuEvent * event);
+    void contextMenuEvent(QContextMenuEvent * event) override;
     void editLostFocus();
 protected:
     KisAbstractSliderSpinBoxPrivate* const d_ptr;
 
     // QWidget interface
 protected:
-    virtual void changeEvent(QEvent *e);
+    void changeEvent(QEvent *e) override;
     void paint(QPainter& painter);
     void paintFusion(QPainter& painter);
     void paintPlastique(QPainter& painter);
@@ -115,7 +123,7 @@ class KRITAUI_EXPORT KisSliderSpinBox : public KisAbstractSliderSpinBox
     Q_PROPERTY( int maximum READ maximum WRITE setMaximum )
 public:
     KisSliderSpinBox(QWidget* parent = 0);
-    ~KisSliderSpinBox();
+    ~KisSliderSpinBox() override;
 
     void setRange(int minimum, int maximum);
 
@@ -138,8 +146,8 @@ public Q_SLOTS:
     void setValue(int value);
 
 protected:
-    virtual QString valueString() const;
-    virtual void setInternalValue(int value, bool blockUpdateSignal);
+    QString valueString() const override;
+    void setInternalValue(int value, bool blockUpdateSignal) override;
 Q_SIGNALS:
     void valueChanged(int value);
 };
@@ -150,7 +158,7 @@ class KRITAUI_EXPORT KisDoubleSliderSpinBox : public KisAbstractSliderSpinBox
     Q_DECLARE_PRIVATE(KisDoubleSliderSpinBox)
 public:
     KisDoubleSliderSpinBox(QWidget* parent = 0);
-    ~KisDoubleSliderSpinBox();
+    ~KisDoubleSliderSpinBox() override;
 
     void setRange(qreal minimum, qreal maximum, int decimals = 0);
 
@@ -162,12 +170,14 @@ public:
     void setFastSliderStep(qreal step);
 
     qreal value();
+    void setSingleStep(qreal value);
+
+public Q_SLOTS:
     void setValue(qreal value);
 
-    void setSingleStep(qreal value);
 protected:
-    virtual QString valueString() const;
-    virtual void setInternalValue(int value, bool blockUpdateSignal);
+    QString valueString() const override;
+    void setInternalValue(int value, bool blockUpdateSignal) override;
 Q_SIGNALS:
     void valueChanged(qreal value);
 };
