@@ -27,7 +27,6 @@
 
 #include <QGridLayout>
 #include <QButtonGroup>
-#include <QPushButton>
 #include <QHeaderView>
 #include <QAbstractProxyModel>
 #include <QLabel>
@@ -38,6 +37,7 @@
 #include <QSplitter>
 #include <QToolButton>
 #include <QWheelEvent>
+#include <QLineEdit>
 
 #include <klocalizedstring.h>
 
@@ -53,6 +53,8 @@
 #include "KoTagFilterWidget.h"
 #include "KoTagChooserWidget.h"
 #include "KoResourceItemChooserSync.h"
+#include "kis_assert.h"
+
 
 class Q_DECL_HIDDEN KoResourceItemChooser::Private
 {
@@ -106,12 +108,14 @@ KoResourceItemChooser::KoResourceItemChooser(QSharedPointer<KoAbstractResourceSe
     connect(d->model, SIGNAL(afterResourcesLayoutReset()), SLOT(slotAfterResourcesLayoutReset()));
 
     d->view = new KoResourceItemView(this);
+    d->view->setObjectName("ResourceItemview");
     d->view->setModel(d->model);
     d->view->setItemDelegate(new KoResourceItemDelegate(this));
     d->view->setSelectionMode(QAbstractItemView::SingleSelection);
     d->view->viewport()->installEventFilter(this);
 
     connect(d->view, SIGNAL(currentResourceChanged(QModelIndex)), this, SLOT(activated(QModelIndex)));
+    connect(d->view, SIGNAL(currentResourceClicked(QModelIndex)), this, SLOT(clicked(QModelIndex)));
     connect(d->view, SIGNAL(contextMenuRequested(QPoint)), this, SLOT(contextMenuRequested(QPoint)));
 
     connect(d->view, SIGNAL(sigSizeChanged()), this, SLOT(updateView()));
@@ -145,19 +149,18 @@ KoResourceItemChooser::KoResourceItemChooser(QSharedPointer<KoAbstractResourceSe
 
     d->buttonLayout = new QGridLayout();
 
-    QPushButton *button = new QPushButton(this);
-    button->setIcon(koIcon("document-open"));
-    button->setToolTip(i18nc("@info:tooltip", "Import resource"));
-    button->setEnabled(true);
-    d->buttonGroup->addButton(button, Button_Import);
-    d->buttonLayout->addWidget(button, 0, 0);
+    importButton = new QPushButton(this);
 
-    button = new QPushButton(this);
-    button->setIcon(koIcon("trash-empty"));
-    button->setToolTip(i18nc("@info:tooltip", "Delete resource"));
-    button->setEnabled(false);
-    d->buttonGroup->addButton(button, Button_Remove);
-    d->buttonLayout->addWidget(button, 0, 1);
+    importButton->setToolTip(i18nc("@info:tooltip", "Import resource"));
+    importButton->setEnabled(true);
+    d->buttonGroup->addButton(importButton, Button_Import);
+    d->buttonLayout->addWidget(importButton, 0, 0);
+
+    deleteButton = new QPushButton(this);
+    deleteButton->setToolTip(i18nc("@info:tooltip", "Delete resource"));
+    deleteButton->setEnabled(false);
+    d->buttonGroup->addButton(deleteButton, Button_Remove);
+    d->buttonLayout->addWidget(deleteButton, 0, 1);
 
     connect(d->buttonGroup, SIGNAL(buttonClicked(int)), this, SLOT(slotButtonClicked(int)));
 
@@ -168,11 +171,11 @@ KoResourceItemChooser::KoResourceItemChooser(QSharedPointer<KoAbstractResourceSe
     d->buttonLayout->setMargin(0);
 
     d->viewModeButton = new QToolButton(this);
-    d->viewModeButton->setIcon(koIcon("view-choose"));
     d->viewModeButton->setPopupMode(QToolButton::InstantPopup);
     d->viewModeButton->setVisible(false);
 
     d->tagManager = new KoResourceTaggingManager(d->model, this);
+    connect(d->tagManager, SIGNAL(updateView()), this, SLOT(updateView()));
 
     layout->addWidget(d->tagManager->tagChooserWidget(), 0, 0);
     layout->addWidget(d->viewModeButton, 0, 1);
@@ -181,6 +184,9 @@ KoResourceItemChooser::KoResourceItemChooser(QSharedPointer<KoAbstractResourceSe
     layout->addLayout(d->buttonLayout, 3, 0, 1, 2);
     layout->setMargin(0);
     layout->setSpacing(0);
+
+    updateView();
+
     updateButtonState();
     showTaggingBar(false);
     activated(d->model->index(0, 0));
@@ -197,7 +203,7 @@ void KoResourceItemChooser::slotButtonClicked(int button)
     if (button == Button_Import) {
         QString extensions = d->model->extensions();
         QStringList mimeTypes;
-        Q_FOREACH(const QString &suffix, extensions.split(";")) {
+        Q_FOREACH(const QString &suffix, extensions.split(":")) {
             mimeTypes << KisMimeDatabase::mimeTypeForSuffix(suffix);
         }
 
@@ -302,11 +308,9 @@ void KoResourceItemChooser::setCurrentResource(KoResource *resource)
     }
 
     QModelIndex index = d->model->indexFromResource(resource);
-    if (!index.isValid())
-        return;
 
     d->view->setCurrentIndex(index);
-    updatePreview(resource);
+    updatePreview(index.isValid() ? resource : 0);
 }
 
 void KoResourceItemChooser::slotBeforeResourcesLayoutReset(KoResource *activateAfterReset)
@@ -356,19 +360,39 @@ void KoResourceItemChooser::setProxyModel(QAbstractProxyModel *proxyModel)
     d->view->setModel(proxyModel);
 }
 
-void KoResourceItemChooser::activated(const QModelIndex &/*index*/)
+void KoResourceItemChooser::activated(const QModelIndex &index)
 {
-    KoResource *resource = currentResource();
+    if (!index.isValid()) return;
+
+    KoResource *resource = 0;
+
+    if (index.isValid()) {
+        resource = resourceFromModelIndex(index);
+    }
+
+    KIS_SAFE_ASSERT_RECOVER (resource) {
+        resource = currentResource();
+    }
+
     if (resource) {
         d->updatesBlocked = true;
         emit resourceSelected(resource);
         d->updatesBlocked = false;
-
         updatePreview(resource);
         updateButtonState();
     }
 }
 
+void KoResourceItemChooser::clicked(const QModelIndex &index)
+{
+    Q_UNUSED(index);
+
+    KoResource *resource = currentResource();
+    if (resource) {
+        emit resourceClicked(resource);
+    }
+}
+
 void KoResourceItemChooser::updateButtonState()
 {
     QAbstractButton *removeButton = d->buttonGroup->button(Button_Remove);
@@ -377,7 +401,7 @@ void KoResourceItemChooser::updateButtonState()
 
     KoResource *resource = currentResource();
     if (resource) {
-        removeButton->setEnabled(true);
+        removeButton->setEnabled(!resource->permanent());
         return;
     }
     removeButton->setEnabled(false);
@@ -385,12 +409,17 @@ void KoResourceItemChooser::updateButtonState()
 
 void KoResourceItemChooser::updatePreview(KoResource *resource)
 {
-    if (!d->usePreview || !resource) return;
+    if (!d->usePreview) return;
+
+    if (!resource) {
+        d->previewLabel->setPixmap(QPixmap());
+        return;
+    }
 
     QImage image = resource->image();
 
-    if (image.format() != QImage::Format_RGB32 ||
-        image.format() != QImage::Format_ARGB32 ||
+    if (image.format() != QImage::Format_RGB32 &&
+        image.format() != QImage::Format_ARGB32 &&
         image.format() != QImage::Format_ARGB32_Premultiplied) {
 
         image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
@@ -475,7 +504,7 @@ void KoResourceItemChooser::setSynced(bool sync)
     d->synced = sync;
     KoResourceItemChooserSync *chooserSync = KoResourceItemChooserSync::instance();
     if (sync) {
-        connect(chooserSync, SIGNAL(baseLenghtChanged(int)), SLOT(baseLengthChanged(int)));
+        connect(chooserSync, SIGNAL(baseLengthChanged(int)), SLOT(baseLengthChanged(int)));
         baseLengthChanged(chooserSync->baseLength());
     } else {
         chooserSync->disconnect(this);
@@ -487,9 +516,9 @@ void KoResourceItemChooser::baseLengthChanged(int length)
     if (d->synced) {
         int resourceCount = d->model->resourcesCount();
         int width = d->view->width();
-        int maxColums = width / length;
+        int maxColumns = width / length;
         int cols = width / (2 * length) + 1;
-        while (cols <= maxColums) {
+        while (cols <= maxColumns) {
             int size = width / cols;
             int rows = ceil(resourceCount / (double)cols);
             if (rows * size < (d->view->height() - 5)) {
@@ -518,6 +547,61 @@ bool KoResourceItemChooser::eventFilter(QObject *object, QEvent *event)
     return QObject::eventFilter(object, event);
 }
 
+void KoResourceItemChooser::configureKineticScrolling(int gesture, int sensitivity, bool scrollbar)
+{
+    QScroller::ScrollerGestureType gestureType;
+
+    switch (gesture) {
+    case 1: {
+        gestureType = QScroller::TouchGesture;
+        break;
+    }
+    case 2: {
+        gestureType = QScroller::LeftMouseButtonGesture;
+        break;
+    }
+    default:
+        return;
+    }
+
+    KoResourceItemView *view = itemView();
+
+    view->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+    if (!scrollbar) {
+        view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    }
+
+    QScroller *scroller = QScroller::scroller(view);
+    scroller->grabGesture(view, gestureType);
+
+    QScrollerProperties sp;
+
+    // DragStartDistance seems to be based on meter per second; though it's
+    // not explicitly documented, other QScroller values are in that metric.
+
+    // To start kinetic scrolling, with minimal sensitity, we expect a drag
+    // of 10 mm, with minimum sensitity any > 0 mm.
+
+    const float mm = 0.001f; // 1 millimeter
+    const float resistance = 1.0f - (sensitivity / 100.0f);
+
+    sp.setScrollMetric(QScrollerProperties::DragStartDistance, resistance * 10.0f * mm);
+    sp.setScrollMetric(QScrollerProperties::DragVelocitySmoothingFactor, 1.0f);
+    sp.setScrollMetric(QScrollerProperties::MinimumVelocity, 0.0f);
+    sp.setScrollMetric(QScrollerProperties::AxisLockThreshold, 1.0f);
+    sp.setScrollMetric(QScrollerProperties::MaximumClickThroughVelocity, 0.0f);
+    sp.setScrollMetric(QScrollerProperties::MousePressEventDelay, 1.0f - 0.75f * resistance);
+    sp.setScrollMetric(QScrollerProperties::AcceleratingFlickSpeedupFactor, 1.5f);
+
+    sp.setScrollMetric(QScrollerProperties::VerticalOvershootPolicy, QScrollerProperties::OvershootAlwaysOn);
+    sp.setScrollMetric(QScrollerProperties::OvershootDragResistanceFactor, 0.1);
+    sp.setScrollMetric(QScrollerProperties::OvershootDragDistanceFactor, 0.3);
+    sp.setScrollMetric(QScrollerProperties::OvershootScrollDistanceFactor, 0.1);
+    sp.setScrollMetric(QScrollerProperties::OvershootScrollTime, 0.4);
+
+    scroller->setScrollerProperties(sp);
+}
+
 void KoResourceItemChooser::resizeEvent(QResizeEvent *event)
 {
     QWidget::resizeEvent(event);
@@ -528,6 +612,13 @@ void KoResourceItemChooser::showEvent(QShowEvent *event)
 {
     QWidget::showEvent(event);
     updateView();
+
+    /* Give the search field focus onShowEvent to allow for
+     * faster preset filtering... */
+    if( d->tagManager ) {
+        d->tagManager->searchField()->setFocus(Qt::FocusReason::OtherFocusReason);
+        d->tagManager->searchField()->selectAll();
+    }
 }
 
 void KoResourceItemChooser::updateView()
@@ -536,4 +627,9 @@ void KoResourceItemChooser::updateView()
         KoResourceItemChooserSync *chooserSync = KoResourceItemChooserSync::instance();
         baseLengthChanged(chooserSync->baseLength());
     }
+
+    /// helps to set icons here in case the theme is changed
+    d->viewModeButton->setIcon(koIcon("view-choose"));
+    importButton->setIcon(koIcon("document-open"));
+    deleteButton->setIcon(koIcon("trash-empty"));
 }
