@@ -29,7 +29,6 @@
 #include <QMutex>
 #include "kis_debug.h"
 
-#include "WidgetUtilsDebug.h"
 
 Q_GLOBAL_STATIC(KoResourcePaths, s_instance);
 
@@ -43,7 +42,7 @@ static QStringList cleanup(const QStringList &pathList)
 {
     QStringList cleanedPathList;
     Q_FOREACH(const QString &path, pathList) {
-      cleanedPathList << cleanup(path);
+        cleanedPathList << cleanup(path);
     }
     return cleanedPathList;
 }
@@ -51,16 +50,16 @@ static QStringList cleanup(const QStringList &pathList)
 
 static QString cleanupDirs(const QString &path)
 {
-  return QDir::cleanPath(path) + QDir::separator();
+    return QDir::cleanPath(path) + QDir::separator();
 }
 
 static QStringList cleanupDirs(const QStringList &pathList)
 {
-  QStringList cleanedPathList;
-  Q_FOREACH(const QString &path, pathList) {
-    cleanedPathList << cleanupDirs(path);
-  }
-  return cleanedPathList;
+    QStringList cleanedPathList;
+    Q_FOREACH(const QString &path, pathList) {
+        cleanedPathList << cleanupDirs(path);
+    }
+    return cleanedPathList;
 }
 
 void appendResources(QStringList *dst, const QStringList &src, bool eliminateDuplicates)
@@ -80,38 +79,58 @@ static const Qt::CaseSensitivity cs = Qt::CaseInsensitive;
 static const Qt::CaseSensitivity cs = Qt::CaseSensitive;
 #endif
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_OSX
 #include <ApplicationServices/ApplicationServices.h>
 #include <CoreFoundation/CoreFoundation.h>
 #include <CoreServices/CoreServices.h>
 #endif
 
 QString getInstallationPrefix() {
-#ifdef Q_OS_MAC
-     CFURLRef appUrlRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
-     CFStringRef macPath = CFURLCopyFileSystemPath(appUrlRef,
-                                            kCFURLPOSIXPathStyle);
-     const char *pathPtr = CFStringGetCStringPtr(macPath,
-                                            CFStringGetSystemEncoding());
-     QString bundlePath = QString::fromLatin1(pathPtr);
+#ifdef Q_OS_OSX
+    QString appPath = qApp->applicationDirPath();
 
-     debugWidgetUtils << "1" << bundlePath << (bundlePath + QString::fromLatin1("/Contents/MacOS/share"));
-     if (QFile(bundlePath + QString::fromLatin1("/Contents/share")).exists()) {
-         debugWidgetUtils << "running from a deployed bundle";
-         bundlePath += QString::fromLatin1("/Contents/");
-     }
-     else {
-         debugWidgetUtils << "running from make install";
-         bundlePath += "/../../";
-     }
+    dbgResources << "1" << appPath;
+    appPath.chop(QString("MacOS/").length());
+    dbgResources << "2" << appPath;
 
-     CFRelease(appUrlRef);
-     CFRelease(macPath);
-     debugWidgetUtils << ">>>>>>>>>>>" << bundlePath;
-     return bundlePath;
- #else
-     return qApp->applicationDirPath() + "/../";
- #endif
+    bool makeInstall = QDir(appPath + "/../../../share/kritaplugins").exists();
+    bool inBundle = QDir(appPath + "/Resources/kritaplugins").exists();
+
+    dbgResources << "3. After make install" << makeInstall;
+    dbgResources << "4. In Bundle" << inBundle;
+
+    QString bundlePath;
+
+    if (inBundle) {
+        bundlePath = appPath + "/";
+    }
+    else if (makeInstall) {
+        appPath.chop(QString("Contents/").length());
+        bundlePath = appPath + "/../../";
+    }
+    else {
+        qFatal("Cannot calculate the bundle path from the app path");
+    }
+
+    dbgResources << ">>>>>>>>>>>" << bundlePath;
+    return bundlePath;
+#else
+#ifdef Q_OS_QWIN
+    QDir appdir(qApp->applicationDirPath());
+
+    // Corrects for mismatched case errors in path (qtdeclarative fails to load)
+    wchar_t buffer[1024];
+    QString absolute = appdir.absolutePath();
+    DWORD rv = ::GetShortPathName((wchar_t*)absolute.utf16(), buffer, 1024);
+    rv = ::GetLongPathName(buffer, buffer, 1024);
+    QString correctedPath((QChar *)buffer);
+    appdir.setPath(correctedPath);
+    appdir.cdUp();
+    return appdir.canonicalPath();
+#else
+    return qApp->applicationDirPath() + "/../";
+#endif
+#endif
 }
 
 class Q_DECL_HIDDEN KoResourcePaths::Private {
@@ -131,12 +150,12 @@ public:
             r += relatives[type];
         }
         relativesMutex.unlock();
-        debugWidgetUtils << "\trelatives" << r;
+        dbgResources << "\trelatives" << r;
         absolutesMutex.lock();
         if (absolutes.contains(type)) {
             a += absolutes[type];
         }
-        debugWidgetUtils << "\tabsolutes" << a;
+        dbgResources << "\tabsolutes" << a;
         absolutesMutex.unlock();
 
         return r + a;
@@ -195,9 +214,9 @@ QString KoResourcePaths::findResource(const char *type, const QString &fileName)
     return cleanup(s_instance->findResourceInternal(QString::fromLatin1(type), fileName));
 }
 
-QStringList KoResourcePaths::findDirs(const char *type, const QString &reldir)
+QStringList KoResourcePaths::findDirs(const char *type)
 {
-    return cleanupDirs(s_instance->findDirsInternal(QString::fromLatin1(type), reldir));
+    return cleanupDirs(s_instance->findDirsInternal(QString::fromLatin1(type)));
 }
 
 QStringList KoResourcePaths::findAllResources(const char *type,
@@ -254,7 +273,7 @@ void KoResourcePaths::addResourceTypeInternal(const QString &type, const QString
     }
     d->relativesMutex.unlock();
 
-    debugWidgetUtils << "addResourceType: type" << type << "basetype" << basetype << "relativename" << relativename << "priority" << priority << d->relatives[type];
+    dbgResources << "addResourceType: type" << type << "basetype" << basetype << "relativename" << relativename << "priority" << priority << d->relatives[type];
 }
 
 void KoResourcePaths::addResourceDirInternal(const QString &type, const QString &absdir, bool priority)
@@ -278,19 +297,19 @@ void KoResourcePaths::addResourceDirInternal(const QString &type, const QString
     }
     d->absolutesMutex.unlock();
 
-    debugWidgetUtils << "addResourceDir: type" << type << "absdir" << absdir << "priority" << priority << d->absolutes[type];
+    dbgResources << "addResourceDir: type" << type << "absdir" << absdir << "priority" << priority << d->absolutes[type];
 }
 
 QString KoResourcePaths::findResourceInternal(const QString &type, const QString &fileName)
 {
     QStringList aliases = d->aliases(type);
-    debugWidgetUtils << "aliases" << aliases << getApplicationRoot();
+    dbgResources<< "aliases" << aliases << getApplicationRoot();
     QString resource = QStandardPaths::locate(QStandardPaths::AppDataLocation, fileName, QStandardPaths::LocateFile);
 
     if (resource.isEmpty()) {
         Q_FOREACH (const QString &alias, aliases) {
             resource = QStandardPaths::locate(d->mapTypeToQStandardPaths(type), alias + '/' + fileName, QStandardPaths::LocateFile);
-            debugWidgetUtils << "\t1" << resource;
+            dbgResources << "\t1" << resource;
             if (QFile::exists(resource)) {
                 continue;
             }
@@ -300,7 +319,8 @@ QString KoResourcePaths::findResourceInternal(const QString &type, const QString
         QString approot = getApplicationRoot();
         Q_FOREACH (const QString &alias, aliases) {
             resource = approot + "/share/" + alias + '/' + fileName;
-            debugWidgetUtils << "\t1" << resource;
+            dbgResources << "\t2" << resource;
+
             if (QFile::exists(resource)) {
                 continue;
             }
@@ -310,69 +330,53 @@ QString KoResourcePaths::findResourceInternal(const QString &type, const QString
         QString approot = getApplicationRoot();
         Q_FOREACH (const QString &alias, aliases) {
             resource = approot + "/share/krita/" + alias + '/' + fileName;
-            debugWidgetUtils << "\t1" << resource;
+            dbgResources << "\t3" << resource;
             if (QFile::exists(resource)) {
                 continue;
             }
         }
     }
 
-    debugWidgetUtils << "findResource: type" << type << "filename" << fileName << "resource" << resource;
+    if (resource.isEmpty() || !QFile::exists(resource)) {
+        QString extraResourceDirs = qgetenv("EXTRA_RESOURCE_DIRS");
+        if (!extraResourceDirs.isEmpty()) {
+            Q_FOREACH(const QString &extraResourceDir, extraResourceDirs.split(':', QString::SkipEmptyParts)) {
+                if (aliases.isEmpty()) {
+                    resource = extraResourceDir + '/' + fileName;
+                    dbgResources<< "\t4" << resource;
+                    if (QFile::exists(resource)) {
+                        continue;
+                    }
+                }
+                else {
+                    Q_FOREACH (const QString &alias, aliases) {
+                        resource = extraResourceDir + '/' + alias + '/' + fileName;
+                        dbgResources<< "\t4" << resource;
+                        if (QFile::exists(resource)) {
+                            continue;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    dbgResources<< "findResource: type" << type << "filename" << fileName << "resource" << resource;
     Q_ASSERT(!resource.isEmpty());
     return resource;
 }
 
-QStringList KoResourcePaths::findDirsInternal(const QString &type, const QString &relDir)
-{
-    QStringList aliases = d->aliases(type);
-    debugWidgetUtils << type << relDir << aliases << d->mapTypeToQStandardPaths(type);
-
-    QStringList dirs;
-
-    {
-        QStringList standardDirs =
-            QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), relDir, QStandardPaths::LocateDirectory);
-        appendResources(&dirs, standardDirs, true);
-    }
-
-    Q_FOREACH (const QString &alias, aliases) {
-        QStringList aliasDirs =
-            QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), alias + '/' + relDir, QStandardPaths::LocateDirectory);
-        appendResources(&dirs, aliasDirs, true);
-    }
-
-#ifdef Q_OS_MAC
-
-    {
-        QStringList bundlePaths;
-        bundlePaths << getApplicationRoot() + "/share/" + relDir;
-        bundlePaths << getApplicationRoot() + "/../share/" + relDir;
-        appendResources(&dirs, bundlePaths, true);
-    }
-#endif
-
-    if (dirs.isEmpty()) {
-        QStringList fallbackPaths;
-        fallbackPaths << getApplicationRoot() + "/share/" + relDir;
-        fallbackPaths << getApplicationRoot() + "/share/krita/" + relDir;
-        appendResources(&dirs, fallbackPaths, true);
-    }
-
-    debugWidgetUtils << "findDirs: type" << type << "relDir" << relDir<< "resource" << dirs;
-    return dirs;
-}
-
 
 QStringList filesInDir(const QString &startdir, const QString & filter, bool recursive)
 {
-    debugWidgetUtils << "filesInDir: startdir" << startdir << "filter" << filter << "recursive" << recursive;
+    dbgResources << "filesInDir: startdir" << startdir << "filter" << filter << "recursive" << recursive;
     QStringList result;
 
     // First the entries in this path
     QStringList nameFilters;
     nameFilters << filter;
     const QStringList fileNames = QDir(startdir).entryList(nameFilters, QDir::Files | QDir::CaseSensitive, QDir::Name);
-    debugWidgetUtils << "\tFound:" << fileNames.size() << ":" << fileNames;
+    dbgResources << "\tFound:" << fileNames.size() << ":" << fileNames;
     Q_FOREACH (const QString &fileName, fileNames) {
         QString file = startdir + '/' + fileName;
         result << file;
@@ -382,23 +386,59 @@ QStringList filesInDir(const QString &startdir, const QString & filter, bool rec
     if (recursive) {
         const QStringList entries = QDir(startdir).entryList(QDir::Dirs | QDir::NoDotAndDotDot);
         Q_FOREACH (const QString &subdir, entries) {
-            debugWidgetUtils << "\tGoing to look in subdir" << subdir << "of" << startdir;
+            dbgResources << "\tGoing to look in subdir" << subdir << "of" << startdir;
             result << filesInDir(startdir + '/' + subdir, filter, recursive);
         }
     }
     return result;
 }
 
+QStringList KoResourcePaths::findDirsInternal(const QString &type)
+{
+    QStringList aliases = d->aliases(type);
+    dbgResources << type << aliases << d->mapTypeToQStandardPaths(type);
+
+    QStringList dirs;
+    QStringList standardDirs =
+            QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), "", QStandardPaths::LocateDirectory);
+    appendResources(&dirs, standardDirs, true);
+
+    Q_FOREACH (const QString &alias, aliases) {
+        QStringList aliasDirs =
+                QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), alias + '/', QStandardPaths::LocateDirectory);
+        appendResources(&dirs, aliasDirs, true);
+
+#ifdef Q_OS_OSX
+        dbgResources << "MAC:" << getApplicationRoot();
+        QStringList bundlePaths;
+        bundlePaths << getApplicationRoot() + "/share/krita/" + alias;
+        bundlePaths << getApplicationRoot() + "/../share/krita/" + alias;
+        dbgResources << "bundlePaths" << bundlePaths;
+        appendResources(&dirs, bundlePaths, true);
+        Q_ASSERT(!dirs.isEmpty());
+#endif
+
+        QStringList fallbackPaths;
+        fallbackPaths << getApplicationRoot() + "/share/" + alias;
+        fallbackPaths << getApplicationRoot() + "/share/krita/" + alias;
+        appendResources(&dirs, fallbackPaths, true);
+
+    }
+    dbgResources << "findDirs: type" << type << "resource" << dirs;
+    return dirs;
+}
+
+
 QStringList KoResourcePaths::findAllResourcesInternal(const QString &type,
                                                       const QString &_filter,
                                                       SearchOptions options) const
 {
-    debugWidgetUtils << "=====================================================";
-    debugWidgetUtils << type << _filter << QStandardPaths::standardLocations(d->mapTypeToQStandardPaths(type));
+    dbgResources << "=====================================================";
+    dbgResources << type << _filter << QStandardPaths::standardLocations(d->mapTypeToQStandardPaths(type));
 
     bool recursive = options & KoResourcePaths::Recursive;
 
-    debugWidgetUtils << "findAllResources: type" << type << "filter" << _filter << "recursive" << recursive;
+    dbgResources << "findAllResources: type" << type << "filter" << _filter << "recursive" << recursive;
 
     QStringList aliases = d->aliases(type);
     QString filter = _filter;
@@ -407,26 +447,50 @@ QStringList KoResourcePaths::findAllResourcesInternal(const QString &type,
     if (filter.indexOf('*') > 0) {
         aliases << filter.split('*').first();
         filter = '*' + filter.split('*')[1];
-        debugWidgetUtils << "Split up alias" << aliases << "filter" << filter;
+        dbgResources << "Split up alias" << aliases << "filter" << filter;
     }
 
     QStringList resources;
     if (aliases.isEmpty()) {
         QStringList standardResources =
-            QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type),
-                                      filter, QStandardPaths::LocateFile);
+                QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type),
+                                          filter, QStandardPaths::LocateFile);
+        dbgResources << "standardResources" << standardResources;
         appendResources(&resources, standardResources, true);
+        dbgResources << "1" << resources;
     }
 
-    debugWidgetUtils << "\tresources from qstandardpaths:" << resources.size();
+
+    QString extraResourceDirs = qgetenv("EXTRA_RESOURCE_DIRS");
+    dbgResources << "extraResourceDirs" << extraResourceDirs;
+    if (!extraResourceDirs.isEmpty()) {
+        Q_FOREACH(const QString &extraResourceDir, extraResourceDirs.split(':', QString::SkipEmptyParts)) {
+            if (aliases.isEmpty()) {
+                appendResources(&resources, filesInDir(extraResourceDir + '/' + type, filter, recursive), true);
+            }
+            else {
+                Q_FOREACH (const QString &alias, aliases) {
+                    appendResources(&resources, filesInDir(extraResourceDir + '/' + alias + '/', filter, recursive), true);
+                }
+            }
+        }
+
+    }
+
+    dbgResources << "\tresources from qstandardpaths:" << resources.size();
 
     Q_FOREACH (const QString &alias, aliases) {
-        debugWidgetUtils << "\t\talias:" << alias;
+        dbgResources << "\t\talias:" << alias;
         QStringList dirs;
 
-        dirs << QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), alias, QStandardPaths::LocateDirectory)
-             << getInstallationPrefix() + "share/" + alias + "/"
-             << getInstallationPrefix() + "share/krita/" + alias + "/";
+        QFileInfo dirInfo(alias);
+        if (dirInfo.exists() && dirInfo.isDir() && dirInfo.isAbsolute()) {
+            dirs << alias;
+        } else {
+            dirs << QStandardPaths::locateAll(d->mapTypeToQStandardPaths(type), alias, QStandardPaths::LocateDirectory)
+                 << getInstallationPrefix() + "share/" + alias + "/"
+                 << getInstallationPrefix() + "share/krita/" + alias + "/";
+        }
 
         Q_FOREACH (const QString &dir, dirs) {
             appendResources(&resources,
@@ -435,19 +499,19 @@ QStringList KoResourcePaths::findAllResourcesInternal(const QString &type,
         }
     }
 
-    debugWidgetUtils << "\tresources also from aliases:" << resources.size();
+    dbgResources << "\tresources also from aliases:" << resources.size();
 
-    if (resources.isEmpty()) {
-        QFileInfo fi(filter);
+    // if the original filter is "input/*", we only want share/input/* and share/krita/input/* here, but not
+    // share/*. therefore, use _filter here instead of filter which was split into alias and "*".
+    QFileInfo fi(_filter);
 
-        QStringList prefixResources;
-        prefixResources << filesInDir(getInstallationPrefix() + "share/" + fi.path(), fi.fileName(), false);
-        prefixResources << filesInDir(getInstallationPrefix() + "share/krita/" + fi.path(), fi.fileName(), false);
-        appendResources(&resources, prefixResources, true);
-    }
+    QStringList prefixResources;
+    prefixResources << filesInDir(getInstallationPrefix() + "share/" + fi.path(), fi.fileName(), false);
+    prefixResources << filesInDir(getInstallationPrefix() + "share/krita/" + fi.path(), fi.fileName(), false);
+    appendResources(&resources, prefixResources, true);
 
-    debugWidgetUtils << "\tresources from installation:" << resources.size();
-    debugWidgetUtils << "=====================================================";
+    dbgResources << "\tresources from installation:" << resources.size();
+    dbgResources << "=====================================================";
 
     return resources;
 }
@@ -470,7 +534,7 @@ QStringList KoResourcePaths::resourceDirsInternal(const QString &type)
         appendResources(&resourceDirs, aliasDirs, true);
     }
 
-    debugWidgetUtils << "resourceDirs: type" << type << resourceDirs;
+    dbgResources << "resourceDirs: type" << type << resourceDirs;
 
     return resourceDirs;
 }
@@ -497,7 +561,7 @@ QString KoResourcePaths::saveLocationInternal(const QString &type, const QString
     if (!d.exists() && create) {
         d.mkpath(path);
     }
-    debugWidgetUtils << "saveLocation: type" << type << "suffix" << suffix << "create" << create << "path" << path;
+    dbgResources << "saveLocation: type" << type << "suffix" << suffix << "create" << create << "path" << path;
 
     return path;
 }
@@ -515,7 +579,7 @@ QString KoResourcePaths::locateInternal(const QString &type, const QString &file
         locations << QStandardPaths::locate(d->mapTypeToQStandardPaths(type),
                                             (alias.endsWith('/') ? alias : alias + '/') + filename, QStandardPaths::LocateFile);
     }
-    debugWidgetUtils << "locate: type" << type << "filename" << filename << "locations" << locations;
+    dbgResources << "locate: type" << type << "filename" << filename << "locations" << locations;
     if (locations.size() > 0) {
         return locations.first();
     }
@@ -527,6 +591,6 @@ QString KoResourcePaths::locateInternal(const QString &type, const QString &file
 QString KoResourcePaths::locateLocalInternal(const QString &type, const QString &filename, bool createDir)
 {
     QString path = saveLocationInternal(type, "", createDir);
-    debugWidgetUtils << "locateLocal: type" << type << "filename" << filename << "CreateDir" << createDir << "path" << path;
+    dbgResources << "locateLocal: type" << type << "filename" << filename << "CreateDir" << createDir << "path" << path;
     return path + '/' + filename;
 }
