@@ -29,7 +29,6 @@
 #include "kactioncategory.h"
 #include "kxmlguiclient.h"
 #include "kxmlguifactory.h"
-#include "kactioncategory.h"
 #include "kis_action_registry.h"
 
 #include <kauthorized.h>
@@ -47,6 +46,10 @@
 
 #include <stdio.h>
 
+#if defined(KCONFIG_BEFORE_5_24)
+# define authorizeAction authorizeKAction
+#endif
+
 class KActionCollectionPrivate
 {
 public:
@@ -291,7 +294,7 @@ QAction *KActionCollection::addAction(const QString &name, QAction *action)
         return action;
     }
 
-    if (!KAuthorized::authorizeKAction(indexName)) {
+    if (!KAuthorized::authorizeAction(indexName)) {
         // Disable this action
         action->setEnabled(false);
         action->setVisible(false);
