@@ -31,12 +31,11 @@
 
 #include <QDebug>
 
+#include "kis_debug.h"
+
 class LcmsColorProfileContainer::Private
 {
 public:
-    Private()
-        : valid(false)
-        , suitableForOutput(false) { }
 
     cmsHPROFILE profile;
     cmsColorSpaceSignature colorSpaceSignature;
@@ -46,31 +45,32 @@ public:
     QString copyright;
     QString name;
     float version;
-    IccColorProfile::Data *data;
-    bool valid;
-    bool suitableForOutput;
+    IccColorProfile::Data *data {0};
+    bool valid {false};
+    bool suitableForOutput {false};
     bool hasColorants;
     bool hasTRC;
     bool adaptedFromD50;
-    cmsNAMEDCOLORLIST *namedColorList;
     cmsCIEXYZ mediaWhitePoint;
     cmsCIExyY whitePoint;
     cmsCIEXYZTRIPLE colorants;
-    cmsToneCurve *redTRC;
-    cmsToneCurve *greenTRC;
-    cmsToneCurve *blueTRC;
-    cmsToneCurve *grayTRC;
-    cmsToneCurve *redTRCReverse;
-    cmsToneCurve *greenTRCReverse;
-    cmsToneCurve *blueTRCReverse;
-    cmsToneCurve *grayTRCReverse;
-    
+    cmsToneCurve *redTRC {0};
+    cmsToneCurve *greenTRC {0};
+    cmsToneCurve *blueTRC {0};
+    cmsToneCurve *grayTRC {0};
+    cmsToneCurve *redTRCReverse {0};
+    cmsToneCurve *greenTRCReverse {0};
+    cmsToneCurve *blueTRCReverse {0};
+    cmsToneCurve *grayTRCReverse {0};
+
     cmsUInt32Number defaultIntent;
     bool isPerceptualCLUT;
     bool isRelativeCLUT;
     bool isAbsoluteCLUT;
     bool isSaturationCLUT;
     bool isMatrixShaper;
+
+    QByteArray uniqueId;
 };
 
 LcmsColorProfileContainer::LcmsColorProfileContainer()
@@ -139,7 +139,7 @@ bool LcmsColorProfileContainer::init()
         cmsGetProfileInfo(d->profile, cmsInfoDescription, cmsNoLanguage, cmsNoCountry, buffer, _BUFFER_SIZE_);
         d->name = QString::fromWCharArray(buffer);
 
-        //apparantly this should give us a localised string??? Not sure about this.
+        //apparently this should give us a localised string??? Not sure about this.
         cmsGetProfileInfo(d->profile, cmsInfoModel, cmsNoLanguage, cmsNoCountry, buffer, _BUFFER_SIZE_);
         d->productDescription = QString::fromWCharArray(buffer);
 
@@ -206,7 +206,7 @@ bool LcmsColorProfileContainer::init()
         }
         //retrieve TRC.
         if (cmsIsTag(d->profile, cmsSigRedTRCTag) && cmsIsTag(d->profile, cmsSigBlueTRCTag) && cmsIsTag(d->profile, cmsSigGreenTRCTag)) {
-        
+
             d->redTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigRedTRCTag));
             d->greenTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigGreenTRCTag));
             d->blueTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigBlueTRCTag));
@@ -214,7 +214,7 @@ bool LcmsColorProfileContainer::init()
             d->greenTRCReverse = cmsReverseToneCurve(d->greenTRC);
             d->blueTRCReverse = cmsReverseToneCurve(d->blueTRC);
             d->hasTRC = true;
-        
+
         } else if (cmsIsTag(d->profile, cmsSigGrayTRCTag)) {
             d->grayTRC = ((cmsToneCurve *)cmsReadTag (d->profile, cmsSigGrayTRCTag));
             d->grayTRCReverse = cmsReverseToneCurve(d->grayTRC);
@@ -423,131 +423,103 @@ QVector <double> LcmsColorProfileContainer::getEstimatedTRC() const
 
 void LcmsColorProfileContainer::LinearizeFloatValue(QVector <double> & Value) const
 {
-    QVector <double> TRCtriplet(3);
-    TRCtriplet[0] = Value[0];
-    TRCtriplet[1] = Value[1];
-    TRCtriplet[2] = Value[2];
-    
-
     if (d->hasColorants) {
-        if (cmsIsToneCurveLinear(d->redTRC)) {
-            TRCtriplet[0] = Value[0];
-        } else {
-            TRCtriplet[0] = cmsEvalToneCurveFloat(d->redTRC, Value[0]);
+        if (!cmsIsToneCurveLinear(d->redTRC)) {
+            Value[0] = cmsEvalToneCurveFloat(d->redTRC, Value[0]);
         }
-        if (cmsIsToneCurveLinear(d->greenTRC)) {
-            TRCtriplet[1] = Value[1];
-        } else {
-            TRCtriplet[1] = cmsEvalToneCurveFloat(d->greenTRC, Value[1]);
+        if (!cmsIsToneCurveLinear(d->greenTRC)) {
+            Value[1] = cmsEvalToneCurveFloat(d->greenTRC, Value[1]);
         }
-        if (cmsIsToneCurveLinear(d->blueTRC)) {
-            TRCtriplet[2] = Value[2];
-        } else {
-            TRCtriplet[2] = cmsEvalToneCurveFloat(d->blueTRC, Value[2]);
+        if (!cmsIsToneCurveLinear(d->blueTRC)) {
+            Value[2] = cmsEvalToneCurveFloat(d->blueTRC, Value[2]);
         }
-            
+
     } else {
         if (cmsIsTag(d->profile, cmsSigGrayTRCTag)) {
-            TRCtriplet.fill(cmsEvalToneCurveFloat(d->grayTRC, Value[0]));
+            Value[0] = cmsEvalToneCurveFloat(d->grayTRC, Value[0]);
         }
     }
-
-    Value[0] = TRCtriplet[0];
-    Value[1] = TRCtriplet[1];
-    Value[2] = TRCtriplet[2];
 }
 
 void LcmsColorProfileContainer::DelinearizeFloatValue(QVector <double> & Value) const
 {
-    QVector <double> TRCtriplet(3);
-    TRCtriplet[0] = Value[0];
-    TRCtriplet[1] = Value[1];
-    TRCtriplet[2] = Value[2];
-    if (cmsIsTag(d->profile, cmsSigRedTRCTag)) {
-        if (cmsIsToneCurveLinear(d->redTRC)) {
-            TRCtriplet[0] = Value[0];
-        } else {
-            TRCtriplet[0] = cmsEvalToneCurveFloat(d->redTRCReverse, Value[0]);
+    if (d->hasColorants) {
+        if (!cmsIsToneCurveLinear(d->redTRC)) {
+            Value[0] = cmsEvalToneCurveFloat(d->redTRCReverse, Value[0]);
         }
-        if (cmsIsToneCurveLinear(d->greenTRC)) {
-            TRCtriplet[1] = Value[1];
-        } else {
-            TRCtriplet[1] = cmsEvalToneCurveFloat(d->greenTRCReverse, Value[1]);
+        if (!cmsIsToneCurveLinear(d->greenTRC)) {
+            Value[1] = cmsEvalToneCurveFloat(d->greenTRCReverse, Value[1]);
         }
-        if (cmsIsToneCurveLinear(d->blueTRC)) {
-            TRCtriplet[2] = Value[2];
-        } else {
-            TRCtriplet[2] = cmsEvalToneCurveFloat(d->blueTRCReverse, Value[2]);
+        if (!cmsIsToneCurveLinear(d->blueTRC)) {
+            Value[2] = cmsEvalToneCurveFloat(d->blueTRCReverse, Value[2]);
         }
-            
+
     } else {
         if (cmsIsTag(d->profile, cmsSigGrayTRCTag)) {
-            TRCtriplet.fill(cmsEvalToneCurveFloat(d->grayTRCReverse, Value[0]));
+            Value[0] = cmsEvalToneCurveFloat(d->grayTRCReverse, Value[0]);
         }
     }
-
-    Value[0] = TRCtriplet[0];
-    Value[1] = TRCtriplet[1];
-    Value[2] = TRCtriplet[2];
 }
 
 void LcmsColorProfileContainer::LinearizeFloatValueFast(QVector <double> & Value) const
 {
-    //we can only reliably delinearise in the 0-1.0 range, outside of that leave the value alone.
-    QVector <quint16> TRCtriplet(3);
-    TRCtriplet[0] = Value[0]*65535;
-    TRCtriplet[1] = Value[1]*65535;
-    TRCtriplet[2] = Value[2]*65535;
-    
+    const qreal scale = 65535.0;
+    const qreal invScale = 1.0 / scale;
 
     if (d->hasColorants) {
+        //we can only reliably delinearise in the 0-1.0 range, outside of that leave the value alone.
+        QVector <quint16> TRCtriplet(3);
+        TRCtriplet[0] = Value[0] * scale;
+        TRCtriplet[1] = Value[1] * scale;
+        TRCtriplet[2] = Value[2] * scale;
+
         if (!cmsIsToneCurveLinear(d->redTRC) && Value[0]<1.0) {
             TRCtriplet[0] = cmsEvalToneCurve16(d->redTRC, TRCtriplet[0]);
-            Value[0] = TRCtriplet[0]/65535.0;
+            Value[0] = TRCtriplet[0] * invScale;
         }
         if (!cmsIsToneCurveLinear(d->greenTRC) && Value[1]<1.0) {
             TRCtriplet[1] = cmsEvalToneCurve16(d->greenTRC, TRCtriplet[1]);
-            Value[1] = TRCtriplet[1]/65535.0;
+            Value[1] = TRCtriplet[1] * invScale;
         }
         if (!cmsIsToneCurveLinear(d->blueTRC) && Value[2]<1.0) {
             TRCtriplet[2] = cmsEvalToneCurve16(d->blueTRC, TRCtriplet[2]);
-            Value[2] = TRCtriplet[2]/65535.0;
+            Value[2] = TRCtriplet[2] * invScale;
         }
-            
     } else {
         if (cmsIsTag(d->profile, cmsSigGrayTRCTag) && Value[0]<1.0) {
-            TRCtriplet[0] = (cmsEvalToneCurve16(d->grayTRC, Value[0]*65535));
-            Value.fill(TRCtriplet[0]/65535.0);
+            quint16 newValue = cmsEvalToneCurve16(d->grayTRC, Value[0] * scale);
+            Value[0] = newValue * invScale;
         }
     }
 }
 void LcmsColorProfileContainer::DelinearizeFloatValueFast(QVector <double> & Value) const
 {
-    //we can only reliably delinearise in the 0-1.0 range, outside of that leave the value alone.
-    QVector <quint16> TRCtriplet(3);
-    TRCtriplet[0] = Value[0]*65535;
-    TRCtriplet[1] = Value[1]*65535;
-    TRCtriplet[2] = Value[2]*65535;
-    
+    const qreal scale = 65535.0;
+    const qreal invScale = 1.0 / scale;
 
     if (d->hasColorants) {
+        //we can only reliably delinearise in the 0-1.0 range, outside of that leave the value alone.
+        QVector <quint16> TRCtriplet(3);
+        TRCtriplet[0] = Value[0] * scale;
+        TRCtriplet[1] = Value[1] * scale;
+        TRCtriplet[2] = Value[2] * scale;
+
         if (!cmsIsToneCurveLinear(d->redTRC) && Value[0]<1.0) {
             TRCtriplet[0] = cmsEvalToneCurve16(d->redTRCReverse, TRCtriplet[0]);
-            Value[0] = TRCtriplet[0]/65535.0;
+            Value[0] = TRCtriplet[0] * invScale;
         }
         if (!cmsIsToneCurveLinear(d->greenTRC) && Value[1]<1.0) {
             TRCtriplet[1] = cmsEvalToneCurve16(d->greenTRCReverse, TRCtriplet[1]);
-            Value[1] = TRCtriplet[1]/65535.0;
+            Value[1] = TRCtriplet[1] * invScale;
         }
         if (!cmsIsToneCurveLinear(d->blueTRC) && Value[2]<1.0) {
             TRCtriplet[2] = cmsEvalToneCurve16(d->blueTRCReverse, TRCtriplet[2]);
-            Value[2] = TRCtriplet[2]/65535.0;
+            Value[2] = TRCtriplet[2] * invScale;
         }
-            
     } else {
         if (cmsIsTag(d->profile, cmsSigGrayTRCTag) && Value[0]<1.0) {
-            TRCtriplet[0] = (cmsEvalToneCurve16(d->grayTRCReverse, Value[0]*65535));
-            Value.fill(TRCtriplet[0]/65535.0);
+            quint16 newValue = cmsEvalToneCurve16(d->grayTRCReverse, Value[0] * scale);
+            Value[0] = newValue * invScale;
         }
     }
 }
@@ -561,3 +533,27 @@ QString LcmsColorProfileContainer::info() const
 {
     return d->productDescription;
 }
+
+QByteArray LcmsColorProfileContainer::getProfileUniqueId() const
+{
+    if (d->uniqueId.isEmpty() && d->profile) {
+        QByteArray id(sizeof(cmsProfileID), 0);
+        cmsGetHeaderProfileID(d->profile, (quint8*)id.data());
+
+        bool isNull = std::all_of(id.constBegin(),
+                                  id.constEnd(),
+                                  [](char c) {return c == 0;});
+        if (isNull) {
+            if (cmsMD5computeID(d->profile)) {
+                cmsGetHeaderProfileID(d->profile, (quint8*)id.data());
+                isNull = false;
+            }
+        }
+
+        if (!isNull) {
+            d->uniqueId = id;
+        }
+    }
+
+    return d->uniqueId;
+}
