@@ -18,22 +18,38 @@
 #ifndef CHANNELDOCKER_DOCK_H
 #define CHANNELDOCKER_DOCK_H
 
+#include <QPointer>
 #include <QDockWidget>
+
 #include <KoCanvasObserverBase.h>
 
+#include <kis_canvas2.h>
+
 class ChannelModel;
 class QTableView;
-class KisCanvas2;
+class KisSignalCompressor;
+class KisIdleWatcher;
 
 class ChannelDockerDock : public QDockWidget, public KoCanvasObserverBase {
     Q_OBJECT
 public:
     ChannelDockerDock();
-    QString observerName() { return "ChannelDockerDock"; }
-    virtual void setCanvas(KoCanvasBase *canvas);
-    virtual void unsetCanvas() { m_canvas = 0; setEnabled(false);}
+
+    QString observerName() override { return "ChannelDockerDock"; }
+    void setCanvas(KoCanvasBase *canvas) override;
+    void unsetCanvas() override;
+    void showEvent(QShowEvent *event) override;
+
+public Q_SLOTS:
+    void startUpdateCanvasProjection();
+
+private Q_SLOTS:
+    void updateChannelTable(void);
+
 private:
-    KisCanvas2 *m_canvas;
+    KisIdleWatcher* m_imageIdleWatcher;
+    KisSignalCompressor *m_compressor;
+    QPointer<KisCanvas2> m_canvas;
     QTableView *m_channelTable;
     ChannelModel *m_model;
 };
