@@ -50,7 +50,7 @@
 #include <kis_config_notifier.h>
 #include <widgets/kis_double_widget.h>
 #include <kis_image.h>
-#include "widgets/squeezedcombobox.h"
+#include <squeezedcombobox.h>
 #include "kis_signals_blocker.h"
 #include "krita_utils.h"
 
@@ -91,7 +91,6 @@ LutDockerDock::LutDockerDock()
     : QDockWidget(i18n("LUT Management"))
     , m_canvas(0)
     , m_draggingSlider(false)
-    , m_ownDisplayFilter(false)
 {
     using namespace std::placeholders; // For _1
     m_exposureCompressor.reset(
@@ -164,28 +163,24 @@ LutDockerDock::LutDockerDock()
     connect(m_cmbInputColorSpace, SIGNAL(currentIndexChanged(int)), SLOT(updateDisplaySettings()));
     connect(m_cmbDisplayDevice, SIGNAL(currentIndexChanged(int)), SLOT(updateDisplaySettings()));
     connect(m_cmbView, SIGNAL(currentIndexChanged(int)), SLOT(updateDisplaySettings()));
+    connect(m_cmbLook, SIGNAL(currentIndexChanged(int)), SLOT(updateDisplaySettings()));
     connect(m_cmbComponents, SIGNAL(currentIndexChanged(int)), SLOT(updateDisplaySettings()));
 
     m_draggingSlider = false;
 
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), SLOT(resetOcioConfiguration()));
 
-    m_displayFilter = 0;
-
     resetOcioConfiguration();
 }
 
 LutDockerDock::~LutDockerDock()
 {
-    if(m_ownDisplayFilter)
-        delete m_displayFilter;
 }
 
 void LutDockerDock::setCanvas(KoCanvasBase* _canvas)
 {
     if (m_canvas) {
         m_canvas->disconnect(this);
-        m_displayFilter = 0;
     }
 
     setEnabled(_canvas != 0);
@@ -194,24 +189,23 @@ void LutDockerDock::setCanvas(KoCanvasBase* _canvas)
         m_canvas = canvas;
         if (m_canvas) {
             if (!m_canvas->displayFilter()) {
-                m_displayFilter = new OcioDisplayFilter(this);
-                m_ownDisplayFilter = true;
                 resetOcioConfiguration();
                 updateDisplaySettings();
             }
             else {
-                m_displayFilter = dynamic_cast<OcioDisplayFilter*>(m_canvas->displayFilter());
-                Q_ASSERT(m_displayFilter);
-                m_ocioConfig = m_displayFilter->config;
+                m_displayFilter = m_canvas->displayFilter();
+                OcioDisplayFilter *displayFilter = qobject_cast<OcioDisplayFilter*>(m_displayFilter.data());
+                Q_ASSERT(displayFilter);
+                m_ocioConfig = displayFilter->config;
                 KisSignalsBlocker exposureBlocker(m_exposureDoubleWidget);
-                m_exposureDoubleWidget->setValue(m_displayFilter->exposure);
+                m_exposureDoubleWidget->setValue(displayFilter->exposure);
                 KisSignalsBlocker gammaBlocker(m_gammaDoubleWidget);
-                m_gammaDoubleWidget->setValue(m_displayFilter->gamma);
+                m_gammaDoubleWidget->setValue(displayFilter->gamma);
                 KisSignalsBlocker componentsBlocker(m_cmbComponents);
-                m_cmbComponents->setCurrentIndex((int)m_displayFilter->swizzle);
+                m_cmbComponents->setCurrentIndex((int)displayFilter->swizzle);
                 KisSignalsBlocker bwBlocker(m_bwPointChooser);
-                m_bwPointChooser->setBlackPoint(m_displayFilter->blackPoint);
-                m_bwPointChooser->setWhitePoint(m_displayFilter->whitePoint);
+                m_bwPointChooser->setBlackPoint(displayFilter->blackPoint);
+                m_bwPointChooser->setWhitePoint(displayFilter->whitePoint);
             }
 
             connect(m_canvas->image(), SIGNAL(sigColorSpaceChanged(const KoColorSpace*)), SLOT(slotImageColorSpaceChanged()), Qt::UniqueConnection);
@@ -221,6 +215,13 @@ void LutDockerDock::setCanvas(KoCanvasBase* _canvas)
     }
 }
 
+void LutDockerDock::unsetCanvas()
+{
+    m_canvas = 0;
+    setEnabled(false);
+    m_displayFilter = QSharedPointer<KisDisplayFilter>(0);
+}
+
 void LutDockerDock::slotUpdateIcons()
 {
     m_btnConvertCurrentColor->setIcon(KisIconUtils::loadIcon("krita_tool_freehand"));
@@ -240,7 +241,8 @@ bool LutDockerDock::canChangeExposureAndGamma() const
 qreal LutDockerDock::currentExposure() const
 {
     if (!m_displayFilter) return 0.0;
-    return canChangeExposureAndGamma() ? m_displayFilter->exposure : 0.0;
+    OcioDisplayFilter *displayFilter = qobject_cast<OcioDisplayFilter*>(m_displayFilter.data());
+    return canChangeExposureAndGamma() ? displayFilter->exposure : 0.0;
 }
 
 void LutDockerDock::setCurrentExposure(qreal value)
@@ -252,7 +254,8 @@ void LutDockerDock::setCurrentExposure(qreal value)
 qreal LutDockerDock::currentGamma() const
 {
     if (!m_displayFilter) return 1.0;
-    return canChangeExposureAndGamma() ? m_displayFilter->gamma : 1.0;
+    OcioDisplayFilter *displayFilter = qobject_cast<OcioDisplayFilter*>(m_displayFilter.data());
+    return canChangeExposureAndGamma() ? displayFilter->gamma : 1.0;
 }
 
 void LutDockerDock::setCurrentGamma(qreal value)
@@ -357,6 +360,8 @@ void LutDockerDock::enableControls()
     m_cmbDisplayDevice->setEnabled(ocioEnabled && externalColorManagementEnabled);
     m_lblView->setEnabled(ocioEnabled && externalColorManagementEnabled);
     m_cmbView->setEnabled(ocioEnabled && externalColorManagementEnabled);
+    m_lblLook->setEnabled(ocioEnabled && externalColorManagementEnabled);
+    m_cmbLook->setEnabled(ocioEnabled && externalColorManagementEnabled);
 
     bool enableConfigPath = m_colorManagement->currentIndex() == (int) KisConfig::OCIO_CONFIG;
 
@@ -375,27 +380,39 @@ void LutDockerDock::updateDisplaySettings()
     writeControls();
 
     if (m_chkUseOcio->isChecked() && m_ocioConfig) {
-        m_displayFilter->config = m_ocioConfig;
-        m_displayFilter->inputColorSpaceName = m_ocioConfig->getColorSpaceNameByIndex(m_cmbInputColorSpace->currentIndex());
-        m_displayFilter->displayDevice = m_ocioConfig->getDisplay(m_cmbDisplayDevice->currentIndex());
-        m_displayFilter->view = m_ocioConfig->getView(m_displayFilter->displayDevice, m_cmbView->currentIndex());
-        m_displayFilter->gamma = m_gammaDoubleWidget->value();
-        m_displayFilter->exposure = m_exposureDoubleWidget->value();
-        m_displayFilter->swizzle = (OCIO_CHANNEL_SWIZZLE)m_cmbComponents->currentIndex();
+        KIS_SAFE_ASSERT_RECOVER_NOOP(!m_canvas->displayFilter() ||
+                                     m_canvas->displayFilter() == m_displayFilter);
 
-        m_displayFilter->blackPoint = m_bwPointChooser->blackPoint();
-        m_displayFilter->whitePoint = m_bwPointChooser->whitePoint();
+        if (!m_displayFilter) {
+            m_displayFilter =
+                m_canvas->displayFilter() ?
+                    m_canvas->displayFilter() :
+                    QSharedPointer<KisDisplayFilter>(new OcioDisplayFilter(this));
+        }
 
-        m_displayFilter->forceInternalColorManagement =
+        OcioDisplayFilter *displayFilter = qobject_cast<OcioDisplayFilter*>(m_displayFilter.data());
+        displayFilter->config = m_ocioConfig;
+        displayFilter->inputColorSpaceName = m_ocioConfig->getColorSpaceNameByIndex(m_cmbInputColorSpace->currentIndex());
+        displayFilter->displayDevice = m_ocioConfig->getDisplay(m_cmbDisplayDevice->currentIndex());
+        displayFilter->view = m_ocioConfig->getView(displayFilter->displayDevice, m_cmbView->currentIndex());
+        displayFilter->look = m_ocioConfig->getLookNameByIndex(m_cmbLook->currentIndex());
+        displayFilter->gamma = m_gammaDoubleWidget->value();
+        displayFilter->exposure = m_exposureDoubleWidget->value();
+        displayFilter->swizzle = (OCIO_CHANNEL_SWIZZLE)m_cmbComponents->currentIndex();
+
+        displayFilter->blackPoint = m_bwPointChooser->blackPoint();
+        displayFilter->whitePoint = m_bwPointChooser->whitePoint();
+
+        displayFilter->forceInternalColorManagement =
             m_colorManagement->currentIndex() == (int)KisConfig::INTERNAL;
 
-        m_displayFilter->setLockCurrentColorVisualRepresentation(m_btnConvertCurrentColor->isChecked());
+        displayFilter->setLockCurrentColorVisualRepresentation(m_btnConvertCurrentColor->isChecked());
 
-        m_displayFilter->updateProcessor();
+        displayFilter->updateProcessor();
         m_canvas->setDisplayFilter(m_displayFilter);
     }
     else {
-        m_canvas->setDisplayFilter(0);
+        m_canvas->setDisplayFilter(QSharedPointer<KisDisplayFilter>(0));
     }
     m_canvas->updateCanvas();
 }
@@ -423,7 +440,7 @@ void LutDockerDock::selectOcioConfiguration()
     KoFileDialog dialog(this, KoFileDialog::OpenFile, "lutdocker");
     dialog.setCaption(i18n("Select OpenColorIO Configuration"));
     dialog.setDefaultDir(QDir::cleanPath(filename));
-    dialog.setMimeTypeFilters(QStringList() << "application/x-opencolorio-configuration", "x-opencolorio-configuration");
+    dialog.setMimeTypeFilters(QStringList() << "application/x-opencolorio-configuration");
     filename = dialog.filename();
     QFile f(filename);
     if (f.exists()) {
@@ -437,8 +454,9 @@ void LutDockerDock::selectOcioConfiguration()
 
 void LutDockerDock::resetOcioConfiguration()
 {
-    m_ocioConfig.reset();
     KisConfig cfg;
+    m_ocioConfig.reset();
+
     try {
         if (cfg.ocioColorManagementMode() == KisConfig::INTERNAL) {
             m_ocioConfig = defaultRawProfile();
@@ -462,7 +480,10 @@ void LutDockerDock::resetOcioConfiguration()
         dbgKrita << "OpenColorIO Error:" << exception.what() << "Cannot create the LUT docker";
     }
 
-    refillControls();
+
+    if (m_ocioConfig) {
+        refillControls();
+    }
 }
 
 void LutDockerDock::refillControls()
@@ -493,34 +514,45 @@ void LutDockerDock::refillControls()
     { // Components
         const KoColorSpace *cs = m_canvas->viewManager()->image()->colorSpace();
 
-        KisSignalsBlocker componentsBlocker(m_cmbComponents);
-        m_cmbComponents->clear();
-        m_cmbComponents->addSqueezedItem(i18n("Luminance"));
-        m_cmbComponents->addSqueezedItem(i18n("All Channels"));
+        QStringList itemsList;
+        itemsList << i18n("Luminance");
+        itemsList << i18n("All Channels");
         Q_FOREACH (KoChannelInfo *channel, KoChannelInfo::displayOrderSorted(cs->channels())) {
-            m_cmbComponents->addSqueezedItem(channel->name());
+            itemsList << channel->name();
+        }
+
+        if (m_cmbComponents->originalTexts() != itemsList) {
+            KisSignalsBlocker componentsBlocker(m_cmbComponents);
+            m_cmbComponents->resetOriginalTexts(itemsList);
+            m_cmbComponents->setCurrentIndex(1); // All Channels...
         }
-        m_cmbComponents->setCurrentIndex(1); // All Channels...
     }
 
     { // Input Color Space
-        KisSignalsBlocker inputCSBlocker(m_cmbInputColorSpace);
-        m_cmbInputColorSpace->clear();
-
+        QStringList itemsList;
         int numOcioColorSpaces = m_ocioConfig->getNumColorSpaces();
         for(int i = 0; i < numOcioColorSpaces; ++i) {
             const char *cs = m_ocioConfig->getColorSpaceNameByIndex(i);
             OCIO::ConstColorSpaceRcPtr colorSpace = m_ocioConfig->getColorSpace(cs);
-            m_cmbInputColorSpace->addSqueezedItem(QString::fromUtf8(colorSpace->getName()));
+            itemsList << QString::fromUtf8(colorSpace->getName());
+        }
+
+        if (itemsList != m_cmbInputColorSpace->originalTexts()) {
+            KisSignalsBlocker inputCSBlocker(m_cmbInputColorSpace);
+            m_cmbInputColorSpace->resetOriginalTexts(itemsList);
         }
     }
 
     { // Display Device
-        KisSignalsBlocker displayDeviceLocker(m_cmbDisplayDevice);
-        m_cmbDisplayDevice->clear();
+        QStringList itemsList;
         int numDisplays = m_ocioConfig->getNumDisplays();
         for (int i = 0; i < numDisplays; ++i) {
-            m_cmbDisplayDevice->addSqueezedItem(QString::fromUtf8(m_ocioConfig->getDisplay(i)));
+            itemsList << QString::fromUtf8(m_ocioConfig->getDisplay(i));
+        }
+
+        if (itemsList != m_cmbDisplayDevice->originalTexts()) {
+            KisSignalsBlocker displayDeviceLocker(m_cmbDisplayDevice);
+            m_cmbDisplayDevice->resetOriginalTexts(itemsList);
         }
     }
 
@@ -531,6 +563,20 @@ void LutDockerDock::refillControls()
     }
 
     refillViewCombobox();
+
+    {
+        QStringList itemsList;
+        int numLooks = m_ocioConfig->getNumLooks();
+        for (int k = 0; k < numLooks; k++) {
+           itemsList << QString::fromUtf8(m_ocioConfig->getLookNameByIndex(k));
+        }
+        itemsList << i18nc("Item to indicate no look transform being selected","None");
+
+        if (itemsList != m_cmbLook->originalTexts()) {
+            KisSignalsBlocker LookComboLocker(m_cmbLook);
+            m_cmbLook->resetOriginalTexts(itemsList);
+        }
+    }
     updateDisplaySettings();
 }
 
