@@ -29,6 +29,7 @@
 #include <QSlider>
 #include <QPoint>
 #include <QColor>
+#include <QButtonGroup>
 
 #include <klocalizedstring.h>
 
@@ -54,7 +55,7 @@
 #include <kis_iterator_ng.h>
 
 KisDesaturateFilter::KisDesaturateFilter()
-        : KisColorTransformationFilter(id(), categoryAdjust(), i18n("&Desaturate"))
+   : KisColorTransformationFilter(id(), categoryAdjust(), i18n("&Desaturate..."))
 {
     setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_U));
     setSupportsPainting(true);
@@ -71,7 +72,7 @@ KisConfigWidget *KisDesaturateFilter::createConfigurationWidget(QWidget *parent,
 }
 
 
-KoColorTransformation* KisDesaturateFilter::createTransformation(const KoColorSpace* cs, const KisFilterConfiguration* config) const
+KoColorTransformation* KisDesaturateFilter::createTransformation(const KoColorSpace* cs, const KisFilterConfigurationSP config) const
 {
     QHash<QString, QVariant> params;
     if (config) {
@@ -80,14 +81,14 @@ KoColorTransformation* KisDesaturateFilter::createTransformation(const KoColorSp
     return  cs->createColorTransformation("desaturate_adjustment", params);
 }
 
-KisFilterConfiguration *KisDesaturateFilter::factoryConfiguration(const KisPaintDeviceSP) const
+KisFilterConfigurationSP KisDesaturateFilter::factoryConfiguration() const
 {
-    KisColorTransformationConfiguration* config = new KisColorTransformationConfiguration(id().id(), 1);
+    KisColorTransformationConfigurationSP config = new KisColorTransformationConfiguration(id().id(), 1);
     config->setProperty("type", 0);
     return config;
 }
 
-KisDesaturateConfigWidget::KisDesaturateConfigWidget(QWidget * parent, Qt::WFlags f) : KisConfigWidget(parent, f)
+KisDesaturateConfigWidget::KisDesaturateConfigWidget(QWidget * parent, Qt::WindowFlags f) : KisConfigWidget(parent, f)
 {
     m_page = new Ui_WdgDesaturate();
     m_page->setupUi(this);
@@ -107,14 +108,14 @@ KisDesaturateConfigWidget::~KisDesaturateConfigWidget()
     delete m_page;
 }
 
-KisPropertiesConfiguration * KisDesaturateConfigWidget::configuration() const
+KisPropertiesConfigurationSP  KisDesaturateConfigWidget::configuration() const
 {
-    KisColorTransformationConfiguration* c = new KisColorTransformationConfiguration(KisDesaturateFilter::id().id(), 0);
+    KisColorTransformationConfigurationSP c = new KisColorTransformationConfiguration(KisDesaturateFilter::id().id(), 0);
     c->setProperty("type", m_group->checkedId());
     return c;
 }
 
-void KisDesaturateConfigWidget::setConfiguration(const KisPropertiesConfiguration * config)
+void KisDesaturateConfigWidget::setConfiguration(const KisPropertiesConfigurationSP  config)
 {
     m_group->button(config->getInt("type", 0))->setChecked(true);
     emit sigConfigurationItemChanged();
