@@ -52,7 +52,7 @@
 #include "csv_layer_record.h"
 
 CSVSaver::CSVSaver(KisDocument *doc, bool batchMode)
-    : m_image(doc->image())
+    : m_image(doc->savingImage())
     , m_doc(doc)
     , m_batchMode(batchMode)
     , m_stop(false)
@@ -63,12 +63,12 @@ CSVSaver::~CSVSaver()
 {
 }
 
-KisImageWSP CSVSaver::image()
+KisImageSP CSVSaver::image()
 {
     return m_image;
 }
 
-KisImageBuilder_Result CSVSaver::encode(const QString &filename)
+KisImageBuilder_Result CSVSaver::encode(QIODevice *io)
 {
     int idx;
     int start, end;
@@ -79,22 +79,25 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
 
     KisImageAnimationInterface *animation = m_image->animationInterface();
 
-    //open the csv file for writing
-    QFile f(filename);
-    if (!f.open(QIODevice::WriteOnly)) {
-        return KisImageBuilder_RESULT_NOT_LOCAL;
-    }
-
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-    //DataStream instead of TextStream for correct line endings
-    QDataStream stream(&f);
+// XXX: Stream was unused?
+//    //DataStream instead of TextStream for correct line endings
+//    QDataStream stream(&f);
 
-    QString path = filename;
+    //Using the original local path
+    QString path = m_doc->localFilePath();
 
     if (path.right(4).toUpper() == ".CSV")
         path = path.left(path.size() - 4);
+    else {
+        // something is wrong: the local file name is not .csv!
+        // trying the given (probably temporary) filename as well
 
+        KIS_SAFE_ASSERT_RECOVER(0 && "Wrong extension of the saved file!") {
+            path = path.left(path.size() - 4);
+        }
+    }
     path.append(".frames");
 
     //create directory
@@ -114,8 +117,8 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
     idx = 0;
 
     while (node) {
-        if (node->inherits("KisPaintLayer")) {
-            KisPaintLayer* paintLayer = dynamic_cast<KisPaintLayer*>(node.data());
+        if (node->inherits("KisLayer")) {
+            KisLayer* paintLayer = qobject_cast<KisLayer*>(node.data());
             CSVLayerRecord* layerRecord = new CSVLayerRecord();
             layers.prepend(layerRecord); //reverse order!
 
@@ -129,7 +132,7 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
             layerRecord->density = (float)(paintLayer->opacity()) / OPACITY_OPAQUE_U8;
             layerRecord->blending = convertToBlending(paintLayer->compositeOpId());
             layerRecord->layer = paintLayer;
-            layerRecord->channel = paintLayer->projection()->keyframeChannel();
+            layerRecord->channel = paintLayer->original()->keyframeChannel();
             layerRecord->last = "";
             layerRecord->frame = 0;
             idx++;
@@ -168,9 +171,10 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
     KisImageBuilder_Result retval= KisImageBuilder_RESULT_OK;
 
     if (!m_batchMode) {
-        emit m_doc->statusBarMessage(i18n("Saving CSV file..."));
-        emit m_doc->sigProgress(0);
-        connect(m_doc, SIGNAL(sigProgressCanceled()), this, SLOT(cancel()));
+        // TODO: use other systems of progress reporting (KisViewManager::createUnthreadedUpdater()
+        //emit m_doc->statusBarMessage(i18n("Saving CSV file..."));
+        //emit m_doc->sigProgress(0);
+        //connect(m_doc, SIGNAL(sigProgressCanceled()), this, SLOT(cancel()));
     }
     int frame = start;
     int step = 0;
@@ -186,92 +190,92 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
         switch(step) {
 
         case 0 :    //first row
-            if (f.write("UTF-8, TVPaint, \"CSV 1.0\"\r\n") < 0) {
+            if (io->write("UTF-8, TVPaint, \"CSV 1.0\"\r\n") < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
             }
             break;
 
         case 1 :    //scene header names
-            if (f.write("Project Name, Width, Height, Frame Count, Layer Count, Frame Rate, Pixel Aspect Ratio, Field Mode\r\n") < 0) {
+            if (io->write("Project Name, Width, Height, Frame Count, Layer Count, Frame Rate, Pixel Aspect Ratio, Field Mode\r\n") < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
             }
             break;
 
         case 2 :    //scene header values
             ba = QString("\"%1\", ").arg(m_image->objectName()).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             ba = QString("%1, %2, ").arg(m_image->width()).arg(m_image->height()).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
 
             ba = QString("%1, %2, ").arg(end - start + 1).arg(layers.size()).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             //the framerate is an integer here
             ba = QString("%1, ").arg((double)(animation->framerate()),0,'f',6).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             ba = QString("%1, Progressive\r\n").arg((double)(m_image->xRes() / m_image->yRes()),0,'f',6).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             break;
 
         case 3 :    //layer header values
-            if (f.write("#Layers") < 0) {          //Layers
+            if (io->write("#Layers") < 0) {          //Layers
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
 
             for (idx = 0; idx < layers.size(); idx++) {
                 ba = QString(", \"%1\"").arg(layers.at(idx)->name).toUtf8();
-                if (f.write(ba.data()) < 0)
+                if (io->write(ba.data()) < 0)
                     break;
             }
             break;
 
         case 4 :
-            if (f.write("\r\n#Density") < 0) {     //Density
+            if (io->write("\r\n#Density") < 0) {     //Density
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             for (idx = 0; idx < layers.size(); idx++) {
                 ba = QString(", %1").arg((double)(layers.at(idx)->density), 0, 'f', 6).toUtf8();
-                if (f.write(ba.data()) < 0)
+                if (io->write(ba.data()) < 0)
                     break;
             }
             break;
 
         case 5 :
-            if (f.write("\r\n#Blending") < 0) {     //Blending
+            if (io->write("\r\n#Blending") < 0) {     //Blending
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             for (idx = 0; idx < layers.size(); idx++) {
                 ba = QString(", \"%1\"").arg(layers.at(idx)->blending).toUtf8();
-                if (f.write(ba.data()) < 0)
+                if (io->write(ba.data()) < 0)
                     break;
             }
             break;
 
         case 6 :
-            if (f.write("\r\n#Visible") < 0) {     //Visible
+            if (io->write("\r\n#Visible") < 0) {     //Visible
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
             for (idx = 0; idx < layers.size(); idx++) {
                 ba = QString(", %1").arg(layers.at(idx)->visible).toUtf8();
-                if (f.write(ba.data()) < 0)
+                if (io->write(ba.data()) < 0)
                     break;
             }
             if (idx < layers.size()) {
@@ -282,7 +286,7 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
         default :    //frames
 
             if (frame > end) {
-                if (f.write("\r\n") < 0)
+                if (io->write("\r\n") < 0)
                     retval = KisImageBuilder_RESULT_FAILURE;
 
                 step = 8;
@@ -290,7 +294,7 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
             }
 
             ba = QString("\r\n#%1").arg(frame, 5, 10, QChar('0')).toUtf8();
-            if (f.write(ba.data()) < 0) {
+            if (io->write(ba.data()) < 0) {
                 retval = KisImageBuilder_RESULT_FAILURE;
                 break;
             }
@@ -312,8 +316,8 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
                 if ( !keyframe.isNull() || (frame == start) ) {
 
                     if (!m_batchMode) {
-                        emit m_doc->sigProgress(((frame - start) * layers.size() + idx) * 100 /
-                                                ((end - start) * layers.size()));
+                        //emit m_doc->sigProgress(((frame - start) * layers.size() + idx) * 100 /
+                        //                        ((end - start) * layers.size()));
                     }
                     retval = getLayer(layer, exportDoc.data(), keyframe, path, frame, idx);
 
@@ -322,7 +326,7 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
                 }
                 ba = QString(", \"%1\"").arg(layer->last).toUtf8();
 
-                if (f.write(ba.data()) < 0)
+                if (io->write(ba.data()) < 0)
                     break;
             }
             if (idx < layers.size())
@@ -336,12 +340,13 @@ KisImageBuilder_Result CSVSaver::encode(const QString &filename)
     } while((retval == KisImageBuilder_RESULT_OK) && (step < 8));
 
     qDeleteAll(layers);
-    f.close();
+
+    // io->close();  it seems this is not required anymore
 
     if (!m_batchMode) {
-        disconnect(m_doc, SIGNAL(sigProgressCanceled()), this, SLOT(cancel()));
-        emit m_doc->sigProgress(100);
-        emit m_doc->clearStatusBarMessage();
+        //disconnect(m_doc, SIGNAL(sigProgressCanceled()), this, SLOT(cancel()));
+        //emit m_doc->sigProgress(100);
+        //emit m_doc->clearStatusBarMessage();
     }
     QApplication::restoreOverrideCursor();
     return retval;
@@ -362,7 +367,7 @@ QString CSVSaver::convertToBlending(const QString &opid)
     if (opid == COMPOSITE_MULT) return "Multiply";
     if (opid == COMPOSITE_SCREEN) return "Screen";
     // "Replace"
-    // "Subtitute"
+    // "Substitute"
     if (opid == COMPOSITE_DIFF) return "Difference";
     if (opid == COMPOSITE_DIVIDE) return "Divide";
     if (opid == COMPOSITE_OVERLAY) return "Overlay";
@@ -384,7 +389,10 @@ QString CSVSaver::convertToBlending(const QString &opid)
 KisImageBuilder_Result CSVSaver::getLayer(CSVLayerRecord* layer, KisDocument* exportDoc, KisKeyframeSP keyframe, const QString &path, int frame, int idx)
 {
     //render to the temp layer
-    KisImageWSP image = exportDoc->image();
+    KisImageSP image = exportDoc->savingImage();
+
+    if (!image) image= exportDoc->image();
+
     KisPaintDeviceSP device = image->rootLayer()->firstChild()->projection();
 
     if (!keyframe.isNull()) {
@@ -404,17 +412,16 @@ KisImageBuilder_Result CSVSaver::getLayer(CSVLayerRecord* layer, KisDocument* ex
     filename.append(layer->last);
 
     //save to PNG
-
     KisSequentialConstIterator it(device, image->bounds());
     const KoColorSpace* cs = device->colorSpace();
 
     bool isThereAlpha = false;
-    do {
+    while (it.nextPixel()) {
         if (cs->opacityU8(it.oldRawData()) != OPACITY_OPAQUE_U8) {
             isThereAlpha = true;
             break;
         }
-    } while (it.nextPixel());
+    }
 
     if (!KisPNGConverter::isColorSpaceSupported(cs)) {
         device = new KisPaintDevice(*device.data());
@@ -443,11 +450,10 @@ KisImageBuilder_Result CSVSaver::getLayer(CSVLayerRecord* layer, KisDocument* ex
 
 void CSVSaver::createTempImage(KisDocument* exportDoc)
 {
-    exportDoc->setAutoSave(0);
-    exportDoc->setOutputMimeType("image/png");
+    exportDoc->setInfiniteAutoSaveInterval();
     exportDoc->setFileBatchMode(true);
 
-    KisImageWSP exportImage = new KisImage(exportDoc->createUndoStore(),
+    KisImageSP exportImage = new KisImage(exportDoc->createUndoStore(),
                                            m_image->width(), m_image->height(), m_image->colorSpace(),
                                            QString());
 
@@ -459,11 +465,12 @@ void CSVSaver::createTempImage(KisDocument* exportDoc)
 }
 
 
-KisImageBuilder_Result CSVSaver::buildAnimation(const QString &filename)
+KisImageBuilder_Result CSVSaver::buildAnimation(QIODevice *io)
 {
-    if (!m_image)
+    if (!m_image) {
         return KisImageBuilder_RESULT_EMPTY;
-    return encode(filename);
+    }
+    return encode(io);
 }
 
 void CSVSaver::cancel()
