@@ -22,10 +22,9 @@
 
 #include <KoColorSpace.h>
 #include <KoColorSpaceConstants.h>
-#include <KisFilterChain.h>
-#include <KisImportExportManager.h>
 
-#include <KoDialog.h>
+#include <KisImportExportManager.h>
+#include <KisExportCheckRegistry.h>
 
 #include <kis_debug.h>
 #include <KisDocument.h>
@@ -34,7 +33,6 @@
 #include <kis_properties_configuration.h>
 #include <kis_config.h>
 
-#include "ui_kis_wdg_options_ppm.h"
 #include <qendian.h>
 #include <KoColorSpaceTraits.h>
 #include <KoColorSpaceRegistry.h>
@@ -74,27 +72,27 @@ class KisPPMAsciiFlow : public KisPPMFlow
 public:
     KisPPMAsciiFlow(QIODevice* device) : m_device(device) {
     }
-    ~KisPPMAsciiFlow() {
+    ~KisPPMAsciiFlow() override {
     }
-    virtual void writeBool(quint8 v) {
+    void writeBool(quint8 v) override {
         if (v > 127) {
             m_device->write("1 ");
         } else {
             m_device->write("0 ");
         }
     }
-    virtual void writeBool(quint16 v) {
+    void writeBool(quint16 v) override {
         writeBool(quint8(v >> 8));
     }
-    virtual void writeNumber(quint8 v) {
+    void writeNumber(quint8 v) override {
         m_device->write(QByteArray::number(v));
         m_device->write(" ");
     }
-    virtual void writeNumber(quint16 v) {
+    void writeNumber(quint16 v) override {
         m_device->write(QByteArray::number(v));
         m_device->write(" ");
     }
-    virtual void flush() {
+    void flush() override {
     }
 private:
     QIODevice* m_device;
@@ -105,9 +103,9 @@ class KisPPMBinaryFlow : public KisPPMFlow
 public:
     KisPPMBinaryFlow(QIODevice* device) : m_device(device), m_pos(0), m_current(0) {
     }
-    virtual ~KisPPMBinaryFlow() {
+    ~KisPPMBinaryFlow() override {
     }
-    virtual void writeBool(quint8 v) {
+    void writeBool(quint8 v) override {
         m_current = m_current << 1;
         m_current |= (v > 127);
         ++m_pos;
@@ -117,17 +115,17 @@ public:
             flush();
         }
     }
-    virtual void writeBool(quint16 v) {
+    void writeBool(quint16 v) override {
         writeBool(quint8(v >> 8));
     }
-    virtual void writeNumber(quint8 v) {
+    void writeNumber(quint8 v) override {
         m_device->write((char*)&v, 1);
     }
-    virtual void writeNumber(quint16 v) {
+    void writeNumber(quint16 v) override {
         quint16 vo = qToBigEndian(v);
         m_device->write((char*)&vo, 2);
     }
-    virtual void flush() {
+    void flush() override {
         m_device->write((char*)&m_current, 1);
     }
 private:
@@ -136,56 +134,14 @@ private:
     quint8 m_current;
 };
 
-KisImportExportFilter::ConversionStatus KisPPMExport::convert(const QByteArray& from, const QByteArray& to)
+KisImportExportFilter::ConversionStatus KisPPMExport::convert(KisDocument *document, QIODevice *io,  KisPropertiesConfigurationSP configuration)
 {
-    dbgFile << "PPM export! From:" << from << ", To:" << to << "";
+    bool rgb = (mimeType() == "image/x-portable-pixmap");
+    bool binary = (configuration->getInt("type") == 0);
 
-    if (from != "application/x-krita")
-        return KisImportExportFilter::NotImplemented;
+    bool bitmap = (mimeType() == "image/x-portable-bitmap");
 
-    KisDocument *input = inputDocument();
-    QString filename = outputFile();
-
-    if (!input)
-        return KisImportExportFilter::NoDocumentCreated;
-
-    if (filename.isEmpty()) return KisImportExportFilter::FileNotFound;
-
-    KoDialog* kdb = new KoDialog(0);
-    kdb->setWindowTitle(i18n("PPM Export Options"));
-    kdb->setButtons(KoDialog::Ok | KoDialog::Cancel);
-
-    Ui::WdgOptionsPPM optionsPPM;
-
-    QWidget* wdg = new QWidget(kdb);
-    optionsPPM.setupUi(wdg);
-
-    kdb->setMainWidget(wdg);
-    QApplication::restoreOverrideCursor();
-
-    QString filterConfig = KisConfig().exportConfiguration("PPM");
-    KisPropertiesConfiguration cfg;
-    cfg.fromXML(filterConfig);
-
-    optionsPPM.type->setCurrentIndex(cfg.getInt("type", 0));
-
-    if (!getBatchMode()) {
-        if (kdb->exec() == QDialog::Rejected) {
-            return KisImportExportFilter::UserCancelled;
-        }
-    }
-
-    bool rgb = (to == "image/x-portable-pixmap");
-    bool binary = optionsPPM.type->currentIndex() == 0;
-    cfg.setProperty("type", optionsPPM.type->currentIndex());
-    KisConfig().setExportConfiguration("PPM", cfg);
-
-    bool bitmap = (to == "image/x-portable-bitmap");
-
-    KisImageWSP image = input->image();
-    Q_CHECK_PTR(image);
-    // the image must be locked at the higher levels
-    KIS_SAFE_ASSERT_RECOVER_NOOP(input->image()->locked());
+    KisImageSP image = document->savingImage();
     KisPaintDeviceSP pd = new KisPaintDevice(*image->projection());
 
     // Test color space
@@ -201,38 +157,34 @@ KisImportExportFilter::ConversionStatus KisPPMExport::convert(const QByteArray&
 
     bool is16bit = pd->colorSpace()->id() == "RGBA16" || pd->colorSpace()->id() == "GRAYAU16";
 
-    // Open the file for writing
-    QFile fp(filename);
-    fp.open(QIODevice::WriteOnly);
-
     // Write the magic
     if (rgb) {
-        if (binary) fp.write("P6");
-        else fp.write("P3");
+        if (binary) io->write("P6");
+        else io->write("P3");
     } else if (bitmap) {
-        if (binary) fp.write("P4");
-        else fp.write("P1");
+        if (binary) io->write("P4");
+        else io->write("P1");
     } else {
-        if (binary) fp.write("P5");
-        else fp.write("P2");
+        if (binary) io->write("P5");
+        else io->write("P2");
     }
-    fp.write("\n");
+    io->write("\n");
 
     // Write the header
-    fp.write(QByteArray::number(image->width()));
-    fp.write(" ");
-    fp.write(QByteArray::number(image->height()));
+    io->write(QByteArray::number(image->width()));
+    io->write(" ");
+    io->write(QByteArray::number(image->height()));
     if (!bitmap) {
-        if (is16bit) fp.write(" 65535\n");
-        else fp.write(" 255\n");
+        if (is16bit) io->write(" 65535\n");
+        else io->write(" 255\n");
     } else {
-        fp.write("\n");
+        io->write("\n");
     }
 
     // Write the data
     KisPPMFlow* flow = 0;
-    if (binary) flow = new KisPPMBinaryFlow(&fp);
-    else flow = new KisPPMAsciiFlow(&fp);
+    if (binary) flow = new KisPPMBinaryFlow(io);
+    else flow = new KisPPMAsciiFlow(io);
 
     for (int y = 0; y < image->height(); ++y) {
         KisHLineIteratorSP it = pd->createHLineIteratorNG(0, y, image->width());
@@ -279,8 +231,53 @@ KisImportExportFilter::ConversionStatus KisPPMExport::convert(const QByteArray&
         flow->flush();
     }
     delete flow;
-    fp.close();
     return KisImportExportFilter::OK;
 }
 
+KisPropertiesConfigurationSP KisPPMExport::defaultConfiguration(const QByteArray &/*from*/, const QByteArray &/*to*/) const
+{
+    KisPropertiesConfigurationSP cfg = new KisPropertiesConfiguration();
+    cfg->setProperty("type", 0);
+    return cfg;
+}
+
+KisConfigWidget *KisPPMExport::createConfigurationWidget(QWidget *parent, const QByteArray &/*from*/, const QByteArray &/*to*/) const
+{
+    return new KisWdgOptionsPPM(parent);
+}
+
+
+void KisWdgOptionsPPM::setConfiguration(const KisPropertiesConfigurationSP cfg)
+{
+    cmbType->setCurrentIndex(cfg->getInt("type", 0));
+}
+
+KisPropertiesConfigurationSP KisWdgOptionsPPM::configuration() const
+{
+    KisPropertiesConfigurationSP cfg = new KisPropertiesConfiguration();
+    cfg->setProperty("type", cmbType->currentIndex());
+    return cfg;
+
+}
+
+void KisPPMExport::initializeCapabilities()
+{
+    addCapability(KisExportCheckRegistry::instance()->get("ColorModelCheck/" + RGBAColorModelID.id() + "/" + Integer8BitsColorDepthID.id())->create(KisExportCheckBase::SUPPORTED));
+    addCapability(KisExportCheckRegistry::instance()->get("ColorModelCheck/" + RGBAColorModelID.id() + "/" + Integer16BitsColorDepthID.id())->create(KisExportCheckBase::SUPPORTED));
+    addCapability(KisExportCheckRegistry::instance()->get("ColorModelCheck/" + GrayAColorModelID.id() + "/" + Integer8BitsColorDepthID.id())->create(KisExportCheckBase::SUPPORTED));
+    addCapability(KisExportCheckRegistry::instance()->get("ColorModelCheck/" + GrayAColorModelID.id() + "/" + Integer16BitsColorDepthID.id())->create(KisExportCheckBase::SUPPORTED));
+
+    QList<QPair<KoID, KoID> > supportedColorModels;
+    supportedColorModels << QPair<KoID, KoID>()
+            << QPair<KoID, KoID>(RGBAColorModelID, Integer8BitsColorDepthID)
+            << QPair<KoID, KoID>(RGBAColorModelID, Integer16BitsColorDepthID)
+            << QPair<KoID, KoID>(GrayAColorModelID, Integer8BitsColorDepthID)
+            << QPair<KoID, KoID>(GrayAColorModelID, Integer16BitsColorDepthID);
+    addSupportedColorModels(supportedColorModels, "PPM");
+
+}
+
+
+
 #include "kis_ppm_export.moc"
+
